@@ -38,7 +38,6 @@ using Mono.Cecil.Cil;
 namespace ICSharpCode.Decompiler.Ast
 {
 	using Ast = ICSharpCode.NRefactory.CSharp;
-	using ClassType = ICSharpCode.NRefactory.CSharp.ClassType;
 	using VarianceModifier = ICSharpCode.NRefactory.TypeSystem.VarianceModifier;
 	
 	[Flags]
@@ -50,7 +49,7 @@ namespace ICSharpCode.Decompiler.Ast
 		DoNotUsePrimitiveTypeNames = 4
 	}
 	
-	public class AstBuilder : BaseCodeMappings
+	public class AstBuilder
 	{
 		DecompilerContext context;
 		CompilationUnit astCompileUnit = new CompilationUnit();
@@ -63,43 +62,64 @@ namespace ICSharpCode.Decompiler.Ast
 				throw new ArgumentNullException("context");
 			this.context = context;
 			this.DecompileMethodBodies = true;
-			
-			this.LocalVariables = new ConcurrentDictionary<int, IEnumerable<ILVariable>>();
-			this.CodeMappings = new Dictionary<int, List<MemberMapping>>();
-			this.DecompiledMemberReferences = new Dictionary<int, MemberReference>();
 		}
 		
-		public static bool MemberIsHidden(MemberReference member, DecompilerSettings settings)
+		public static bool MemberIsHidden (MemberReference member, DecompilerSettings settings)
 		{
 			MethodDefinition method = member as MethodDefinition;
 			if (method != null) {
 				if (method.IsGetter || method.IsSetter || method.IsAddOn || method.IsRemoveOn)
 					return true;
-				if (settings.AnonymousMethods && method.Name.StartsWith("<", StringComparison.Ordinal) && method.IsCompilerGenerated())
+				if (settings.HideNonPublicMembers && !method.IsPublic)
+					return true;
+				if (settings.AnonymousMethods && method.Name.StartsWith ("<", StringComparison.Ordinal) && method.IsCompilerGenerated ())
 					return true;
 			}
+
 			TypeDefinition type = member as TypeDefinition;
-			if (type != null && type.DeclaringType != null) {
-				if (settings.AnonymousMethods && type.Name.StartsWith("<>c__DisplayClass", StringComparison.Ordinal) && type.IsCompilerGenerated())
-					return true;
-				if (settings.YieldReturn && YieldReturnDecompiler.IsCompilerGeneratorEnumerator(type))
-					return true;
-			} else if (type != null && type.IsCompilerGenerated()) {
-				if (type.Name.StartsWith("<PrivateImplementationDetails>", StringComparison.Ordinal))
-					return true;
-				if (type.IsAnonymousType())
+			if (type != null) {
+				if (settings.HideNonPublicMembers && !type.IsPublic)
 					return true;
+				if (type.DeclaringType != null) {
+					if (settings.AnonymousMethods && type.Name.StartsWith ("<>c__DisplayClass", StringComparison.Ordinal) && type.IsCompilerGenerated ())
+						return true;
+					if (settings.YieldReturn && YieldReturnDecompiler.IsCompilerGeneratorEnumerator (type))
+						return true;
+				} else if (type.IsCompilerGenerated ()) {
+					if (type.Name.StartsWith ("<PrivateImplementationDetails>", StringComparison.Ordinal))
+						return true;
+					if (type.IsAnonymousType ())
+						return true;
+				}
 			}
+			
 			FieldDefinition field = member as FieldDefinition;
-			if (field != null && field.IsCompilerGenerated()) {
-				if (settings.AnonymousMethods && field.Name.StartsWith("CS$<>", StringComparison.Ordinal))
+			if (field != null) {
+				if (settings.HideNonPublicMembers && !field.IsPublic)
 					return true;
-				if (settings.AutomaticProperties && field.Name.StartsWith("<", StringComparison.Ordinal) && field.Name.EndsWith("BackingField", StringComparison.Ordinal))
+				if (field.IsCompilerGenerated ()) {
+					if (settings.AnonymousMethods && field.Name.StartsWith ("CS$<>", StringComparison.Ordinal))
+						return true;
+					if (settings.AutomaticProperties && field.Name.StartsWith ("<", StringComparison.Ordinal) && field.Name.EndsWith ("BackingField", StringComparison.Ordinal))
+						return true;
+				}
+				// event-fields are not [CompilerGenerated]
+				if (settings.AutomaticEvents && field.DeclaringType.Events.Any (ev => ev.Name == field.Name))
 					return true;
 			}
-			// event-fields are not [CompilerGenerated]
-			if (field != null && settings.AutomaticEvents && field.DeclaringType.Events.Any(ev => ev.Name == field.Name))
-				return true;
+
+			PropertyDefinition property = member as PropertyDefinition;
+			if (property != null) {
+				if (settings.HideNonPublicMembers && (property.GetMethod == null || !property.GetMethod.IsPublic) && (property.SetMethod == null || !property.SetMethod.IsPublic))
+					return true;
+			}
+
+			EventDefinition evt = member as EventDefinition;
+			if (evt != null) {
+				if (settings.HideNonPublicMembers && (evt.AddMethod == null || !evt.AddMethod.IsPublic) && (evt.RemoveMethod == null || !evt.RemoveMethod.IsPublic))
+					return true;
+			}
+
 			return false;
 		}
 		
@@ -128,30 +148,23 @@ namespace ICSharpCode.Decompiler.Ast
 		/// Generates C# code from the abstract source tree.
 		/// </summary>
 		/// <remarks>This method adds ParenthesizedExpressions into the AST, and will run transformations if <see cref="RunTransformations"/> was not called explicitly</remarks>
-		public void GenerateCode(ITextOutput output)
+		public void GenerateCode(ITextOutput output, CSharpFormattingOptions formattingPolicy = null)
 		{
 			if (!transformationsHaveRun)
 				RunTransformations();
 			
-			astCompileUnit.AcceptVisitor(new InsertParenthesesVisitor { InsertParenthesesForReadability = true }, null);
+			astCompileUnit.AcceptVisitor(new InsertParenthesesVisitor { InsertParenthesesForReadability = true });
 			var outputFormatter = new TextOutputFormatter(output);
-			var formattingPolicy = new CSharpFormattingOptions();
-			// disable whitespace in front of parentheses:
-			formattingPolicy.SpaceBeforeMethodCallParentheses = false;
-			formattingPolicy.SpaceBeforeMethodDeclarationParentheses = false;
-			formattingPolicy.SpaceBeforeConstructorDeclarationParentheses = false;
-			formattingPolicy.SpaceBeforeDelegateDeclarationParentheses = false;
-			astCompileUnit.AcceptVisitor(new OutputVisitor(outputFormatter, formattingPolicy), null);
-		}
-		
-		public void GenerateCode(ITextOutput output, CSharpFormattingOptions formattingPolicy)
-		{
-			if (!transformationsHaveRun)
-				RunTransformations();
-			
-			astCompileUnit.AcceptVisitor(new InsertParenthesesVisitor { InsertParenthesesForReadability = true }, null);
-			var outputFormatter = new TextOutputFormatter(output);
-			astCompileUnit.AcceptVisitor(new OutputVisitor(outputFormatter, formattingPolicy), null);
+			if (formattingPolicy == null)  {
+					
+				formattingPolicy = FormattingOptionsFactory.CreateMono ();
+				// disable whitespace in front of parentheses:
+				formattingPolicy.SpaceBeforeMethodCallParentheses = false;
+				formattingPolicy.SpaceBeforeMethodDeclarationParentheses = false;
+				formattingPolicy.SpaceBeforeConstructorDeclarationParentheses = false;
+				formattingPolicy.SpaceBeforeDelegateDeclarationParentheses = false;
+			}
+			astCompileUnit.AcceptVisitor(new CSharpOutputVisitor(outputFormatter, formattingPolicy));
 		}
 		
 		public void AddAssembly(AssemblyDefinition assemblyDefinition, bool onlyAssemblyLevel = false)
@@ -171,7 +184,7 @@ namespace ICSharpCode.Decompiler.Ast
 								}
 							}
 						}
-					}, AttributedNode.AttributeRole);
+					}, EntityDeclaration.AttributeRole);
 			}
 			
 			ConvertCustomAttributes(astCompileUnit, assemblyDefinition, "assembly");
@@ -211,7 +224,7 @@ namespace ICSharpCode.Decompiler.Ast
 									Arguments = { forwardedType }
 								}
 							}
-						}, AttributedNode.AttributeRole);
+						}, EntityDeclaration.AttributeRole);
 				}
 			}
 		}
@@ -269,16 +282,16 @@ namespace ICSharpCode.Decompiler.Ast
 		/// </summary>
 		/// <param name="typeDef"></param>
 		/// <returns>TypeDeclaration or DelegateDeclaration.</returns>
-		public AttributedNode CreateType(TypeDefinition typeDef)
+		public EntityDeclaration CreateType (TypeDefinition typeDef)
 		{
 			// create type
 			TypeDefinition oldCurrentType = context.CurrentType;
 			context.CurrentType = typeDef;
-			TypeDeclaration astType = new TypeDeclaration();
-			ConvertAttributes(astType, typeDef);
-			astType.AddAnnotation(typeDef);
-			astType.Modifiers = ConvertModifiers(typeDef);
-			astType.Name = CleanName(typeDef.Name);
+			TypeDeclaration astType = new TypeDeclaration ();
+			ConvertAttributes (astType, typeDef);
+			astType.AddAnnotation (typeDef);
+			astType.Modifiers = ConvertModifiers (typeDef);
+			astType.Name = CleanName (typeDef.Name);
 			
 			if (typeDef.IsEnum) {  // NB: Enum is value type
 				astType.ClassType = ClassType.Enum;
@@ -295,55 +308,55 @@ namespace ICSharpCode.Decompiler.Ast
 			
 			IEnumerable<GenericParameter> genericParameters = typeDef.GenericParameters;
 			if (typeDef.DeclaringType != null && typeDef.DeclaringType.HasGenericParameters)
-				genericParameters = genericParameters.Skip(typeDef.DeclaringType.GenericParameters.Count);
-			astType.TypeParameters.AddRange(MakeTypeParameters(genericParameters));
-			astType.Constraints.AddRange(MakeConstraints(genericParameters));
+				genericParameters = genericParameters.Skip (typeDef.DeclaringType.GenericParameters.Count);
+			astType.TypeParameters.AddRange (MakeTypeParameters (genericParameters));
+			astType.Constraints.AddRange (MakeConstraints (genericParameters));
 			
-			AttributedNode result = astType;
+			EntityDeclaration result = astType;
 			if (typeDef.IsEnum) {
 				long expectedEnumMemberValue = 0;
-				bool forcePrintingInitializers = IsFlagsEnum(typeDef);
+				bool forcePrintingInitializers = IsFlagsEnum (typeDef);
 				foreach (FieldDefinition field in typeDef.Fields) {
 					if (field.IsRuntimeSpecialName) {
 						// the value__ field
 						if (field.FieldType != typeDef.Module.TypeSystem.Int32) {
-							astType.AddChild(ConvertType(field.FieldType), TypeDeclaration.BaseTypeRole);
+							astType.AddChild (ConvertType (field.FieldType), Roles.BaseType);
 						}
 					} else {
-						EnumMemberDeclaration enumMember = new EnumMemberDeclaration();
-						enumMember.AddAnnotation(field);
-						enumMember.Name = CleanName(field.Name);
-						long memberValue = (long)CSharpPrimitiveCast.Cast(TypeCode.Int64, field.Constant, false);
+						var enumMember = new EnumMemberDeclaration ();
+						enumMember.AddAnnotation (field);
+						enumMember.Name = CleanName (field.Name);
+						long memberValue = (long)CSharpPrimitiveCast.Cast (TypeCode.Int64, field.Constant, false);
 						if (forcePrintingInitializers || memberValue != expectedEnumMemberValue) {
-							enumMember.AddChild(new PrimitiveExpression(field.Constant), EnumMemberDeclaration.InitializerRole);
+							enumMember.AddChild (new PrimitiveExpression (field.Constant), EnumMemberDeclaration.InitializerRole);
 						}
 						expectedEnumMemberValue = memberValue + 1;
-						astType.AddChild(enumMember, TypeDeclaration.MemberRole);
+						astType.AddChild (enumMember, Roles.TypeMemberRole);
 					}
 				}
 			} else if (typeDef.BaseType != null && typeDef.BaseType.FullName == "System.MulticastDelegate") {
-				DelegateDeclaration dd = new DelegateDeclaration();
+				DelegateDeclaration dd = new DelegateDeclaration ();
 				dd.Modifiers = astType.Modifiers & ~Modifiers.Sealed;
 				dd.Name = astType.Name;
-				dd.AddAnnotation(typeDef);
-				astType.Attributes.MoveTo(dd.Attributes);
-				astType.TypeParameters.MoveTo(dd.TypeParameters);
-				astType.Constraints.MoveTo(dd.Constraints);
+				dd.AddAnnotation (typeDef);
+				astType.Attributes.MoveTo (dd.Attributes);
+				astType.TypeParameters.MoveTo (dd.TypeParameters);
+				astType.Constraints.MoveTo (dd.Constraints);
 				foreach (var m in typeDef.Methods) {
 					if (m.Name == "Invoke") {
-						dd.ReturnType = ConvertType(m.ReturnType, m.MethodReturnType);
-						dd.Parameters.AddRange(MakeParameters(m));
-						ConvertAttributes(dd, m.MethodReturnType, m.Module);
+						dd.ReturnType = ConvertType (m.ReturnType, m.MethodReturnType);
+						dd.Parameters.AddRange (MakeParameters (m));
+						ConvertAttributes (dd, m.MethodReturnType, m.Module);
 					}
 				}
 				result = dd;
 			} else {
 				// Base type
 				if (typeDef.BaseType != null && !typeDef.IsValueType && typeDef.BaseType.FullName != "System.Object") {
-					astType.AddChild(ConvertType(typeDef.BaseType), TypeDeclaration.BaseTypeRole);
+					astType.AddChild (ConvertType (typeDef.BaseType), Roles.BaseType);
 				}
 				foreach (var i in typeDef.Interfaces)
-					astType.AddChild(ConvertType(i), TypeDeclaration.BaseTypeRole);
+					astType.AddChild(ConvertType(i), Roles.BaseType);
 				
 				AddTypeMembers(astType, typeDef);
 
@@ -656,6 +669,10 @@ namespace ICSharpCode.Decompiler.Ast
 					modifiers |= Modifiers.Readonly;
 			}
 			
+			RequiredModifierType modreq = fieldDef.FieldType as RequiredModifierType;
+			if (modreq != null && modreq.ModifierType.FullName == typeof(IsVolatile).FullName)
+				modifiers |= Modifiers.Volatile;
+			
 			return modifiers;
 		}
 		
@@ -700,26 +717,26 @@ namespace ICSharpCode.Decompiler.Ast
 
 		#endregion
 		
-		void AddTypeMembers(TypeDeclaration astType, TypeDefinition typeDef)
+		void AddTypeMembers (TypeDeclaration astType, TypeDefinition typeDef)
 		{
 			// Nested types
 			foreach (TypeDefinition nestedTypeDef in typeDef.NestedTypes) {
-				if (MemberIsHidden(nestedTypeDef, context.Settings))
+				if (MemberIsHidden (nestedTypeDef, context.Settings))
 					continue;
-				var nestedType = CreateType(nestedTypeDef);
-				SetNewModifier(nestedType);
-				astType.AddChild(nestedType, TypeDeclaration.MemberRole);
+				var nestedType = CreateType (nestedTypeDef);
+				SetNewModifier (nestedType);
+				astType.AddChild (nestedType, Roles.TypeMemberRole);
 			}
 			
 			// Add fields
 			foreach(FieldDefinition fieldDef in typeDef.Fields) {
 				if (MemberIsHidden(fieldDef, context.Settings)) continue;
-				astType.AddChild(CreateField(fieldDef), TypeDeclaration.MemberRole);
+				astType.AddChild(CreateField(fieldDef), Roles.TypeMemberRole);
 			}
 			
 			// Add events
 			foreach(EventDefinition eventDef in typeDef.Events) {
-				astType.AddChild(CreateEvent(eventDef), TypeDeclaration.MemberRole);
+				astType.AddChild(CreateEvent(eventDef), Roles.TypeMemberRole);
 			}
 
 			// Add properties
@@ -738,13 +755,9 @@ namespace ICSharpCode.Decompiler.Ast
 			}
 		}
 
-		AttributedNode CreateMethod(MethodDefinition methodDef)
+		EntityDeclaration CreateMethod(MethodDefinition methodDef)
 		{
-			// Create mapping - used in debugger
-			CreateCodeMappings(methodDef.MetadataToken.ToInt32(), methodDef);
-			MemberMapping methodMapping = methodDef.CreateCodeMapping(this.CodeMappings[methodDef.MetadataToken.ToInt32()]);
-			
-			MethodDeclaration astMethod = new MethodDeclaration().WithAnnotation(methodMapping);
+			MethodDeclaration astMethod = new MethodDeclaration();
 			astMethod.AddAnnotation(methodDef);
 			astMethod.ReturnType = ConvertType(methodDef.ReturnType, methodDef.MethodReturnType);
 			astMethod.Name = CleanName(methodDef.Name);
@@ -753,12 +766,12 @@ namespace ICSharpCode.Decompiler.Ast
 			// constraints for override and explicit interface implementation methods are inherited from the base method, so they cannot be specified directly
 			if (!methodDef.IsVirtual || (methodDef.IsNewSlot && !methodDef.IsPrivate)) astMethod.Constraints.AddRange(MakeConstraints(methodDef.GenericParameters));
 			if (!methodDef.DeclaringType.IsInterface) {
-				if (!methodDef.HasOverrides) {
+				if (IsExplicitInterfaceImplementation(methodDef)) {
+					astMethod.PrivateImplementationType = ConvertType(methodDef.Overrides.First().DeclaringType);
+				} else {
 					astMethod.Modifiers = ConvertModifiers(methodDef);
 					if (methodDef.IsVirtual == methodDef.IsNewSlot)
 						SetNewModifier(astMethod);
-				} else {
-					astMethod.PrivateImplementationType = ConvertType(methodDef.Overrides.First().DeclaringType);
 				}
 				astMethod.Body = CreateMethodBody(methodDef, astMethod.Parameters);
 			}
@@ -788,6 +801,11 @@ namespace ICSharpCode.Decompiler.Ast
 			}
 			return astMethod;
 		}
+		
+		bool IsExplicitInterfaceImplementation(MethodDefinition methodDef)
+		{
+			return methodDef.HasOverrides && methodDef.IsPrivate;
+		}
 
 		IEnumerable<TypeParameterDeclaration> MakeTypeParameters(IEnumerable<GenericParameter> genericParameters)
 		{
@@ -807,7 +825,7 @@ namespace ICSharpCode.Decompiler.Ast
 		{
 			foreach (var gp in genericParameters) {
 				Constraint c = new Constraint();
-				c.TypeParameter = CleanName(gp.Name);
+				c.TypeParameter = new SimpleType(CleanName(gp.Name));
 				// class/struct must be first
 				if (gp.HasReferenceTypeConstraint)
 					c.BaseTypes.Add(new PrimitiveType("class"));
@@ -829,10 +847,6 @@ namespace ICSharpCode.Decompiler.Ast
 		
 		ConstructorDeclaration CreateConstructor(MethodDefinition methodDef)
 		{
-			// Create mapping - used in debugger
-			CreateCodeMappings(methodDef.MetadataToken.ToInt32(), methodDef);
-			MemberMapping methodMapping = methodDef.CreateCodeMapping(this.CodeMappings[methodDef.MetadataToken.ToInt32()]);
-			
 			ConstructorDeclaration astMethod = new ConstructorDeclaration();
 			astMethod.AddAnnotation(methodDef);
 			astMethod.Modifiers = ConvertModifiers(methodDef);
@@ -844,9 +858,8 @@ namespace ICSharpCode.Decompiler.Ast
 			astMethod.Parameters.AddRange(MakeParameters(methodDef));
 			astMethod.Body = CreateMethodBody(methodDef, astMethod.Parameters);
 			ConvertAttributes(astMethod, methodDef);
-			astMethod.WithAnnotation(methodMapping);
 			if (methodDef.IsStatic && methodDef.DeclaringType.IsBeforeFieldInit && !astMethod.Body.IsNull) {
-				astMethod.Body.InsertChildAfter(null, new Comment(" Note: this type is marked as 'beforefieldinit'."), AstNode.Roles.Comment);
+				astMethod.Body.InsertChildAfter(null, new Comment(" Note: this type is marked as 'beforefieldinit'."), Roles.Comment);
 			}
 			return astMethod;
 		}
@@ -864,14 +877,14 @@ namespace ICSharpCode.Decompiler.Ast
 			return m & ~Modifiers.Private;
 		}
 
-		MemberDeclaration CreateProperty(PropertyDefinition propDef)
+		EntityDeclaration CreateProperty(PropertyDefinition propDef)
 		{
 			PropertyDeclaration astProp = new PropertyDeclaration();
 			astProp.AddAnnotation(propDef);
 			var accessor = propDef.GetMethod ?? propDef.SetMethod;
 			Modifiers getterModifiers = Modifiers.None;
 			Modifiers setterModifiers = Modifiers.None;
-			if (accessor.HasOverrides) {
+			if (IsExplicitInterfaceImplementation(accessor)) {
 				astProp.PrivateImplementationType = ConvertType(accessor.Overrides.First().DeclaringType);
 			} else if (!propDef.DeclaringType.IsInterface) {
 				getterModifiers = ConvertModifiers(propDef.GetMethod);
@@ -897,10 +910,6 @@ namespace ICSharpCode.Decompiler.Ast
 			astProp.ReturnType = ConvertType(propDef.PropertyType, propDef);
 			
 			if (propDef.GetMethod != null) {
-				// Create mapping - used in debugger
-				CreateCodeMappings(propDef.GetMethod.MetadataToken.ToInt32(), propDef);
-				MemberMapping methodMapping = propDef.GetMethod.CreateCodeMapping(this.CodeMappings[propDef.GetMethod.MetadataToken.ToInt32()], propDef);
-				
 				astProp.Getter = new Accessor();
 				astProp.Getter.Body = CreateMethodBody(propDef.GetMethod);
 				astProp.Getter.AddAnnotation(propDef.GetMethod);
@@ -908,14 +917,8 @@ namespace ICSharpCode.Decompiler.Ast
 				
 				if ((getterModifiers & Modifiers.VisibilityMask) != (astProp.Modifiers & Modifiers.VisibilityMask))
 					astProp.Getter.Modifiers = getterModifiers & Modifiers.VisibilityMask;
-				
-				astProp.Getter.WithAnnotation(methodMapping);
 			}
 			if (propDef.SetMethod != null) {
-				// Create mapping - used in debugger
-				CreateCodeMappings(propDef.SetMethod.MetadataToken.ToInt32(), propDef);
-				MemberMapping methodMapping = propDef.SetMethod.CreateCodeMapping(this.CodeMappings[propDef.SetMethod.MetadataToken.ToInt32()], propDef);
-				
 				astProp.Setter = new Accessor();
 				astProp.Setter.Body = CreateMethodBody(propDef.SetMethod);
 				astProp.Setter.AddAnnotation(propDef.SetMethod);
@@ -930,12 +933,10 @@ namespace ICSharpCode.Decompiler.Ast
 				
 				if ((setterModifiers & Modifiers.VisibilityMask) != (astProp.Modifiers & Modifiers.VisibilityMask))
 					astProp.Setter.Modifiers = setterModifiers & Modifiers.VisibilityMask;
-				
-				astProp.Setter.WithAnnotation(methodMapping);
 			}
 			ConvertCustomAttributes(astProp, propDef);
 
-			MemberDeclaration member = astProp;
+			EntityDeclaration member = astProp;
 			if(propDef.IsIndexer())
 				member = ConvertPropertyToIndexer(astProp, propDef);
 			if(!accessor.HasOverrides && !accessor.DeclaringType.IsInterface)
@@ -959,7 +960,7 @@ namespace ICSharpCode.Decompiler.Ast
 			return astIndexer;
 		}
 		
-		AttributedNode CreateEvent(EventDefinition eventDef)
+		EntityDeclaration CreateEvent(EventDefinition eventDef)
 		{
 			if (eventDef.AddMethod != null && eventDef.AddMethod.IsAbstract) {
 				// An abstract event cannot be custom
@@ -977,34 +978,22 @@ namespace ICSharpCode.Decompiler.Ast
 				astEvent.AddAnnotation(eventDef);
 				astEvent.Name = CleanName(eventDef.Name);
 				astEvent.ReturnType = ConvertType(eventDef.EventType, eventDef);
-				if (eventDef.AddMethod == null || !eventDef.AddMethod.HasOverrides)
+				if (eventDef.AddMethod == null || !IsExplicitInterfaceImplementation(eventDef.AddMethod))
 					astEvent.Modifiers = ConvertModifiers(eventDef.AddMethod);
 				else
 					astEvent.PrivateImplementationType = ConvertType(eventDef.AddMethod.Overrides.First().DeclaringType);
 				
 				if (eventDef.AddMethod != null) {
-					// Create mapping - used in debugger
-					CreateCodeMappings(eventDef.AddMethod.MetadataToken.ToInt32(), eventDef);
-					MemberMapping methodMapping = eventDef.AddMethod.CreateCodeMapping(this.CodeMappings[eventDef.AddMethod.MetadataToken.ToInt32()], eventDef);
-					
 					astEvent.AddAccessor = new Accessor {
 						Body = CreateMethodBody(eventDef.AddMethod)
 					}.WithAnnotation(eventDef.AddMethod);
 					ConvertAttributes(astEvent.AddAccessor, eventDef.AddMethod);
-					
-					astEvent.AddAccessor.WithAnnotation(methodMapping);
 				}
 				if (eventDef.RemoveMethod != null) {
-					// Create mapping - used in debugger
-					CreateCodeMappings(eventDef.RemoveMethod.MetadataToken.ToInt32(), eventDef);
-					MemberMapping methodMapping = eventDef.RemoveMethod.CreateCodeMapping(this.CodeMappings[eventDef.RemoveMethod.MetadataToken.ToInt32()], eventDef);
-					
 					astEvent.RemoveAccessor = new Accessor {
 						Body = CreateMethodBody(eventDef.RemoveMethod)
 					}.WithAnnotation(eventDef.RemoveMethod);
 					ConvertAttributes(astEvent.RemoveAccessor, eventDef.RemoveMethod);
-					
-					astEvent.RemoveAccessor.WithAnnotation(methodMapping);
 				}
 				MethodDefinition accessor = eventDef.AddMethod ?? eventDef.RemoveMethod;
 				if (accessor.IsVirtual == accessor.IsNewSlot) {
@@ -1019,19 +1008,17 @@ namespace ICSharpCode.Decompiler.Ast
 		BlockStatement CreateMethodBody(MethodDefinition method, IEnumerable<ParameterDeclaration> parameters = null)
 		{
 			if (DecompileMethodBodies)
-				return AstMethodBodyBuilder.CreateMethodBody(method, context, parameters, LocalVariables);
+				return AstMethodBodyBuilder.CreateMethodBody(method, context, parameters);
 			else
 				return null;
 		}
 
 		FieldDeclaration CreateField(FieldDefinition fieldDef)
 		{
-			this.DecompiledMemberReferences.Add(fieldDef.MetadataToken.ToInt32(), fieldDef);
-			
 			FieldDeclaration astField = new FieldDeclaration();
 			astField.AddAnnotation(fieldDef);
 			VariableInitializer initializer = new VariableInitializer(CleanName(fieldDef.Name));
-			astField.AddChild(initializer, FieldDeclaration.Roles.Variable);
+			astField.AddChild(initializer, Roles.Variable);
 			astField.ReturnType = ConvertType(fieldDef.FieldType, fieldDef);
 			astField.Modifiers = ConvertModifiers(fieldDef);
 			if (fieldDef.HasConstant) {
@@ -1042,7 +1029,7 @@ namespace ICSharpCode.Decompiler.Ast
 			return astField;
 		}
 		
-		static Expression CreateExpressionForConstant(object constant, TypeReference type, bool isEnumMemberDeclaration = false)
+		static Expression CreateExpressionForConstant(object constant, TypeReference type, bool isEnumEntityDeclaration = false)
 		{
 			if (constant == null) {
 				if (type.IsValueType && !(type.Namespace == "System" && type.Name == "Nullable`1"))
@@ -1051,7 +1038,7 @@ namespace ICSharpCode.Decompiler.Ast
 					return new NullReferenceExpression();
 			} else {
 				TypeCode c = Type.GetTypeCode(constant.GetType());
-				if (c >= TypeCode.SByte && c <= TypeCode.UInt64 && !isEnumMemberDeclaration) {
+				if (c >= TypeCode.SByte && c <= TypeCode.UInt64 && !isEnumEntityDeclaration) {
 					return MakePrimitive((long)CSharpPrimitiveCast.Cast(TypeCode.Int64, constant, false), type);
 				} else {
 					return new PrimitiveExpression(constant);
@@ -1111,7 +1098,7 @@ namespace ICSharpCode.Decompiler.Ast
 		}
 		
 		#region ConvertAttributes
-		void ConvertAttributes(AttributedNode attributedNode, TypeDefinition typeDefinition)
+		void ConvertAttributes(EntityDeclaration attributedNode, TypeDefinition typeDefinition)
 		{
 			ConvertCustomAttributes(attributedNode, typeDefinition);
 			ConvertSecurityAttributes(attributedNode, typeDefinition);
@@ -1167,7 +1154,7 @@ namespace ICSharpCode.Decompiler.Ast
 			#endregion
 		}
 		
-		void ConvertAttributes(AttributedNode attributedNode, MethodDefinition methodDefinition)
+		void ConvertAttributes(EntityDeclaration attributedNode, MethodDefinition methodDefinition)
 		{
 			ConvertCustomAttributes(attributedNode, methodDefinition);
 			ConvertSecurityAttributes(attributedNode, methodDefinition);
@@ -1267,7 +1254,7 @@ namespace ICSharpCode.Decompiler.Ast
 			ConvertAttributes(attributedNode, methodDefinition.MethodReturnType, methodDefinition.Module);
 		}
 		
-		void ConvertAttributes(AttributedNode attributedNode, MethodReturnType methodReturnType, ModuleDefinition module)
+		void ConvertAttributes(EntityDeclaration attributedNode, MethodReturnType methodReturnType, ModuleDefinition module)
 		{
 			ConvertCustomAttributes(attributedNode, methodReturnType, "return");
 			if (methodReturnType.HasMarshalInfo) {
@@ -1276,7 +1263,7 @@ namespace ICSharpCode.Decompiler.Ast
 			}
 		}
 		
-		internal static void ConvertAttributes(AttributedNode attributedNode, FieldDefinition fieldDefinition, string attributeTarget = null)
+		internal static void ConvertAttributes(EntityDeclaration attributedNode, FieldDefinition fieldDefinition, string attributeTarget = null)
 		{
 			ConvertCustomAttributes(attributedNode, fieldDefinition);
 			
@@ -1362,7 +1349,7 @@ namespace ICSharpCode.Decompiler.Ast
 		{
 			if (customAttributeProvider.HasCustomAttributes) {
 				var attributes = new List<ICSharpCode.NRefactory.CSharp.Attribute>();
-				foreach (var customAttribute in customAttributeProvider.CustomAttributes) {
+				foreach (var customAttribute in customAttributeProvider.CustomAttributes.OrderBy(a => a.AttributeType.FullName)) {
 					if (customAttribute.AttributeType.Name == "ExtensionAttribute" && customAttribute.AttributeType.Namespace == "System.Runtime.CompilerServices") {
 						// don't show the ExtensionAttribute (it's converted to the 'this' modifier)
 						continue;
@@ -1415,14 +1402,14 @@ namespace ICSharpCode.Decompiler.Ast
 						var section = new AttributeSection();
 						section.AttributeTarget = attributeTarget;
 						section.Attributes.Add(attribute);
-						attributedNode.AddChild(section, AttributedNode.AttributeRole);
+						attributedNode.AddChild(section, EntityDeclaration.AttributeRole);
 					}
 				} else if (attributes.Count > 0) {
 					// use single section for all attributes
 					var section = new AttributeSection();
 					section.AttributeTarget = attributeTarget;
 					section.Attributes.AddRange(attributes);
-					attributedNode.AddChild(section, AttributedNode.AttributeRole);
+					attributedNode.AddChild(section, EntityDeclaration.AttributeRole);
 				}
 			}
 		}
@@ -1432,8 +1419,8 @@ namespace ICSharpCode.Decompiler.Ast
 			if (!secDeclProvider.HasSecurityDeclarations)
 				return;
 			var attributes = new List<ICSharpCode.NRefactory.CSharp.Attribute>();
-			foreach (var secDecl in secDeclProvider.SecurityDeclarations) {
-				foreach (var secAttribute in secDecl.SecurityAttributes) {
+			foreach (var secDecl in secDeclProvider.SecurityDeclarations.OrderBy(d => d.Action)) {
+				foreach (var secAttribute in secDecl.SecurityAttributes.OrderBy(a => a.AttributeType.FullName)) {
 					var attribute = new ICSharpCode.NRefactory.CSharp.Attribute();
 					attribute.AddAnnotation(secAttribute);
 					attribute.Type = ConvertType(secAttribute.AttributeType);
@@ -1475,14 +1462,14 @@ namespace ICSharpCode.Decompiler.Ast
 					var section = new AttributeSection();
 					section.AttributeTarget = attributeTarget;
 					section.Attributes.Add(attribute);
-					attributedNode.AddChild(section, AttributedNode.AttributeRole);
+					attributedNode.AddChild(section, EntityDeclaration.AttributeRole);
 				}
 			} else if (attributes.Count > 0) {
 				// use single section for all attributes
 				var section = new AttributeSection();
 				section.AttributeTarget = attributeTarget;
 				section.Attributes.AddRange(attributes);
-				attributedNode.AddChild(section, AttributedNode.AttributeRole);
+				attributedNode.AddChild(section, EntityDeclaration.AttributeRole);
 			}
 		}
 		
@@ -1496,6 +1483,7 @@ namespace ICSharpCode.Decompiler.Ast
 				ArrayType arrayType = argument.Type as ArrayType;
 				return new ArrayCreateExpression {
 					Type = ConvertType(arrayType != null ? arrayType.ElementType : argument.Type),
+					AdditionalArraySpecifiers = { new ArraySpecifier() },
 					Initializer = arrayInit
 				};
 			} else if (argument.Value is CustomAttributeArgument) {
@@ -1606,7 +1594,7 @@ namespace ICSharpCode.Decompiler.Ast
 		/// Sets new modifier if the member hides some other member from a base type.
 		/// </summary>
 		/// <param name="member">The node of the member which new modifier state should be determined.</param>
-		static void SetNewModifier(AttributedNode member)
+		static void SetNewModifier(EntityDeclaration member)
 		{
 			try {
 				bool addNewModifier = false;
@@ -1626,7 +1614,7 @@ namespace ICSharpCode.Decompiler.Ast
 			}
 		}
 
-		private static bool HidesBaseMember(AttributedNode member)
+		private static bool HidesBaseMember(EntityDeclaration member)
 		{
 			var memberDefinition = member.Annotation<IMemberDefinition>();
 			bool addNewModifier = false;
@@ -1678,11 +1666,5 @@ namespace ICSharpCode.Decompiler.Ast
 			                   && (condition == null || condition(m))
 			                   && TypesHierarchyHelpers.IsVisibleFromDerived(m, derived.DeclaringType));
 		}
-		
-		/// <summary>
-		/// Gets the local variables for the current decompiled type, method, etc.
-		/// <remarks>The key is the metadata token.</remarks>
-		/// </summary>
-		public ConcurrentDictionary<int, IEnumerable<ILVariable>> LocalVariables { get; private set; }
 	}
 }
