@@ -42,15 +42,18 @@ namespace ICSharpCode.Decompiler.ILAst
 		SimplifyTernaryOperator,
 		SimplifyNullCoalescing,
 		JoinBasicBlocks,
+		SimplifyLogicNot,
 		SimplifyShiftOperators,
 		TransformDecimalCtorToConstant,
 		SimplifyLdObjAndStObj,
 		SimplifyCustomShortCircuit,
+		SimplifyLiftedOperators,
 		TransformArrayInitializers,
 		TransformMultidimensionalArrayInitializers,
 		TransformObjectInitializers,
 		MakeAssignmentExpression,
 		IntroducePostIncrement,
+		InlineExpressionTreeParameterDeclarations,
 		InlineVariables2,
 		FindLoops,
 		FindConditions,
@@ -134,6 +137,9 @@ namespace ICSharpCode.Decompiler.ILAst
 					if (abortBeforeStep == ILAstOptimizationStep.JoinBasicBlocks) return;
 					modified |= block.RunOptimization(new SimpleControlFlow(context, method).JoinBasicBlocks);
 
+					if (abortBeforeStep == ILAstOptimizationStep.SimplifyLogicNot) return;
+					modified |= block.RunOptimization(SimplifyLogicNot);
+
 					if (abortBeforeStep == ILAstOptimizationStep.SimplifyShiftOperators) return;
 					modified |= block.RunOptimization(SimplifyShiftOperators);
 
@@ -146,6 +152,9 @@ namespace ICSharpCode.Decompiler.ILAst
 					
 					if (abortBeforeStep == ILAstOptimizationStep.SimplifyCustomShortCircuit) return;
 					modified |= block.RunOptimization(new SimpleControlFlow(context, method).SimplifyCustomShortCircuit);
+
+					if (abortBeforeStep == ILAstOptimizationStep.SimplifyLiftedOperators) return;
+//					modified |= block.RunOptimization(SimplifyLiftedOperators);
 					
 					if (abortBeforeStep == ILAstOptimizationStep.TransformArrayInitializers) return;
 					modified |= block.RunOptimization(TransformArrayInitializers);
@@ -163,6 +172,11 @@ namespace ICSharpCode.Decompiler.ILAst
 					if (abortBeforeStep == ILAstOptimizationStep.IntroducePostIncrement) return;
 					modified |= block.RunOptimization(IntroducePostIncrement);
 					
+					if (abortBeforeStep == ILAstOptimizationStep.InlineExpressionTreeParameterDeclarations) return;
+					if (context.Settings.ExpressionTrees) {
+						modified |= block.RunOptimization(InlineExpressionTreeParameterDeclarations);
+					}
+					
 					if (abortBeforeStep == ILAstOptimizationStep.InlineVariables2) return;
 					modified |= new ILInlining(method).InlineAllInBlock(block);
 					new ILInlining(method).CopyPropagation();
@@ -307,27 +321,30 @@ namespace ICSharpCode.Decompiler.ILAst
 			for (int i = 0; i < block.Body.Count; i++) {
 				ILExpression expr = block.Body[i] as ILExpression;
 				if (expr != null && expr.Prefixes == null) {
+					ILCode op;
 					switch(expr.Code) {
 						case ILCode.Switch:
 						case ILCode.Brtrue:
 							expr.Arguments.Single().ILRanges.AddRange(expr.ILRanges);
 							expr.ILRanges.Clear();
 							continue;
-							case ILCode.__Brfalse:  block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.LogicNot, null, expr.Arguments.Single())); break;
-							case ILCode.__Beq:      block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.Ceq, null, expr.Arguments)); break;
-							case ILCode.__Bne_Un:   block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.LogicNot, null, new ILExpression(ILCode.Ceq, null, expr.Arguments))); break;
-							case ILCode.__Bgt:      block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.Cgt, null, expr.Arguments)); break;
-							case ILCode.__Bgt_Un:   block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.Cgt_Un, null, expr.Arguments)); break;
-							case ILCode.__Ble:      block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.LogicNot, null, new ILExpression(ILCode.Cgt, null, expr.Arguments))); break;
-							case ILCode.__Ble_Un:   block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.LogicNot, null, new ILExpression(ILCode.Cgt_Un, null, expr.Arguments))); break;
-							case ILCode.__Blt:      block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.Clt, null, expr.Arguments)); break;
-							case ILCode.__Blt_Un:   block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.Clt_Un, null, expr.Arguments)); break;
-							case ILCode.__Bge:	    block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.LogicNot, null, new ILExpression(ILCode.Clt, null, expr.Arguments))); break;
-							case ILCode.__Bge_Un:   block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, new ILExpression(ILCode.LogicNot, null, new ILExpression(ILCode.Clt_Un, null, expr.Arguments))); break;
+							case ILCode.__Brfalse:  op = ILCode.LogicNot; break;
+							case ILCode.__Beq:      op = ILCode.Ceq; break;
+							case ILCode.__Bne_Un:   op = ILCode.Cne; break;
+							case ILCode.__Bgt:      op = ILCode.Cgt; break;
+							case ILCode.__Bgt_Un:   op = ILCode.Cgt_Un; break;
+							case ILCode.__Ble:      op = ILCode.Cle; break;
+							case ILCode.__Ble_Un:   op = ILCode.Cle_Un; break;
+							case ILCode.__Blt:      op = ILCode.Clt; break;
+							case ILCode.__Blt_Un:   op = ILCode.Clt_Un; break;
+							case ILCode.__Bge:	    op = ILCode.Cge; break;
+							case ILCode.__Bge_Un:   op = ILCode.Cge_Un; break;
 						default:
 							continue;
 					}
-					((ILExpression)block.Body[i]).Arguments.Single().ILRanges.AddRange(expr.ILRanges);
+					var newExpr = new ILExpression(op, null, expr.Arguments);
+					block.Body[i] = new ILExpression(ILCode.Brtrue, expr.Operand, newExpr);
+					newExpr.ILRanges = expr.ILRanges;
 				}
 			}
 		}
@@ -721,6 +738,7 @@ namespace ICSharpCode.Decompiler.ILAst
 				case ILCode.Ldc_I8:
 				case ILCode.Ldc_R4:
 				case ILCode.Ldc_R8:
+				case ILCode.Ldc_Decimal:
 					return true;
 				default:
 					return false;
