@@ -397,9 +397,21 @@ namespace ICSharpCode.Decompiler.ILAst
 				return false;
 			
 			ILExpression op = expr.Arguments.Last();
-			if (!CanBeRepresentedAsCompoundAssignment(op.Code))
+			// in case of compound assignments with a lifted operator the result is inside NullableOf and the operand is inside ValueOf
+			bool liftedOperator = false;
+			if (op.Code == ILCode.NullableOf) {
+				op = op.Arguments[0];
+				liftedOperator = true;
+			}
+			if (!CanBeRepresentedAsCompoundAssignment(op))
 				return false;
+
 			ILExpression ldelem = op.Arguments[0];
+			if (liftedOperator) {
+				if (ldelem.Code != ILCode.ValueOf)
+					return false;
+				ldelem = ldelem.Arguments[0];
+			}
 			if (ldelem.Code != expectedLdelemCode)
 				return false;
 			Debug.Assert(ldelem.Arguments.Count == expr.Arguments.Count - 1);
@@ -414,9 +426,9 @@ namespace ICSharpCode.Decompiler.ILAst
 			return true;
 		}
 		
-		static bool CanBeRepresentedAsCompoundAssignment(ILCode code)
+		static bool CanBeRepresentedAsCompoundAssignment(ILExpression expr)
 		{
-			switch (code) {
+			switch (expr.Code) {
 				case ILCode.Add:
 				case ILCode.Add_Ovf:
 				case ILCode.Add_Ovf_Un:
@@ -437,6 +449,24 @@ namespace ICSharpCode.Decompiler.ILAst
 				case ILCode.Shr:
 				case ILCode.Shr_Un:
 					return true;
+				case ILCode.Call:
+					var m = expr.Operand as MethodReference;
+					if (m == null || m.HasThis || expr.Arguments.Count != 2) return false;
+					switch (m.Name) {
+						case "op_Addition":
+						case "op_Subtraction":
+						case "op_Multiply":
+						case "op_Division":
+						case "op_Modulus":
+						case "op_BitwiseAnd":
+						case "op_BitwiseOr":
+						case "op_ExclusiveOr":
+						case "op_LeftShift":
+						case "op_RightShift":
+							return true;
+						default:
+							return false;
+					}
 				default:
 					return false;
 			}
@@ -869,6 +899,78 @@ namespace ICSharpCode.Decompiler.ILAst
 		}
 		#endregion
 		
+		#region SimplifyLogicNot
+		static bool SimplifyLogicNot(List<ILNode> body, ILExpression expr, int pos)
+		{
+			bool modified = false;
+			expr = SimplifyLogicNot(expr, ref modified);
+			Debug.Assert(expr == null);
+			return modified;
+		}
+
+		static ILExpression SimplifyLogicNot(ILExpression expr, ref bool modified)
+		{
+			ILExpression a;
+			// "ceq(a, ldc.i4.0)" becomes "logicnot(a)" if the inferred type for expression "a" is boolean
+			if (expr.Code == ILCode.Ceq && TypeAnalysis.IsBoolean(expr.Arguments[0].InferredType) && (a = expr.Arguments[1]).Code == ILCode.Ldc_I4 && (int)a.Operand == 0) {
+				expr.Code = ILCode.LogicNot;
+				expr.ILRanges.AddRange(a.ILRanges);
+				expr.Arguments.RemoveAt(1);
+				modified = true;
+			}
+
+			ILExpression res = null;
+			while (expr.Code == ILCode.LogicNot) {
+				a = expr.Arguments[0];
+				// remove double negation
+				if (a.Code == ILCode.LogicNot) {
+					res = a.Arguments[0];
+					res.ILRanges.AddRange(expr.ILRanges);
+					res.ILRanges.AddRange(a.ILRanges);
+					expr = res;
+				} else {
+					if (SimplifyLogicNotArgument(expr)) res = expr = a;
+					break;
+				}
+			}
+
+			for (int i = 0; i < expr.Arguments.Count; i++) {
+				a = SimplifyLogicNot(expr.Arguments[i], ref modified);
+				if (a != null) {
+					expr.Arguments[i] = a;
+					modified = true;
+				}
+			}
+
+			return res;
+		}
+
+		/// <summary>
+		/// If the argument is a binary comparison operation then the negation is pushed through it
+		/// </summary>
+		static bool SimplifyLogicNotArgument(ILExpression expr)
+		{
+			var a = expr.Arguments[0];
+			ILCode c;
+			switch (a.Code) {
+					case ILCode.Ceq: c = ILCode.Cne; break;
+					case ILCode.Cne: c = ILCode.Ceq; break;
+					case ILCode.Cgt: c = ILCode.Cle; break;
+					case ILCode.Cgt_Un: c = ILCode.Cle_Un; break;
+					case ILCode.Cge: c = ILCode.Clt; break;
+					case ILCode.Cge_Un: c = ILCode.Clt_Un; break;
+					case ILCode.Clt: c = ILCode.Cge; break;
+					case ILCode.Clt_Un: c = ILCode.Cge_Un; break;
+					case ILCode.Cle: c = ILCode.Cgt; break;
+					case ILCode.Cle_Un: c = ILCode.Cgt_Un; break;
+					default: return false;
+			}
+			a.Code = c;
+			a.ILRanges.AddRange(expr.ILRanges);
+			return true;
+		}
+		#endregion
+
 		#region SimplifyShiftOperators
 		static bool SimplifyShiftOperators(List<ILNode> body, ILExpression expr, int pos)
 		{
@@ -880,20 +982,20 @@ namespace ICSharpCode.Decompiler.ILAst
 
 		static void SimplifyShiftOperators(ILExpression expr, ref bool modified)
 		{
-			for (int i = 0; i < expr.Arguments.Count; i++) 
+			for (int i = 0; i < expr.Arguments.Count; i++)
 				SimplifyShiftOperators(expr.Arguments[i], ref modified);
-			if (expr.Code != ILCode.Shl && expr.Code != ILCode.Shr && expr.Code != ILCode.Shr_Un) 
+			if (expr.Code != ILCode.Shl && expr.Code != ILCode.Shr && expr.Code != ILCode.Shr_Un)
 				return;
 			var a = expr.Arguments[1];
-			if (a.Code != ILCode.And || a.Arguments[1].Code != ILCode.Ldc_I4 || expr.InferredType == null) 
+			if (a.Code != ILCode.And || a.Arguments[1].Code != ILCode.Ldc_I4 || expr.InferredType == null)
 				return;
 			int mask;
 			switch (expr.InferredType.MetadataType) {
 				case MetadataType.Int32:
-				case MetadataType.UInt32: mask = 31; break;
+					case MetadataType.UInt32: mask = 31; break;
 				case MetadataType.Int64:
-				case MetadataType.UInt64: mask = 63; break;
-				default: return;
+					case MetadataType.UInt64: mask = 63; break;
+					default: return;
 			}
 			if ((int)a.Arguments[1].Operand != mask) return;
 			var res = a.Arguments[0];
@@ -903,5 +1005,81 @@ namespace ICSharpCode.Decompiler.ILAst
 			modified = true;
 		}
 		#endregion
+		
+		#region InlineExpressionTreeParameterDeclarations
+		bool InlineExpressionTreeParameterDeclarations(List<ILNode> body, ILExpression expr, int pos)
+		{
+			// When there is a Expression.Lambda() call, and the parameters are declared in the
+			// IL statement immediately prior to the one containing the Lambda() call,
+			// using this code for the3 declaration:
+			//   stloc(v, call(Expression::Parameter, call(Type::GetTypeFromHandle, ldtoken(...)), ldstr(...)))
+			// and the variables v are assigned only once (in that statements), and read only in a Expression::Lambda
+			// call that immediately follows the assignment statements, then we will inline those assignments
+			// into the Lambda call using ILCode.ExpressionTreeParameterDeclarations.
+			
+			// This is sufficient to allow inlining over the expression tree construction. The remaining translation
+			// of expression trees into C# will be performed by a C# AST transformer.
+			
+			for (int i = expr.Arguments.Count - 1; i >= 0; i--) {
+				if (InlineExpressionTreeParameterDeclarations(body, expr.Arguments[i], pos))
+					return true;
+			}
+			
+			MethodReference mr;
+			ILExpression lambdaBodyExpr, parameterArray;
+			if (!(expr.Match(ILCode.Call, out mr, out lambdaBodyExpr, out parameterArray) && mr.Name == "Lambda"))
+				return false;
+			if (!(parameterArray.Code == ILCode.InitArray && mr.DeclaringType.FullName == "System.Linq.Expressions.Expression"))
+				return false;
+			int firstParameterPos = pos - parameterArray.Arguments.Count;
+			if (firstParameterPos < 0)
+				return false;
+			
+			ILExpression[] parameterInitExpressions = new ILExpression[parameterArray.Arguments.Count + 1];
+			for (int i = 0; i < parameterArray.Arguments.Count; i++) {
+				parameterInitExpressions[i] = body[firstParameterPos + i] as ILExpression;
+				if (!MatchParameterVariableAssignment(parameterInitExpressions[i]))
+					return false;
+				ILVariable v = (ILVariable)parameterInitExpressions[i].Operand;
+				if (!parameterArray.Arguments[i].MatchLdloc(v))
+					return false;
+				// TODO: validate that the variable is only used here and within 'body'
+			}
+			
+			parameterInitExpressions[parameterInitExpressions.Length - 1] = lambdaBodyExpr;
+			Debug.Assert(expr.Arguments[0] == lambdaBodyExpr);
+			expr.Arguments[0] = new ILExpression(ILCode.ExpressionTreeParameterDeclarations, null, parameterInitExpressions);
+			
+			body.RemoveRange(firstParameterPos, parameterArray.Arguments.Count);
+			
+			return true;
+		}
+		
+		bool MatchParameterVariableAssignment(ILExpression expr)
+		{
+			// stloc(v, call(Expression::Parameter, call(Type::GetTypeFromHandle, ldtoken(...)), ldstr(...)))
+			ILVariable v;
+			ILExpression init;
+			if (!expr.Match(ILCode.Stloc, out v, out init))
+				return false;
+			if (v.IsGenerated || v.IsParameter || v.IsPinned)
+				return false;
+			if (v.Type == null || v.Type.FullName != "System.Linq.Expressions.ParameterExpression")
+				return false;
+			MethodReference parameterMethod;
+			ILExpression typeArg, nameArg;
+			if (!init.Match(ILCode.Call, out parameterMethod, out typeArg, out nameArg))
+				return false;
+			if (!(parameterMethod.Name == "Parameter" && parameterMethod.DeclaringType.FullName == "System.Linq.Expressions.Expression"))
+				return false;
+			MethodReference getTypeFromHandle;
+			ILExpression typeToken;
+			if (!typeArg.Match(ILCode.Call, out getTypeFromHandle, out typeToken))
+				return false;
+			if (!(getTypeFromHandle.Name == "GetTypeFromHandle" && getTypeFromHandle.DeclaringType.FullName == "System.Type"))
+				return false;
+			return typeToken.Code == ILCode.Ldtoken && nameArg.Code == ILCode.Ldstr;
+		}
+		#endregion
 	}
 }
