@@ -13,7 +13,8 @@ namespace ICSharpCode.OldNRefactory.PrettyPrinter
 		EndOfLineWithoutSpace,
 		NextLine,
 		NextLineShifted,
-		NextLineShifted2
+		NextLineShifted2,
+		BannerStyle
 	}
 	
 	public enum BraceForcement {
