@@ -37,9 +37,6 @@ namespace MonoDevelop.AspNet.StateEngine
 	
 	public class HtmlTagState : XmlTagState
 	{
-		XmlAttributeState AttributeState;
-		XmlNameState NameState;
-		XmlMalformedTagState MalformedTagState;
 		HtmlScriptBodyState ScriptState;
 		bool warnAutoClose;
 		
@@ -92,7 +89,7 @@ namespace MonoDevelop.AspNet.StateEngine
 							parent.Name.Name, element.Name.Name), parent.Region);
 					}
 					//parent.Region.End = element.Region.Start;
-					//parent.Region.End.Column = Math.Max (parent.Region.End.Column - 1, 1);
+					//parent.Region.EndColumn = Math.Max (parent.Region.EndColumn - 1, 1);
 					parent.Close (parent);
 					context.Nodes.Push (element);
 					
