@@ -39,6 +39,7 @@ using MonoDevelop.Ide.CodeCompletion;
 
 using CBinding.Parser;
 using MonoDevelop.Core;
+using ICSharpCode.NRefactory.Completion;
 
 namespace CBinding
 {
@@ -46,9 +47,17 @@ namespace CBinding
 	{
 		private Mono.TextEditor.TextEditorData editor;
 		private List<Function> functions = new List<Function> ();
+		int startOffset;
+
+		public int StartOffset {
+			get {
+				return startOffset;
+			}
+		}	
 		
-		public ParameterDataProvider (Document document, ProjectInformation info, string functionName)
+		public ParameterDataProvider (int startOffset, Document document, ProjectInformation info, string functionName)
 		{
+			this.startOffset = startOffset;
 			this.editor = document.Editor;
 			
 			foreach (Function f in info.Functions) {
@@ -71,7 +80,7 @@ namespace CBinding
 		}
 		
 		// Returns the number of methods
-		public int OverloadCount {
+		public int Count {
 			get { return functions.Count; }
 		}
 		
@@ -108,7 +117,7 @@ namespace CBinding
 		
 		// Returns the markup to use to represent the specified method overload
 		// in the parameter information window.
-		public string GetMethodMarkup (int overload, string[] parameterMarkup, int currentParameter)
+		public string GetHeading (int overload, string[] parameterMarkup, int currentParameter)
 		{
 			Function function = functions[overload];
 			string paramTxt = string.Join (", ", parameterMarkup);
@@ -127,8 +136,13 @@ namespace CBinding
 			return prename + "<b>" + function.Name + "</b>" + " (" + paramTxt + ")" + cons;
 		}
 		
+		public string GetDescription (int overload, int currentParameter)
+		{
+			return "";
+		}
+		
 		// Returns the text to use to represent the specified parameter
-		public string GetParameterMarkup (int overload, int paramIndex)
+		public string GetParameterDescription (int overload, int paramIndex)
 		{
 			Function function = functions[overload];
 			
@@ -140,6 +154,11 @@ namespace CBinding
 		{
 			return functions[overload].Parameters.Length;
 		}
+		
+		public bool AllowParameterList (int overload)
+		{
+			return false;
+		}
 	}
 	
 	public class CompletionData : MonoDevelop.Ide.CodeCompletion.CompletionData
