@@ -55,12 +55,13 @@ namespace MonoDevelop.CSharp
 
 		public static BuildResult Compile (ProjectItemCollection projectItems, DotNetProjectConfiguration configuration, ConfigurationSelector configSelector, IProgressMonitor monitor)
 		{
-			CSharpCompilerParameters compilerParameters = (CSharpCompilerParameters)configuration.CompilationParameters ?? new CSharpCompilerParameters ();
-			CSharpProjectParameters projectParameters = (CSharpProjectParameters)configuration.ProjectParameters ?? new CSharpProjectParameters ();
-			
-			FilePath outputName       = configuration.CompiledOutputName;
-			string responseFileName = Path.GetTempFileName();
+			var compilerParameters = (CSharpCompilerParameters)configuration.CompilationParameters ?? new CSharpCompilerParameters ();
+			var projectParameters = (CSharpProjectParameters)configuration.ProjectParameters ?? new CSharpProjectParameters ();
 			
+			FilePath outputName = configuration.CompiledOutputName;
+			string responseFileName = Path.GetTempFileName ();
+
+			//make sure that the output file is writable
 			if (File.Exists (outputName)) {
 				bool isWriteable = false;
 				int count = 0;
@@ -79,18 +80,46 @@ namespace MonoDevelop.CSharp
 					return null;
 				}
 			}
-			
+
+			//get the runtime
 			TargetRuntime runtime = MonoDevelop.Core.Runtime.SystemAssemblyService.DefaultRuntime;
 			DotNetProject project = configuration.ParentItem as DotNetProject;
 			if (project != null)
 				runtime = project.TargetRuntime;
 
-			StringBuilder sb = new StringBuilder ();
+			//get the compiler name
+			string compilerName;
+			try {
+				compilerName = GetCompilerName (runtime, configuration.TargetFramework);
+			} catch (Exception e) {
+				string message = "Could not obtain a C# compiler";
+				monitor.ReportError (message, e);
+				return null;
+			}
+
+			var sb = new StringBuilder ();
+
+			HashSet<string> alreadyAddedReference = new HashSet<string> ();
+
+			var monoRuntime = runtime as MonoTargetRuntime;
+			if (!compilerParameters.NoStdLib && (monoRuntime == null || monoRuntime.HasMultitargetingMcs)) {
+				string corlib = project.AssemblyContext.GetAssemblyFullName ("mscorlib", project.TargetFramework);
+				if (corlib != null) {
+					corlib = project.AssemblyContext.GetAssemblyLocation (corlib, project.TargetFramework);
+				}
+				if (corlib == null) {
+					var br = new BuildResult ();
+					br.AddError (string.Format ("Could not find mscorlib for framework {0}", project.TargetFramework.Id));
+					return br;
+				}
+				AppendQuoted (sb, "/r:", corlib);
+				sb.AppendLine ("-nostdlib");
+			}
+
 			List<string> gacRoots = new List<string> ();
 			sb.AppendFormat ("\"/out:{0}\"", outputName);
 			sb.AppendLine ();
 			
-			HashSet<string> alreadyAddedReference = new HashSet<string> ();
 			foreach (ProjectReference lib in projectItems.GetAll <ProjectReference> ()) {
 				if (lib.ReferenceType == ReferenceType.Project && !(lib.OwnerProject.ParentSolution.FindProjectByName (lib.Reference) is DotNetProject))
 					continue;
@@ -146,7 +175,7 @@ namespace MonoDevelop.CSharp
 			}
 			
 			if (configuration.DebugMode) {
-				sb.AppendLine ("/debug:+");
+//				sb.AppendLine ("/debug:+");
 				sb.AppendLine ("/debug:full");
 			}
 			
@@ -280,14 +309,7 @@ namespace MonoDevelop.CSharp
 			
 			File.WriteAllText (responseFileName, sb.ToString ());
 			
-			string compilerName;
-			try {
-				compilerName = GetCompilerName (runtime, configuration.TargetFramework);
-			} catch (Exception e) {
-				string message = "Could not obtain a C# compiler";
-				monitor.ReportError (message, e);
-				return null;
-			}
+
 			
 			monitor.Log.WriteLine (compilerName + " /noconfig " + sb.ToString ().Replace ('\n',' '));
 			
