@@ -34,12 +34,13 @@ using System.IO;
 using System.CodeDom;
 using System.CodeDom.Compiler;
 using Microsoft.CSharp;
-using ICSharpCode.NRefactory;
-using ICSharpCode.NRefactory.Parser;
-using ICSharpCode.NRefactory.Visitors;
+using ICSharpCode.OldNRefactory;
+using ICSharpCode.OldNRefactory.Parser;
+using ICSharpCode.OldNRefactory.Visitors;
 
 namespace MonoDevelop.CSharp
 {
+	[System.ComponentModel.DesignerCategory ("Code")]
 	public class CSharpEnhancedCodeProvider : CSharpCodeProvider
 	{
 		private ICodeParser codeParser;
