@@ -70,12 +70,15 @@ namespace MonoDevelop.Platform {
 			public IntPtr prev;
 		}
 
-		static DesktopApplication AppFromAppInfoPtr (IntPtr handle)
+		static GnomeDesktopApplication AppFromAppInfoPtr (IntPtr handle, DesktopApplication defaultApp)
 		{
 			string id = GLib.Marshaller.Utf8PtrToString (g_app_info_get_id (handle));
 			string name = GLib.Marshaller.Utf8PtrToString (g_app_info_get_name (handle));
 			string executable = GLib.Marshaller.Utf8PtrToString (g_app_info_get_executable (handle));
-			return new DesktopApplication (id, name, executable);
+			
+			if (!string.IsNullOrEmpty (name) && !string.IsNullOrEmpty (executable) && !executable.Contains ("monodevelop "))
+				return new GnomeDesktopApplication (executable, name, defaultApp != null && defaultApp.Id == id);
+			return null;
 		}
 
 		static IntPtr ContentTypeFromMimeType (string mime_type)
@@ -96,24 +99,29 @@ namespace MonoDevelop.Platform {
 			IntPtr content_type = ContentTypeFromMimeType (mime_type);
 			IntPtr ret = g_app_info_get_default_for_type (content_type, false);
 			GLib.Marshaller.Free (content_type);
-			return ret == IntPtr.Zero ? new DesktopApplication () : AppFromAppInfoPtr (ret);
+			return ret == IntPtr.Zero ? null : AppFromAppInfoPtr (ret, null);
 		}
 
-		public static DesktopApplication[] GetAllForType (string mime_type)
+		public static System.Collections.Generic.IList<DesktopApplication> GetAllForType (string mime_type)
 		{
+			var def = GetDefaultForType (mime_type);
+			
 			IntPtr content_type = ContentTypeFromMimeType (mime_type);
 			IntPtr ret = g_app_info_get_all_for_type (content_type);
 			GLib.Marshaller.Free (content_type);
 			IntPtr l = ret;
-			ArrayList apps = new ArrayList ();
+			var apps = new System.Collections.Generic.List<DesktopApplication> ();
 			while (l != IntPtr.Zero) {
 				GList node = (GList) Marshal.PtrToStructure (l, typeof (GList));
-				if (node.data != IntPtr.Zero)
-					apps.Add (AppFromAppInfoPtr (node.data));
+				if (node.data != IntPtr.Zero) {
+					var app = AppFromAppInfoPtr (node.data, def);
+					if (app != null)
+						apps.Add (app);
+				}
 				l = node.next;
 			}
 			g_list_free (ret);
-			return (DesktopApplication[]) apps.ToArray (typeof (DesktopApplication));
+			return apps;
 		}
 
 		public static string GetMimeTypeDescription (string mime_type)
