@@ -39,6 +39,6 @@ namespace MonoDevelop.AssemblyBrowser
 		string GetDocumentationMarkup (ITreeNavigator navigator);
 		string GetDescription (ITreeNavigator navigator);
 		List<ReferenceSegment> Disassemble (TextEditorData data, ITreeNavigator navigator);
-		List<ReferenceSegment> Decompile (TextEditorData data, ITreeNavigator navigator);
+		List<ReferenceSegment> Decompile (TextEditorData data, ITreeNavigator navigator, bool publicOnly);
 	}
 }
