@@ -33,6 +33,7 @@ using Mono.Debugging.Client;
 using System.Collections.Generic;
 using System.Text;
 using System.Reflection;
+using System.Reflection.Emit;
 using ST = System.Threading;
 using Mono.Debugging.Backend;
 
@@ -40,6 +41,24 @@ namespace Mono.Debugging.Soft
 {
 	public class SoftDebuggerAdaptor : ObjectValueAdaptor
 	{
+		static Dictionary<Type, OpCode> convertOps = new Dictionary<Type, OpCode> ();
+		delegate object TypeCastDelegate (object value);
+
+		static SoftDebuggerAdaptor ()
+		{
+			convertOps.Add (typeof (double), OpCodes.Conv_R8);
+			convertOps.Add (typeof (float), OpCodes.Conv_R4);
+			convertOps.Add (typeof (ulong), OpCodes.Conv_U8);
+			convertOps.Add (typeof (uint), OpCodes.Conv_U4);
+			convertOps.Add (typeof (ushort), OpCodes.Conv_U2);
+			convertOps.Add (typeof (char), OpCodes.Conv_U2);
+			convertOps.Add (typeof (byte), OpCodes.Conv_U1);
+			convertOps.Add (typeof (long), OpCodes.Conv_I8);
+			convertOps.Add (typeof (int), OpCodes.Conv_I4);
+			convertOps.Add (typeof (short), OpCodes.Conv_I2);
+			convertOps.Add (typeof (sbyte), OpCodes.Conv_I1);
+		}
+
 		public SoftDebuggerAdaptor ()
 		{
 		}
@@ -48,6 +67,9 @@ namespace Mono.Debugging.Soft
 		{
 			SoftEvaluationContext cx = (SoftEvaluationContext) ctx;
 			
+			if (obj == null)
+				return null;
+			
 			if (obj is StringMirror)
 				return ((StringMirror)obj).Value;
 			else if (obj is EnumMirror) {
@@ -56,20 +78,20 @@ namespace Mono.Debugging.Soft
 			}
 			else if (obj is PrimitiveValue)
 				return ((PrimitiveValue)obj).Value.ToString ();
+			else if (obj is PointerValue)
+				return string.Format ("0x{0:x}", ((PointerValue)obj).Address);
 			else if ((obj is StructMirror) && ((StructMirror)obj).Type.IsPrimitive) {
 				// Boxed primitive
 				StructMirror sm = (StructMirror) obj;
 				if (sm.Fields.Length > 0 && (sm.Fields[0] is PrimitiveValue))
 					return ((PrimitiveValue)sm.Fields[0]).Value.ToString ();
 			}
-			else if (obj == null)
-				return string.Empty;
 			else if ((obj is ObjectMirror) && cx.Options.AllowTargetInvoke) {
 				ObjectMirror ob = (ObjectMirror) obj;
 				MethodMirror method = OverloadResolve (cx, "ToString", ob.Type, new TypeMirror[0], true, false, false);
 				if (method != null && method.DeclaringType.FullName != "System.Object") {
 					StringMirror res = cx.RuntimeInvoke (method, obj, new Value[0]) as StringMirror;
-					return res != null ? res.Value : string.Empty;
+					return res != null ? res.Value : null;
 				}
 			}
 			else if ((obj is StructMirror) && cx.Options.AllowTargetInvoke) {
@@ -77,23 +99,25 @@ namespace Mono.Debugging.Soft
 				MethodMirror method = OverloadResolve (cx, "ToString", ob.Type, new TypeMirror[0], true, false, false);
 				if (method != null && method.DeclaringType.FullName != "System.ValueType") {
 					StringMirror res = cx.RuntimeInvoke (method, obj, new Value[0]) as StringMirror;
-					return res != null ? res.Value : string.Empty;
+					return res != null ? res.Value : null;
 				}
 			}
+			
 			return GetDisplayTypeName (GetValueTypeName (ctx, obj));
 		}
 
 		public override object TryConvert (EvaluationContext ctx, object obj, object targetType)
 		{
 			object res = TryCast (ctx, obj, targetType);
-			if (res != null)
+			
+			if (res != null || obj == null)
 				return res;
-			if (obj == null)
-				return null;
+			
 			object otype = GetValueType (ctx, obj);
 			if (otype is Type) {
 				if (targetType is TypeMirror)
 					targetType = Type.GetType (((TypeMirror)targetType).FullName, false);
+				
 				Type tt = targetType as Type;
 				if (tt != null) {
 					try {
@@ -108,49 +132,182 @@ namespace Mono.Debugging.Soft
 			return null;
 		}
 
+		static TypeCastDelegate GenerateTypeCastDelegate (string methodName, Type fromType, Type toType)
+		{
+			var argTypes = new Type[] {
+				typeof (object)
+			};
+			var method = new DynamicMethod (methodName, typeof (object), argTypes, true);
+			ILGenerator il = method.GetILGenerator ();
+			ConstructorInfo ctorInfo;
+			MethodInfo methodInfo;
+			OpCode conv;
+
+			il.Emit (OpCodes.Ldarg_0);
+			il.Emit (OpCodes.Unbox_Any, fromType);
+
+			if (fromType.IsSubclassOf (typeof (System.Nullable))) {
+				PropertyInfo propInfo = fromType.GetProperty ("Value");
+				methodInfo = propInfo.GetGetMethod ();
+
+				il.Emit (OpCodes.Stloc_0);
+				il.Emit (OpCodes.Ldloca_S);
+				il.Emit (OpCodes.Call, methodInfo);
+
+				fromType = methodInfo.ReturnType;
+			}
+
+			if (!convertOps.TryGetValue (toType, out conv)) {
+				argTypes = new Type[] {
+					fromType
+				};
+
+				if (toType == typeof (string)) {
+					methodInfo = fromType.GetMethod ("ToString", new Type[0]);
+					il.Emit (OpCodes.Call, methodInfo);
+				} else if ((methodInfo = toType.GetMethod ("op_Explicit", argTypes)) != null) {
+					il.Emit (OpCodes.Call, methodInfo);
+				} else if ((methodInfo = toType.GetMethod ("op_Implicit", argTypes)) != null) {
+					il.Emit (OpCodes.Call, methodInfo);
+				} else if ((ctorInfo = toType.GetConstructor (argTypes)) != null) {
+					il.Emit (OpCodes.Call, ctorInfo);
+				} else {
+					// No idea what else to try...
+					throw new InvalidCastException ();
+				}
+			} else {
+				il.Emit (conv);
+			}
+
+			il.Emit (OpCodes.Box, toType);
+			il.Emit (OpCodes.Ret);
+
+			return (TypeCastDelegate) method.CreateDelegate (typeof (TypeCastDelegate));
+		}
+
+		static object DynamicCast (object value, Type target)
+		{
+			string methodName = string.Format ("CastFrom{0}To{1}", value.GetType ().Name, target.Name);
+			TypeCastDelegate method = GenerateTypeCastDelegate (methodName, value.GetType (), target);
+
+			return method.Invoke (value);
+		}
+
+		object TryForceCast (EvaluationContext ctx, Value value, TypeMirror fromType, TypeMirror toType)
+		{
+			SoftEvaluationContext cx = (SoftEvaluationContext) ctx;
+			MethodMirror method;
+
+			method = OverloadResolve (cx, "op_Explicit", toType, new TypeMirror[] { fromType }, false, true, false);
+			if (method != null)
+				return cx.RuntimeInvoke (method, toType, new Value[] { value });
+
+			method = OverloadResolve (cx, "op_Implicit", toType, new TypeMirror[] { fromType }, false, true, false);
+			if (method != null)
+				return cx.RuntimeInvoke (method, toType, new Value[] { value });
+
+			// Finally, try a ctor...
+			try {
+				return CreateValue (ctx, toType, value);
+			} catch {
+				return null;
+			}
+		}
+
 		public override object TryCast (EvaluationContext ctx, object obj, object targetType)
 		{
 			SoftEvaluationContext cx = (SoftEvaluationContext) ctx;
+			TypeMirror toType = targetType as TypeMirror;
+			TypeMirror fromType;
+			
 			if (obj == null)
 				return null;
+			
 			object valueType = GetValueType (ctx, obj);
 			if (valueType is TypeMirror) {
-				if ((targetType is TypeMirror) && ((TypeMirror)targetType).IsAssignableFrom ((TypeMirror)valueType))
+				fromType = (TypeMirror) valueType;
+
+				if (toType != null && toType.IsAssignableFrom (fromType))
 					return obj;
+				
 				// Try casting the primitive type of the enum
 				EnumMirror em = obj as EnumMirror;
 				if (em != null)
 					return TryCast (ctx, CreateValue (ctx, em.Value), targetType);
+
+				if (toType == null)
+					return null;
+
+				MethodMirror method;
+
+				if (toType.CSharpName == "string") {
+					method = OverloadResolve (cx, "ToString", fromType, new TypeMirror[0], true, false, false);
+					if (method != null)
+						return cx.RuntimeInvoke (method, obj, new Value[0]);
+				}
+
+				if (fromType.IsGenericType && fromType.FullName.StartsWith ("System.Nullable`")) {
+					method = OverloadResolve (cx, "get_Value", fromType, new TypeMirror[0], true, false, false);
+					if (method != null) {
+						obj = cx.RuntimeInvoke (method, obj, new Value[0]);
+						return TryCast (ctx, obj, targetType);
+					}
+				}
+
+				return TryForceCast (ctx, (Value) obj, fromType, toType);
 			} else if (valueType is Type) {
-				if (targetType is TypeMirror) {
-					TypeMirror tm = (TypeMirror) targetType;
-					if (tm.IsEnum) {
-						PrimitiveValue casted = TryCast (ctx, obj, tm.EnumUnderlyingType) as PrimitiveValue;
+				if (toType != null) {
+					if (toType.IsEnum) {
+						PrimitiveValue casted = TryCast (ctx, obj, toType.EnumUnderlyingType) as PrimitiveValue;
 						if (casted == null)
 							return null;
-						return cx.Session.VirtualMachine.CreateEnumMirror (tm, casted);
+						return cx.Session.VirtualMachine.CreateEnumMirror (toType, casted);
 					}
-					targetType = Type.GetType (((TypeMirror)targetType).FullName, false);
+
+					targetType = Type.GetType (toType.FullName, false);
 				}
+				
 				Type tt = targetType as Type;
 				if (tt != null) {
-					if (tt.IsAssignableFrom ((Type)valueType))
+					if (tt.IsAssignableFrom ((Type) valueType))
 						return obj;
-					if (obj is PrimitiveValue)
-						obj = ((PrimitiveValue)obj).Value;
-					if (tt != typeof(string) && !(obj is string)) {
-						try {
+
+					try {
+						if (tt.IsPrimitive || tt == typeof (string)) {
+							if (obj is PrimitiveValue)
+								obj = ((PrimitiveValue) obj).Value;
+
 							if (obj == null)
 								return null;
-							object res = System.Convert.ChangeType (obj, tt);
+
+							object res;
+
+							try {
+								res = System.Convert.ChangeType (obj, tt);
+							} catch {
+								res = DynamicCast (obj, tt);
+							}
+
 							return CreateValue (ctx, res);
-						} catch {
+						} else {
+							fromType = (TypeMirror) ForceLoadType (ctx, ((Type) valueType).FullName);
+							if (toType == null)
+								toType = (TypeMirror) ForceLoadType (ctx, tt.FullName);
+
+							return TryForceCast (ctx, (Value) obj, fromType, toType);
 						}
+					} catch {
 					}
 				}
 			}
+
 			return null;
 		}
+		
+		public override IStringAdaptor CreateStringAdaptor (EvaluationContext ctx, object str)
+		{
+			return new StringAdaptor ((StringMirror) str);
+		}
 
 		public override ICollectionAdaptor CreateArrayAdaptor (EvaluationContext ctx, object arr)
 		{
@@ -184,6 +341,9 @@ namespace Mono.Debugging.Soft
 				values[n] = (Value) args [n];
 			
 			MethodMirror ctor = OverloadResolve (cx, ".ctor", t, types, true, true, true);
+			if (ctor == null)
+				return null;
+
 			return t.NewInstance (cx.Thread, ctor, values);
 		}
 
@@ -244,9 +404,15 @@ namespace Mono.Debugging.Soft
 				}
 				type = type.BaseType;
 			}
+			
 			MethodMirror idx = OverloadResolve ((SoftEvaluationContext) ctx, targetType.Name, null, types, candidates, true);
 			int i = candidates.IndexOf (idx);
-			return new PropertyValueReference (ctx, props[i], target, null, values);
+			
+			MethodMirror getter = props[i].GetGetMethod (true);
+			if (getter == null)
+				return null;
+			
+			return new PropertyValueReference (ctx, props[i], target, null, getter, values);
 		}
 		
 		static bool InGeneratedClosureOrIteratorType (EvaluationContext ctx)
@@ -255,26 +421,30 @@ namespace Mono.Debugging.Soft
 			if (cx.Frame.Method.IsStatic)
 				return false;
 			TypeMirror tm = cx.Frame.Method.DeclaringType;
-			return IsGeneratedClosureOrIteratorType (tm);
+			return IsGeneratedType (tm);
 		}
 		
-		static bool IsGeneratedClosureOrIteratorType (TypeMirror tm)
+		internal static bool IsGeneratedType (TypeMirror tm)
 		{
-			return IsGeneratedClosureType (tm) || IsGeneratedIteratorType (tm);
+			//
+			// This should cover all C# generated special containers
+			// - anonymous methods
+			// - lambdas
+			// - iterators
+			// - async methods
+			//
+			// which allow stepping into
+			//
+			return tm.Name[0] == '<' &&
+				// mcs is of the form <${NAME}>.c__{KIND}${NUMBER}
+				(tm.Name.IndexOf (">c__") > 0 ||
+				// csc is of form <${NAME}>d__${NUMBER}
+				 tm.Name.IndexOf (">d__") > 0);
 		}
-		
-		static bool IsGeneratedClosureType (TypeMirror tm)
+
+		internal static string GetNameFromGeneratedType (TypeMirror tm)
 		{
-			return tm.Name.IndexOf (">c__") != -1;
-		}
-		
-		static bool IsGeneratedIteratorType (TypeMirror tm)
-		{
-			return
-				//mcs
-				tm.Name.IndexOf (">c__Iterator") != -1
-				//csc is of form <NAME>d__NUMBER
-				|| (tm.Name.StartsWith ("<") && tm.Name.IndexOf (">d__") > -1);
+			return tm.Name.Substring (1, tm.Name.IndexOf ('>') - 1);
 		}
 		
 		static bool IsHoistedThisReference (FieldInfoMirror field)
@@ -298,7 +468,7 @@ namespace Mono.Debugging.Soft
 			
 			return
 				// mcs
-				local.Name.Length == 0 || local.Name.StartsWith ("<") || local.Name.StartsWith ("$locvar")
+				local.Name.Length == 0 || local.Name[0] == '<' || local.Name.StartsWith ("$locvar")
 				// csc
 				|| local.Name.StartsWith ("CS$<>");
 		}
@@ -335,7 +505,7 @@ namespace Mono.Debugging.Soft
 				return new ValueReference [0];
 			
 			TypeMirror tm = (TypeMirror) vthis.Type;
-			bool isIterator = IsGeneratedIteratorType (tm);
+			bool isIterator = IsGeneratedType (tm);
 			
 			var list = new List<ValueReference> ();
 			TypeMirror type = (TypeMirror) vthis.Type;
@@ -445,7 +615,7 @@ namespace Mono.Debugging.Soft
 			foreach (LocalVariable local in locals) {
 				if (local.IsArg)
 					continue;
-				if (IsClosureReferenceLocal (local) && IsGeneratedClosureType (local.Type)) {
+				if (IsClosureReferenceLocal (local) && IsGeneratedType (local.Type)) {
 					foreach (var gv in GetHoistedLocalVariables (cx, new VariableValueReference (cx, local.Name, local))) {
 						yield return gv;
 					}
@@ -455,22 +625,70 @@ namespace Mono.Debugging.Soft
 			}
 		}
 
+		public override bool HasMember (EvaluationContext ctx, object type, string memberName, BindingFlags bindingFlags)
+		{
+			TypeMirror tm = (TypeMirror) type;
+
+			while (tm != null) {
+				FieldInfoMirror field = FindByName (tm.GetFields (), f => f.Name, memberName, ctx.CaseSensitive);
+				if (field != null)
+					return true;
+
+				PropertyInfoMirror prop = FindByName (tm.GetProperties (), p => p.Name, memberName, ctx.CaseSensitive);
+				if (prop != null) {
+					MethodMirror getter = prop.GetGetMethod (bindingFlags.HasFlag (BindingFlags.NonPublic));
+					if (getter != null)
+						return true;
+				}
+
+				if (bindingFlags.HasFlag (BindingFlags.DeclaredOnly))
+					break;
+
+				tm = tm.BaseType;
+			}
+
+			return false;
+		}
+
 		protected override ValueReference GetMember (EvaluationContext ctx, object t, object co, string name)
 		{
 			TypeMirror type = (TypeMirror) t;
+
 			while (type != null) {
 				FieldInfoMirror field = FindByName (type.GetFields(), f => f.Name, name, ctx.CaseSensitive);
 				if (field != null && (field.IsStatic || co != null))
 					return new FieldValueReference (ctx, field, co, type);
+
 				PropertyInfoMirror prop = FindByName (type.GetProperties(), p => p.Name, name, ctx.CaseSensitive);
-				if (prop != null && (IsStatic (prop) || co != null))
-					return new PropertyValueReference (ctx, prop, co, type, null);
+				if (prop != null && (IsStatic (prop) || co != null)) {
+					// Optimization: if the property has a CompilerGenerated backing field, use that instead.
+					// This way we avoid overhead of invoking methods on the debugee when the value is requested.
+					string cgFieldName = string.Format ("<{0}>k__BackingField", prop.Name);
+					if ((field = FindByName (type.GetFields (), f => f.Name, cgFieldName, true)) != null && IsCompilerGenerated (field))
+						return new FieldValueReference (ctx, field, co, type, prop.Name, ObjectValueFlags.Property);
+
+					// Backing field not available, so do things the old fashioned way.
+					MethodMirror getter = prop.GetGetMethod (true);
+					if (getter == null)
+						return null;
+					
+					return new PropertyValueReference (ctx, prop, co, type, getter, null);
+				}
+
 				type = type.BaseType;
 			}
+
 			return null;
 		}
+
+		static bool IsCompilerGenerated (FieldInfoMirror field)
+		{
+			CustomAttributeDataMirror[] attrs = field.GetCustomAttributes (true);
+			var cga = GetAttribute<System.Runtime.CompilerServices.CompilerGeneratedAttribute> (attrs);
+			return cga != null;
+		}
 		
-		bool IsStatic (PropertyInfoMirror prop)
+		static bool IsStatic (PropertyInfoMirror prop)
 		{
 			MethodMirror met = prop.GetGetMethod (true) ?? prop.GetSetMethod (true);
 			return met.IsStatic;
@@ -525,24 +743,29 @@ namespace Mono.Debugging.Soft
 					yield return new FieldValueReference (ctx, field, co, type);
 				}
 				foreach (PropertyInfoMirror prop in type.GetProperties (bindingFlags)) {
-					MethodMirror met = prop.GetGetMethod (true);
-					if (met == null || met.GetParameters ().Length != 0 || met.IsAbstract)
+					MethodMirror getter = prop.GetGetMethod (true);
+					if (getter == null || getter.GetParameters ().Length != 0 || getter.IsAbstract)
 						continue;
-					if (met.IsStatic && ((bindingFlags & BindingFlags.Static) == 0))
+					if (getter.IsStatic && ((bindingFlags & BindingFlags.Static) == 0))
 						continue;
-					if (!met.IsStatic && ((bindingFlags & BindingFlags.Instance) == 0))
+					if (!getter.IsStatic && ((bindingFlags & BindingFlags.Instance) == 0))
 						continue;
-					if (met.IsPublic && ((bindingFlags & BindingFlags.Public) == 0))
+					if (getter.IsPublic && ((bindingFlags & BindingFlags.Public) == 0))
 						continue;
-					if (!met.IsPublic && ((bindingFlags & BindingFlags.NonPublic) == 0))
+					if (!getter.IsPublic && ((bindingFlags & BindingFlags.NonPublic) == 0))
 						continue;
 					
 					// If a property is overriden, return the override instead of the base property
-					PropertyInfoMirror overriden;
-					if (met.IsVirtual && subProps.TryGetValue (prop.Name, out overriden))
-						yield return new PropertyValueReference (ctx, overriden, co, overriden.DeclaringType, null);
-					else
-						yield return new PropertyValueReference (ctx, prop, co, type, null);
+					PropertyInfoMirror overridden;
+					if (getter.IsVirtual && subProps.TryGetValue (prop.Name, out overridden)) {
+						getter = overridden.GetGetMethod (true);
+						if (getter == null)
+							continue;
+						
+						yield return new PropertyValueReference (ctx, overridden, co, overridden.DeclaringType, getter, null);
+					} else {
+						yield return new PropertyValueReference (ctx, prop, co, type, getter, null);
+					}
 				}
 				if ((bindingFlags & BindingFlags.DeclaredOnly) != 0)
 					break;
@@ -720,9 +943,14 @@ namespace Mono.Debugging.Soft
 		public override string GetTypeName (EvaluationContext ctx, object val)
 		{
 			TypeMirror tm = val as TypeMirror;
-			if (tm != null)
+			if (tm != null) {
+				if (IsGeneratedType (tm)) {
+					// Return the name of the container-type.
+					return tm.FullName.Substring (0, tm.FullName.LastIndexOf ('+'));
+				}
+				
 				return tm.FullName;
-			else
+			} else
 				return ((Type)val).FullName;
 		}
 		
@@ -736,6 +964,8 @@ namespace Mono.Debugging.Soft
 				return ((EnumMirror)val).Type;
 			if (val is StructMirror)
 				return ((StructMirror)val).Type;
+			if (val is PointerValue)
+				return ((PointerValue) val).Type;
 			if (val is PrimitiveValue) {
 				PrimitiveValue pv = (PrimitiveValue) val;
 				if (pv.Value == null)
@@ -743,7 +973,7 @@ namespace Mono.Debugging.Soft
 				else
 					return pv.Value.GetType ();
 			}
-			
+
 			throw new NotSupportedException ();
 		}
 		
@@ -758,11 +988,9 @@ namespace Mono.Debugging.Soft
 		public override bool HasMethod (EvaluationContext gctx, object targetType, string methodName, object[] argTypes, BindingFlags flags)
 		{
 			SoftEvaluationContext ctx = (SoftEvaluationContext) gctx;
-			
-			TypeMirror[] types;
-			if (argTypes == null)
-				types = new TypeMirror [0];
-			else {
+			TypeMirror[] types = null;
+
+			if (argTypes != null) {
 				types = new TypeMirror [argTypes.Length];
 				for (int n=0; n<argTypes.Length; n++) {
 					if (argTypes [n] is TypeMirror)
@@ -786,7 +1014,11 @@ namespace Mono.Debugging.Soft
 				return true;
 		}
 
-
+		public override bool IsString (EvaluationContext ctx, object val)
+		{
+			return val is StringMirror;
+		}
+		
 		public override bool IsArray (EvaluationContext ctx, object val)
 		{
 			return val is ArrayMirror;
@@ -800,12 +1032,17 @@ namespace Mono.Debugging.Soft
 
 		public override bool IsNull (EvaluationContext ctx, object val)
 		{
-			return val == null || ((val is PrimitiveValue) && ((PrimitiveValue)val).Value == null);
+			return val == null || ((val is PrimitiveValue) && ((PrimitiveValue)val).Value == null) || ((val is PointerValue) && ((PointerValue)val).Address == 0);
 		}
 
 		public override bool IsPrimitive (EvaluationContext ctx, object val)
 		{
-			return val is PrimitiveValue || val is StringMirror || ((val is StructMirror) && ((StructMirror)val).Type.IsPrimitive);
+			return val is PrimitiveValue || val is StringMirror || ((val is StructMirror) && ((StructMirror)val).Type.IsPrimitive) || val is PointerValue;
+		}
+
+		public override bool IsPointer (EvaluationContext ctx, object val)
+		{
+			return val is PointerValue;
 		}
 
 		public override bool IsEnum (EvaluationContext ctx, object val)
@@ -871,7 +1108,7 @@ namespace Mono.Debugging.Soft
 			return new TypeDisplayData (proxyType, valueString, typeString, nameString, isCompilerGenerated, memberData);
 		}
 		
-		T GetAttribute<T> (CustomAttributeDataMirror[] attrs)
+		static T GetAttribute<T> (CustomAttributeDataMirror[] attrs)
 		{
 			foreach (CustomAttributeDataMirror attr in attrs) {
 				if (attr.Constructor.DeclaringType.FullName == typeof(T).FullName)
@@ -890,6 +1127,7 @@ namespace Mono.Debugging.Soft
 			SoftEvaluationContext ctx = (SoftEvaluationContext) gctx;
 			if (!ctx.Options.AllowTargetInvoke)
 				return null;
+			
 			TypeMirror tm = (TypeMirror) ctx.Thread.Type.GetTypeObject ().Type;
 			TypeMirror stype = ctx.Session.GetType ("System.String");
 			if (stype == null) {
@@ -897,20 +1135,23 @@ namespace Mono.Debugging.Soft
 				StringMirror ss = ctx.Thread.Domain.CreateString ("");
 				stype = ss.Type;
 			}
+			
 			TypeMirror[] ats = new TypeMirror[] { stype };
 			MethodMirror met = OverloadResolve (ctx, "GetType", tm, ats, false, true, true);
+			
 			try {
-				tm.InvokeMethod (ctx.Thread, met, new Value[] {(Value) CreateValue (ctx, typeName)});
+				tm.InvokeMethod (ctx.Thread, met, new Value[] {(Value) CreateValue (ctx, typeName)}, InvokeOptions.DisableBreakpoints | InvokeOptions.SingleThreaded);
 			} catch {
 				return null;
 			} finally {
 				ctx.Session.StackVersion++;
 			}
+			
 			return GetType (ctx, typeName);
 		}
 
 		
-		T BuildAttribute<T> (CustomAttributeDataMirror attr)
+		static T BuildAttribute<T> (CustomAttributeDataMirror attr)
 		{
 			List<object> args = new List<object> ();
 			foreach (CustomAttributeTypedArgumentMirror arg in attr.ConstructorArguments) {
@@ -954,15 +1195,9 @@ namespace Mono.Debugging.Soft
 		public override object RuntimeInvoke (EvaluationContext gctx, object targetType, object target, string methodName, object[] argTypes, object[] argValues)
 		{
 			SoftEvaluationContext ctx = (SoftEvaluationContext) gctx;
-			ctx.AssertTargetInvokeAllowed ();
+			TypeMirror type = ToTypeMirror (ctx, targetType);
 			
-			TypeMirror type;
-			if (target is ObjectMirror)
-				type = ((ObjectMirror) target).Type;
-			else if (target is StructMirror)
-				type = ((StructMirror) target).Type;
-			else
-				type = (TypeMirror) targetType;
+			ctx.AssertTargetInvokeAllowed ();
 			
 			TypeMirror[] types = new TypeMirror [argTypes.Length];
 			for (int n=0; n<argTypes.Length; n++)
@@ -979,43 +1214,51 @@ namespace Mono.Debugging.Soft
 		public static MethodMirror OverloadResolve (SoftEvaluationContext ctx, string methodName, TypeMirror type, TypeMirror[] argtypes, bool allowInstance, bool allowStatic, bool throwIfNotFound)
 		{
 			List<MethodMirror> candidates = new List<MethodMirror> ();
+			var cache = ctx.Session.OverloadResolveCache;
 			TypeMirror currentType = type;
+			
 			while (currentType != null) {
-				//
-				// Use a simple cached stored in SoftDebuggerSession, since
-				// GetMethodsByNameFlags might not do any caching.
-				//
-				var cache = ctx.Session.OverloadResolveCache;
 				MethodMirror[] methods = null;
+				
 				if (ctx.CaseSensitive) {
 					lock (cache) {
 						cache.TryGetValue (Tuple.Create (currentType, methodName), out methods);
 					}
 				}
+				
 				if (methods == null) {
 					if (currentType.VirtualMachine.Version.AtLeast (2, 7))
 						methods = currentType.GetMethodsByNameFlags (methodName, BindingFlags.Public|BindingFlags.NonPublic|BindingFlags.Instance|BindingFlags.Static, !ctx.CaseSensitive);
 					else
 						methods = currentType.GetMethods ();
-				}
-				if (ctx.CaseSensitive) {
-					lock (cache) {
-						cache [Tuple.Create (currentType, methodName)] = methods;
+					
+					if (ctx.CaseSensitive) {
+						lock (cache) {
+							cache [Tuple.Create (currentType, methodName)] = methods;
+						}
 					}
 				}
 				
 				foreach (MethodMirror met in methods) {
 					if (met.Name == methodName || (!ctx.CaseSensitive && met.Name.Equals (methodName, StringComparison.CurrentCultureIgnoreCase))) {
 						ParameterInfoMirror[] pars = met.GetParameters ();
-						if (pars.Length == argtypes.Length && (met.IsStatic && allowStatic || !met.IsStatic && allowInstance))
+						if (argtypes == null || pars.Length == argtypes.Length && ((met.IsStatic && allowStatic) || (!met.IsStatic && allowInstance)))
 							candidates.Add (met);
 					}
 				}
+
+				if (argtypes == null && candidates.Count > 0)
+					break; // when argtypes is null, we are just looking for *any* match (not a specific match)
+				
 				if (methodName == ".ctor")
 					break; // Can't create objects using constructor from base classes
-				currentType = currentType.BaseType;
+				
+				// Make sure that we always pull in at least System.Object methods (this is mostly needed for cases where 'type' was an interface)
+				if (currentType.BaseType == null && currentType.FullName != "System.Object")
+					currentType = ctx.Session.GetType ("System.Object");
+				else
+					currentType = currentType.BaseType;
 			}
-			
 
 			return OverloadResolve (ctx, type.Name, methodName, argtypes, candidates, throwIfNotFound);
 		}
@@ -1026,7 +1269,6 @@ namespace Mono.Debugging.Soft
 			matchCount = 0;
 
 			for (int i = 0; i < types.Length; i++) {
-				
 				TypeMirror param_type = mparams[i].ParameterType;
 
 				if (param_type.FullName == types [i].FullName) {
@@ -1049,9 +1291,22 @@ namespace Mono.Debugging.Soft
 
 		static MethodMirror OverloadResolve (SoftEvaluationContext ctx, string typeName, string methodName, TypeMirror[] argtypes, List<MethodMirror> candidates, bool throwIfNotFound)
 		{
+			if (candidates.Count == 0) {
+				if (throwIfNotFound)
+					throw new EvaluatorException ("Method `{0}' not found in type `{1}'.", methodName, typeName);
+				else
+					return null;
+			}
+
+			if (argtypes == null) {
+				// This is just a probe to see if the type contains *any* methods of the given name
+				return candidates[0];
+			}
+
 			if (candidates.Count == 1) {
 				string error;
 				int matchCount;
+
 				if (IsApplicable (ctx, candidates[0], argtypes, out error, out matchCount))
 					return candidates [0];
 
@@ -1060,13 +1315,6 @@ namespace Mono.Debugging.Soft
 				else
 					return null;
 			}
-
-			if (candidates.Count == 0) {
-				if (throwIfNotFound)
-					throw new EvaluatorException ("Method `{0}' not found in type `{1}'.", methodName, typeName);
-				else
-					return null;
-			}
 			
 			// Ok, now we need to find an exact match.
 			MethodMirror match = null;
@@ -1137,9 +1385,11 @@ namespace Mono.Debugging.Soft
 				}
 				
 				return str;
-			} else if (obj is PrimitiveValue)
+			} else if (obj is PrimitiveValue) {
 				return ((PrimitiveValue)obj).Value;
-			else if ((obj is StructMirror) && ((StructMirror)obj).Type.IsPrimitive) {
+			} else if (obj is PointerValue) {
+				return new IntPtr (((PointerValue)obj).Address);
+			} else if ((obj is StructMirror) && ((StructMirror)obj).Type.IsPrimitive) {
 				// Boxed primitive
 				StructMirror sm = (StructMirror) obj;
 				if (sm.Type.FullName == "System.IntPtr")
@@ -1186,8 +1436,10 @@ namespace Mono.Debugging.Soft
 					handle = ((TypeMirror)obj).BeginInvokeMethod (ctx.Thread, function, args, options, null, null);
 				else if (obj is StructMirror)
 					handle = ((StructMirror)obj).BeginInvokeMethod (ctx.Thread, function, args, options, null, null);
+				else if (obj is PrimitiveValue)
+					handle = ((PrimitiveValue)obj).BeginInvokeMethod (ctx.Thread, function, args, options, null, null);
 				else
-					throw new ArgumentException (obj.GetType ().ToString ());
+					throw new ArgumentException ("Soft debugger method calls cannot be invoked on objects of type " + obj.GetType ().Name);
 			} catch (InvocationException ex) {
 				ctx.Session.StackVersion++;
 				exception = ex;
@@ -1222,8 +1474,10 @@ namespace Mono.Debugging.Soft
 					result = ((ObjectMirror)obj).EndInvokeMethod (handle);
 				else if (obj is TypeMirror)
 					result = ((TypeMirror)obj).EndInvokeMethod (handle);
-				else
+				else if (obj is StructMirror)
 					result = ((StructMirror)obj).EndInvokeMethod (handle);
+				else
+					result = ((PrimitiveValue)obj).EndInvokeMethod (handle);
 			} catch (InvocationException ex) {
 				if (!Aborting && ex.Exception != null) {
 					string ename = ctx.Adapter.GetValueTypeName (ctx, ex.Exception);
