@@ -56,8 +56,16 @@ namespace MonoDevelop.Debugger
 		{
 			this.editor = editor;
 		}
-		
-		public void DrawIcon (Mono.TextEditor.TextEditor editor, Cairo.Context cr, LineSegment line, int lineNumber, double x, double y, double width, double height)
+
+
+		public override void Draw (TextEditor editor, Cairo.Context cr, Pango.Layout layout, bool selected, int startOffset, int endOffset, double y, double startXPos, double endXPos)
+		{
+			if (!(this is CurrentDebugLineTextMarker) && LineSegment.Markers.Any (m => m is CurrentDebugLineTextMarker))
+				return;
+			base.Draw (editor, cr, layout, selected, startOffset, endOffset, y, startXPos, endXPos);
+		}
+
+		public void DrawIcon (Mono.TextEditor.TextEditor editor, Cairo.Context cr, DocumentLine line, int lineNumber, double x, double y, double width, double height)
 		{
 			double size;
 			if (width > height) {
@@ -166,7 +174,7 @@ namespace MonoDevelop.Debugger
 			IncludedStyles |= StyleFlag.BackgroundColor | StyleFlag.Color;
 			IsTracepoint = isTracePoint;
 		}
-		
+
 		protected override void DrawIcon (Cairo.Context cr, double x, double y, double size)
 		{
 			Cairo.Color color1 = editor.ColorStyle.BreakpointMarkerColor1;
