@@ -37,9 +37,10 @@ using MonoDevelop.Ide;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom;
 using MonoDevelop.Projects.Text;
 using MonoDevelop.Debugger.Viewers;
+using ICSharpCode.NRefactory.TypeSystem;
+using ICSharpCode.NRefactory.Semantics;
 
 /*
  * Some places we should be doing some error handling we used to toss
@@ -69,7 +70,9 @@ namespace MonoDevelop.Debugger
 		static DebuggerSession session;
 		static Backtrace currentBacktrace;
 		static int currentFrame;
-
+		
+		static ExceptionCaughtDialog exceptionDialog;
+		
 		static BusyEvaluatorDialog busyDialog;
 		static bool isBusy;
 		static StatusBarIcon busyStatusIcon;
@@ -96,6 +99,8 @@ namespace MonoDevelop.Debugger
 				IdeApp.Workspace.LoadingUserPreferences += OnLoadUserPrefs;
 				IdeApp.Workspace.LastWorkspaceItemClosed += OnSolutionClosed;
 				busyDialog = new BusyEvaluatorDialog ();
+				busyDialog.TransientFor = MessageService.RootWindow;
+				busyDialog.DestroyWithParent = true;
 			};
 			AddinManager.AddExtensionNodeHandler (FactoriesPath, delegate {
 				// Regresh the engine list
@@ -279,12 +284,20 @@ namespace MonoDevelop.Debugger
 		
 		public static void ShowExceptionCaughtDialog ()
 		{
-			ExceptionInfo val = CurrentFrame.GetException ();
+			EvaluationOptions ops = session.EvaluationOptions.Clone ();
+			ops.MemberEvaluationTimeout = 0;
+			ops.EvaluationTimeout = 0;
+			ops.EllipsizeStrings = false;
+			
+			ExceptionInfo val = CurrentFrame.GetException (ops);
 			if (val != null) {
-				ExceptionCaughtDialog dlg = new ExceptionCaughtDialog (val);
-				dlg.TransientFor = IdeApp.Workbench.RootWindow;
-				MessageService.PlaceDialog (dlg, IdeApp.Workbench.RootWindow);
-				dlg.Show ();
+				exceptionDialog = new ExceptionCaughtDialog (val);
+				exceptionDialog.TransientFor = IdeApp.Workbench.RootWindow;
+				MessageService.PlaceDialog (exceptionDialog, IdeApp.Workbench.RootWindow);
+				exceptionDialog.Destroyed += (o, args) => {
+					exceptionDialog = null;
+				};
+				exceptionDialog.Show ();
 			}
 		}
 
@@ -335,7 +348,12 @@ namespace MonoDevelop.Debugger
 			
 			if (!IsDebugging)
 				return;
-
+			
+			if (exceptionDialog != null) {
+				exceptionDialog.Destroy ();
+				exceptionDialog = null;
+			}
+			
 			if (busyStatusIcon != null) {
 				busyStatusIcon.Dispose ();
 				busyStatusIcon = null;
@@ -432,23 +450,26 @@ namespace MonoDevelop.Debugger
 		
 		public static DebuggerSessionOptions GetUserOptions ()
 		{
-			EvaluationOptions eops = EvaluationOptions.DefaultOptions;
-			eops.AllowTargetInvoke = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.AllowTargetInvoke", true);
-			eops.AllowToStringCalls = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.AllowToStringCalls", true);
-			eops.EvaluationTimeout = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.EvaluationTimeout", 2500);
-			eops.FlattenHierarchy = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.FlattenHierarchy", false);
-			eops.GroupPrivateMembers = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.GroupPrivateMembers", true);
-			eops.GroupStaticMembers = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.GroupStaticMembers", true);
-			eops.MemberEvaluationTimeout = eops.EvaluationTimeout * 2;
+			EvaluationOptions eval = EvaluationOptions.DefaultOptions;
+			eval.AllowTargetInvoke = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.AllowTargetInvoke", true);
+			eval.AllowToStringCalls = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.AllowToStringCalls", true);
+			eval.EvaluationTimeout = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.EvaluationTimeout", 2500);
+			eval.FlattenHierarchy = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.FlattenHierarchy", false);
+			eval.GroupPrivateMembers = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.GroupPrivateMembers", true);
+			eval.GroupStaticMembers = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.GroupStaticMembers", true);
+			eval.MemberEvaluationTimeout = eval.EvaluationTimeout * 2;
 			return new DebuggerSessionOptions () {
+				StepOverPropertiesAndOperators = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.StepOverPropertiesAndOperators", true),
 				ProjectAssembliesOnly = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.ProjectAssembliesOnly", true),
-				EvaluationOptions = eops,
+				EvaluationOptions = eval,
 			};
 		}
 		
 		public static void SetUserOptions (DebuggerSessionOptions options)
 		{
+			PropertyService.Set ("MonoDevelop.Debugger.DebuggingService.StepOverPropertiesAndOperators", options.StepOverPropertiesAndOperators);
 			PropertyService.Set ("MonoDevelop.Debugger.DebuggingService.ProjectAssembliesOnly", options.ProjectAssembliesOnly);
+			
 			PropertyService.Set ("MonoDevelop.Debugger.DebuggingService.AllowTargetInvoke", options.EvaluationOptions.AllowTargetInvoke);
 			PropertyService.Set ("MonoDevelop.Debugger.DebuggingService.AllowToStringCalls", options.EvaluationOptions.AllowToStringCalls);
 			PropertyService.Set ("MonoDevelop.Debugger.DebuggingService.EvaluationTimeout", options.EvaluationOptions.EvaluationTimeout);
@@ -549,7 +570,7 @@ namespace MonoDevelop.Debugger
 						busyStatusIcon.SetAlertMode (100);
 						busyStatusIcon.ToolTip = GettextCatalog.GetString ("The Debugger is waiting for an expression evaluation to finish.");
 						busyStatusIcon.EventBox.ButtonPressEvent += delegate {
-							busyDialog.Show ();
+							MessageService.PlaceDialog (busyDialog, MessageService.RootWindow);
 						};
 					}
 				} else {
@@ -564,7 +585,7 @@ namespace MonoDevelop.Debugger
 		static bool CheckIsBusy ()
 		{
 			if (isBusy && !busyDialog.Visible)
-				busyDialog.Show ();
+				MessageService.PlaceDialog (busyDialog, MessageService.RootWindow);
 			return isBusy;
 		}
 		
@@ -874,13 +895,13 @@ namespace MonoDevelop.Debugger
 			if (doc != null) {
 				ITextEditorResolver textEditorResolver = doc.GetContent <ITextEditorResolver> ();
 				if (textEditorResolver != null) {
-					ResolveResult rr = textEditorResolver.GetLanguageItem (doc.Editor.Document.LocationToOffset (location.Line, 1), identifier);
-					NamespaceResolveResult ns = rr as NamespaceResolveResult;
+					var rr = textEditorResolver.GetLanguageItem (doc.Editor.Document.LocationToOffset (location.Line, 1), identifier);
+					var ns = rr as NamespaceResolveResult;
 					if (ns != null)
-						return ns.Namespace;
-					MemberResolveResult result = rr as MemberResolveResult;
-					if (result != null && (result.ResolvedMember == null || result.ResolvedMember is IType) && result.ResolvedType != null)
-						return result.ResolvedType.FullName;
+						return ns.NamespaceName;
+					var result = rr as TypeResolveResult;
+					if (result != null && !result.IsError)
+						return result.Type.FullName;
 				}
 			}
 			return null;
@@ -944,8 +965,7 @@ namespace MonoDevelop.Debugger
 	internal class GtkConnectionDialog : IConnectionDialog
 	{
 		bool disposed;
-		Gtk.Dialog dialog;
-		Gtk.Label label;
+		System.Threading.CancellationTokenSource cts;
 		
 		public event EventHandler UserCancelled;
 		
@@ -955,26 +975,16 @@ namespace MonoDevelop.Debugger
 
 		public void SetMessage (DebuggerStartInfo dsi, string message, bool listening, int attemptNumber)
 		{
-			if (disposed)
+			//FIXME: we don't support changing the message
+			if (disposed || cts != null)
 				return;
 			
-			if (string.IsNullOrEmpty (message))
-				message = DefaultListenMessage;
+			cts = new System.Threading.CancellationTokenSource ();
 			
-			if (dialog == null) {
-				Gtk.Application.Invoke (delegate {
-					if (disposed)
-						return;
-					RunDialog (message);
-				});
-			} else {
-				Gtk.Application.Invoke (delegate {
-					if (disposed)
-						return;
-					if (label != null)
-						label.Text = message;
-				});
-			}
+			//MessageService is threadsafe but we want this to be async
+			Gtk.Application.Invoke (delegate {
+				RunDialog (message);
+			});
 		}
 		
 		void RunDialog (string message)
@@ -982,24 +992,29 @@ namespace MonoDevelop.Debugger
 			if (disposed)
 				return;
 			
-			dialog = new Gtk.Dialog () {
-				Title = "Waiting for debugger"
-			};
+			string title;
 			
-			var label = new Gtk.Alignment (0.5f, 0.5f, 1f, 1f) {
-				Child = new Gtk.Label (message),
-				BorderWidth = 12
-			};
-			dialog.VBox.PackStart (label);
-			label.ShowAll ();
+			if (message == null) {
+				title = GettextCatalog.GetString ("Waiting for debugger");
+			} else {
+				message = message.Trim ();
+				int i = message.IndexOfAny (new char[] { '\n', '\r' });
+				if (i > 0) {
+					title = message.Substring (0, i).Trim ();
+					message = message.Substring (i).Trim ();
+				} else {
+					title = message;
+					message = null;
+				}
+			}
+
+			var gm = new GenericMessage (title, message, cts.Token);
+			gm.Buttons.Add (AlertButton.Cancel);
+			gm.DefaultButton = 0;
+			MessageService.GenericAlert (gm);
+			cts = null;
 			
-			dialog.AddButton ("Cancel", Gtk.ResponseType.Cancel);
-			
-			int response = MonoDevelop.Ide.MessageService.ShowCustomDialog (dialog);
-			dialog.Destroy ();
-			dialog = null;
-			
-			if (!disposed && response != (int) Gtk.ResponseType.Ok && UserCancelled != null) {
+			if (!disposed && UserCancelled != null) {
 				UserCancelled (null, null);
 			}
 		}
@@ -1009,13 +1024,9 @@ namespace MonoDevelop.Debugger
 			if (disposed)
 				return;
 			disposed = true;
-			
-			if (dialog != null) {
-				Gtk.Application.Invoke (delegate {
-					if (dialog != null)
-						dialog.Respond (Gtk.ResponseType.Ok);
-				});
-			}
+			var c = cts;
+			if (c != null)
+				c.Cancel ();
 		}
 	}
 }
