@@ -273,7 +273,7 @@ namespace MonoDevelop.Debugger
 			if (lastLine != int.MinValue && loc2.Line >= editor.Document.LineCount - FillMarginLines) {
 				int num = (loc2.Line - (editor.Document.LineCount - FillMarginLines) + 1) * 2;
 				int newFirst;
-				InsertLines (editor.Document.Length, lastLine + 1, lastLine + num, out newFirst, out lastLine);
+				InsertLines (editor.Document.TextLength, lastLine + 1, lastLine + num, out newFirst, out lastLine);
 			}
 		}
 		
