@@ -121,6 +121,7 @@ namespace MonoDevelop.Debugger
 			menuSet = new CommandEntrySet ();
 			menuSet.AddItem (DebugCommands.AddWatch);
 			menuSet.AddSeparator ();
+			menuSet.AddItem (EditCommands.Copy);
 			menuSet.AddItem (EditCommands.Rename);
 			menuSet.AddItem (EditCommands.DeleteKey);
 		}
@@ -163,7 +164,7 @@ namespace MonoDevelop.Debugger
 			valueCol.PackStart (crpViewer, false);
 			valueCol.AddAttribute (crpViewer, "visible", ViewerButtonVisibleCol);
 			crpButton = new CellRendererIcon ();
-			crpButton.StockSize = (uint) Gtk.IconSize.Menu;
+			crpButton.StockSize = (uint)Gtk.IconSize.Menu;
 			crpButton.IconId = Gtk.Stock.Refresh;
 			valueCol.PackStart (crpButton, false);
 			valueCol.AddAttribute (crpButton, "visible", ValueButtonVisibleCol);
@@ -215,10 +216,17 @@ namespace MonoDevelop.Debugger
 			this.EnableAutoTooltips ();
 			
 			createMsg = GettextCatalog.GetString ("Click here to add a new watch");
+			CompletionWindowManager.WindowClosed += HandleCompletionWindowClosed;
+		}
+
+		void HandleCompletionWindowClosed (object sender, EventArgs e)
+		{
+			currentCompletionData = null;
 		}
 
 		protected override void OnDestroyed ()
 		{
+			CompletionWindowManager.WindowClosed -= HandleCompletionWindowClosed;
 			crtExp.Edited -= OnExpEdited;
 			crtExp.EditingStarted -= OnExpEditing;
 			crtExp.EditingCanceled -= OnEditingCancelled;
@@ -406,6 +414,7 @@ namespace MonoDevelop.Debugger
 		
 		public void RemoveExpression (string exp)
 		{
+			cachedValues.Remove (exp);
 			valueNames.Remove (exp);
 			Refresh ();
 		}
@@ -842,11 +851,14 @@ namespace MonoDevelop.Debugger
 				string exp = (string) store.GetValue (it, NameCol);
 				if (args.NewText == exp)
 					return;
+				
 				int i = valueNames.IndexOf (exp);
 				if (args.NewText.Length != 0)
 					valueNames [i] = args.NewText;
 				else
 					valueNames.RemoveAt (i);
+				
+				cachedValues.Remove (exp);
 				Refresh ();
 			}
 		}
@@ -936,21 +948,20 @@ namespace MonoDevelop.Debugger
 			Gtk.Entry entry = (Gtk.Entry)s;
 			
 			if (currentCompletionData != null) {
-				KeyActions ka;
-				bool ret = CompletionWindowManager.PreProcessKeyEvent (args.Event.Key, (char)args.Event.Key, args.Event.State, out ka);
-				CompletionWindowManager.PostProcessKeyEvent (ka, args.Event.Key, (char)args.Event.Key, args.Event.State);
+				bool ret = CompletionWindowManager.PreProcessKeyEvent (args.Event.Key, (char)args.Event.Key, args.Event.State);
+				CompletionWindowManager.PostProcessKeyEvent (args.Event.Key, (char)args.Event.Key, args.Event.State);
 				args.RetVal = ret;
 			}
 			
 			Gtk.Application.Invoke (delegate {
-				char c = (char) Gdk.Keyval.ToUnicode (args.Event.KeyValue);
+				char c = (char)Gdk.Keyval.ToUnicode (args.Event.KeyValue);
 				if (currentCompletionData == null && IsCompletionChar (c)) {
 					string exp = entry.Text.Substring (0, entry.CursorPosition);
 					currentCompletionData = GetCompletionData (exp);
 					if (currentCompletionData != null) {
 						DebugCompletionDataList dataList = new DebugCompletionDataList (currentCompletionData);
 						CodeCompletionContext ctx = ((ICompletionWidget)this).CreateCodeCompletionContext (entry.CursorPosition - currentCompletionData.ExpressionLenght);
-						CompletionWindowManager.ShowWindow (c, dataList, this, ctx, OnCompletionWindowClosed);
+						CompletionWindowManager.ShowWindow (null, c, dataList, this, ctx);
 					} else
 						currentCompletionData = null;
 				}
@@ -1063,6 +1074,29 @@ namespace MonoDevelop.Debugger
 			IdeApp.CommandService.ShowContextMenu (this, evt, menuSet, this);
 		}
 		
+		[CommandHandler (EditCommands.Copy)]
+		protected void OnCopy ()
+		{
+			TreePath[] selected = Selection.GetSelectedRows ();
+			TreeIter iter;
+			
+			if (selected == null || selected.Length != 1)
+				return;
+			
+			if (!store.GetIter (out iter, selected[0]))
+				return;
+
+			object focus = IdeApp.Workbench.RootWindow.Focus;
+
+			if (focus is Gtk.Editable) {
+				((Gtk.Editable) focus).CopyClipboard ();
+				return;
+			}
+			
+			string value = (string) store.GetValue (iter, ValueCol);
+			Clipboard.Get (Gdk.Selection.Clipboard).Text = value;
+		}
+		
 		[CommandHandler (EditCommands.Delete)]
 		[CommandHandler (EditCommands.DeleteKey)]
 		protected void OnDelete ()
@@ -1071,9 +1105,8 @@ namespace MonoDevelop.Debugger
 				TreeIter it;
 				if (store.GetIter (out it, tp)) {
 					string exp = (string) store.GetValue (it, NameCol);
-					int i = valueNames.IndexOf (exp);
-					if (i != -1)
-						valueNames.RemoveAt (i);
+					cachedValues.Remove (exp);
+					valueNames.Remove (exp);
 				}
 			}
 			Refresh ();
@@ -1221,11 +1254,7 @@ namespace MonoDevelop.Debugger
 			return (char.IsLetterOrDigit (c) || char.IsPunctuation (c) || char.IsSymbol (c) || char.IsWhiteSpace (c));
 		}
 		
-		void OnCompletionWindowClosed ()
-		{
-			currentCompletionData = null;
-		}
-		
+
 		#region ICompletionWidget implementation 
 		
 		CodeCompletionContext ICompletionWidget.CurrentCodeCompletionContext {
@@ -1256,6 +1285,12 @@ namespace MonoDevelop.Debugger
 				editEntry.Text = editEntry.Text.Insert (offset, text);
 		}
 		
+		int ICompletionWidget.CaretOffset {
+			get {
+				return editEntry.Position;
+			}
+		}
+		
 		char ICompletionWidget.GetChar (int offset)
 		{
 			string txt = editEntry.Text;
@@ -1375,7 +1410,7 @@ namespace MonoDevelop.Debugger
 		}
 	}
 	
-	class DebugCompletionDataList: List<MonoDevelop.Ide.CodeCompletion.CompletionData>, ICompletionDataList
+	class DebugCompletionDataList: List<ICSharpCode.NRefactory.Completion.ICompletionData>, ICompletionDataList
 	{
 		public bool IsSorted { get; set; }
 		public DebugCompletionDataList (Mono.Debugging.Client.CompletionData data)
@@ -1398,7 +1433,13 @@ namespace MonoDevelop.Debugger
 		public bool AutoCompleteEmptyMatch {
 			get { return false; }
 		}
-
+		
+		public bool CloseOnSquareBrackets {
+			get {
+				return false;
+			}
+		}
+		
 		public CompletionSelectionMode CompletionSelectionMode {
 			get;
 			set;
