@@ -1,4 +1,4 @@
-// 
+﻿// 
 // DocGenerator.cs
 //  
 // Author:
@@ -30,11 +30,9 @@ using System.Text;
 using System.Xml;
 using Mono.TextEditor;
 using MonoDevelop.Core;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.Dom.Output;
 using MonoDevelop.Refactoring;
-
-
+using ICSharpCode.NRefactory.TypeSystem;
+using MonoDevelop.Ide.TypeSystem;
 
 namespace MonoDevelop.DocFood
 {
@@ -42,36 +40,33 @@ namespace MonoDevelop.DocFood
 	{
 		public List<Section> sections = new List<Section> ();
 		public Dictionary<string, string> tags = new Dictionary<string, string> ();
-		TextEditorData data;
-		INRefactoryASTProvider provider;
-
+//		TextEditorData data;
+		
 		public DocGenerator ()
 		{
 			
 		}
-
+		
 		public DocGenerator (TextEditorData data)
 		{
-			this.data = data;
-			if (data != null)
-				provider = RefactoringService.GetASTProvider (data.Document.MimeType);
+//			this.data = data;
 		}
 		
-		public static string GetBaseDocumentation (IMember member)
+		public static string GetBaseDocumentation (IEntity member)
 		{
-			if (member.DeclaringType == null || member.DeclaringType.SourceProjectDom == null)
+			if (member.DeclaringTypeDefinition == null)
 				return null;
-			if (member is IMethod && (((IMethod)member).IsConstructor || ((IMethod)member).IsFinalizer))
+			if (member is IMethod && (((IMethod)member).IsConstructor || ((IMethod)member).IsDestructor))
 				return null;
-			foreach (IType type in member.DeclaringType.SourceProjectDom.GetInheritanceTree (member.DeclaringType)) {
-				if (type.DecoratedFullName == member.DeclaringType.DecoratedFullName)
+			foreach (var type in member.DeclaringTypeDefinition.GetAllBaseTypeDefinitions ()) {
+				if (type.Equals (member.DeclaringTypeDefinition))
 					continue;
 				IMember documentMember = null;
-				foreach (IMember searchedMember in type.SearchMember (member.Name, true)) {
-					if (searchedMember.MemberType == member.MemberType && searchedMember.Name == member.Name && searchedMember.CanHaveParameters == member.CanHaveParameters) {
-						if (searchedMember.CanHaveParameters && searchedMember.Parameters.Count != member.Parameters.Count)
+				foreach (var searchedMember in type.Members.Where (m => m.Name == member.Name)) {
+					if (searchedMember.EntityType == member.EntityType && searchedMember.Name == member.Name) {
+						if ((searchedMember is IParameterizedMember) && ((IParameterizedMember)searchedMember).Parameters.Count != ((IParameterizedMember)member).Parameters.Count)
 							continue;
-						if (searchedMember.Modifiers != member.Modifiers)
+						if (searchedMember.Accessibility != member.Accessibility)
 							continue;
 						documentMember = searchedMember;
 						break;
@@ -127,43 +122,43 @@ namespace MonoDevelop.DocFood
 			}
 		}
 
-		public IBaseMember member;
-		MemberVisitor visitor = new MemberVisitor ();
+		public INamedElement member;
+//		MemberVisitor visitor = new MemberVisitor ();
 		string currentType;
 		int wordCount;
 		
-		static string GetType (IBaseMember member)
+		static string GetType (IEntity member)
 		{
-			switch (member.MemberType) {
-			case MemberType.Event:
+			switch (member.EntityType) {
+			case EntityType.Event:
 				return "event";
-			case MemberType.Field:
+			case EntityType.Field:
 				return "field";
-			case MemberType.Method:
-				if (((IMethod)member).IsConstructor)
-					return "constructor";
-				if (((IMethod)member).IsFinalizer)
-					return "destructor";
-				if (((IMethod)member).IsSpecialName && GetOperator (member.Name) != null)
-					return "operator";
+			case EntityType.Constructor:
+				return "constructor";
+			case EntityType.Destructor:
+				return "destructor";
+			case EntityType.Operator:
+				return "operator";
+			case EntityType.Method:
 				return "method";
-			case MemberType.Parameter:
-				return "parameter";
-			case MemberType.Property:
-				if (((IProperty)member).IsIndexer)
-					return "indexer";
+//			case MemberType.Parameter:
+//				return "parameter";
+			case EntityType.Indexer:
+				return "indexer";
+			case EntityType.Property:
 				return "property";
-			case MemberType.Type:
-				switch (((IType)member).ClassType) {
-				case ClassType.Class:
+			case EntityType.TypeDefinition:
+				switch (((ITypeDefinition)member).Kind) {
+				case TypeKind.Class:
 					return "class";
-				case ClassType.Delegate:
+				case TypeKind.Delegate:
 					return "delegate";
-				case ClassType.Enum:
+				case TypeKind.Enum:
 					return "enum";
-				case ClassType.Interface:
+				case TypeKind.Interface:
 					return "interface";
-				case ClassType.Struct:
+				case TypeKind.Struct:
 					return "struct";
 				}
 				break;
@@ -242,31 +237,33 @@ namespace MonoDevelop.DocFood
 						break;
 					case "modifier":
 						if (member is IMember) {
-							var mod = (Modifiers)Enum.Parse (typeof(Modifiers), val);
-							result |=  (mod & ((IMember)member).Modifiers) == mod;
+							try {
+								var mod = (Accessibility)Enum.Parse (typeof(Accessibility), val);
+								result |=  ((IMember)member).Accessibility == mod;
+							} catch (Exception) {
+							}
 						}
 						break;
 					case "paramCount":
-						if (member is IMember)
-							result |= Int32.Parse (val) == ((IMember)member).Parameters.Count;
+						if (member is IParameterizedMember)
+							result |= Int32.Parse (val) == ((IParameterizedMember)member).Parameters.Count;
 						break;
 					case "parameter":
-						if (!(member is IMember))
+						if (!(member is IParameterizedMember))
 							break;
 						string[] par = val.Split(':');
 						int idx = Int32.Parse (par[0]);
 						string name = par[1];
-						result |= idx < ((IMember)member).Parameters.Count && name == ((IMember)member).Parameters[idx].Name;
+						result |= idx < ((IParameterizedMember)member).Parameters.Count && name == ((IParameterizedMember)member).Parameters[idx].Name;
 						break;
 					case "returns":
-						if (member == null || member.ReturnType == null)
+						if ((member as IMember) == null)
 							break;
-						result |= val == member.ReturnType.DecoratedFullName;
+						result |= val == ((IMember)member).ReturnType.ToString ();
 						break;
 					case "name":
-
 						IMethod method = member as IMethod;
-						if (method != null && method.IsSpecialName) {
+						if (method != null && method.IsSynthetic) {
 							string op = GetOperator (method.Name);
 							if (op != null) {
 								result |= val == op;
@@ -305,7 +302,7 @@ namespace MonoDevelop.DocFood
 		}
 		
 		internal string curName;
-		public void GenerateDoc (IMember member)
+		public void GenerateDoc (IEntity member)
 		{
 			Init (member);
 			
@@ -313,9 +310,9 @@ namespace MonoDevelop.DocFood
 			this.currentType = GetType (member);
 			DocConfig.Instance.Rules.ForEach (r => r.Run (this));
 			
-			if (member.CanHaveParameters) {
+			if (member is IParameterizedMember) {
 				this.currentType = "parameter";
-				foreach (IParameter p in member.Parameters) {
+				foreach (var p in ((IParameterizedMember)member).Parameters) {
 					curName = p.Name;
 					this.member = member;
 					SplitWords (p, p.Name);
@@ -349,82 +346,83 @@ namespace MonoDevelop.DocFood
 				}
 			}
 			
-			IType type;
-			if (member is IType) {
-				type = (IType)member;
-			} else {
-				type = member.DeclaringType;
-			}
+//			ITypeDefinition type;
+//			if (member is ITypeDefinition) {
+//				type = (ITypeDefinition)member;
+//			} else {
+//				type = ((IMember)member).DeclaringTypeDefinition;
+//			}
 			
-			this.currentType = "exception";
-			foreach (var exception in visitor.Exceptions) {
-				var exceptionType = MonoDevelop.Refactoring.HelperMethods.ConvertToReturnType (exception);
-				
-				
-				curName = exceptionType.FullName;
-				tags["Exception"] = exceptionType.ToInvariantString ();
-				SplitWords (exceptionType, exceptionType.Name);
-				
-				if (type != null) {
-					IType resolvedType = type.SourceProjectDom.SearchType (type.CompilationUnit, type, type.Location, exceptionType);
-					string sentence = AmbienceService.GetDocumentationSummary (resolvedType);
-					if (! string.IsNullOrEmpty(sentence)) {
-						sentence = sentence.Trim ();
-						if (sentence.StartsWith ("<para>") && sentence.EndsWith ("</para>"))
-							sentence = sentence.Substring ("<para>".Length, sentence.Length - "<para>".Length - "</para>".Length).Trim ();
-						if (sentence.StartsWith ("Represents the error that occurs when"))
-							sentence = "Is thrown when" + sentence.Substring ("Represents the error that occurs when".Length);
-						if (!string.IsNullOrEmpty (sentence))
-							Set ("exception", curName, sentence);
-					}
-				}
-				
-				DocConfig.Instance.Rules.ForEach (r => r.Run (this));
-			}
+// TODO: Exceptions!
+//			this.currentType = "exception";
+//			foreach (var exception in visitor.Exceptions) {
+//				var exceptionType = MonoDevelop.Refactoring.HelperMethods.ConvertToReturnType (exception);
+//				
+//				
+//				curName = exceptionType.FullName;
+//				tags["Exception"] = exceptionType.ToInvariantString ();
+//				SplitWords (exceptionType, exceptionType.Name);
+//				
+//				if (type != null) {
+//					IType resolvedType = type.GetProjectContent ().SearchType (type.CompilationUnit, type, type.Location, exceptionType);
+//					string sentence = AmbienceService.GetDocumentationSummary (resolvedType);
+//					if (! string.IsNullOrEmpty(sentence)) {
+//						sentence = sentence.Trim ();
+//						if (sentence.StartsWith ("<para>") && sentence.EndsWith ("</para>"))
+//							sentence = sentence.Substring ("<para>".Length, sentence.Length - "<para>".Length - "</para>".Length).Trim ();
+//						if (sentence.StartsWith ("Represents the error that occurs when"))
+//							sentence = "Is thrown when" + sentence.Substring ("Represents the error that occurs when".Length);
+//						if (!string.IsNullOrEmpty (sentence))
+//							Set ("exception", curName, sentence);
+//					}
+//				}
+//				
+//				DocConfig.Instance.Rules.ForEach (r => r.Run (this));
+//			}
 		}
 		
-		void Init (IMember member)
+		void Init (IEntity member)
 		{
 			FillDocumentation (GetBaseDocumentation (member));
-			if (provider != null && !member.Location.IsEmpty && member.BodyRegion.End.Line > 1) {
-				LineSegment start = data.Document.GetLine (member.Location.Line);
-				LineSegment end = data.Document.GetLine (member.BodyRegion.End.Line);
-				if (start != null && end != null) {
-					var result = provider.ParseFile ("class A {" + data.Document.GetTextAt (start.Offset, end.EndOffset - start.Offset) + "}");
-					result.AcceptVisitor (visitor, null);
-				}
-			}
+			//			if (provider != null && !member.Location.IsEmpty && member.BodyRegion.EndLine > 1) {
+			//				LineSegment start = data.Document.GetLine (member.Region.BeginLine);
+			//				LineSegment end = data.Document.GetLine (member.BodyRegion.EndLine);
+			//				if (start != null && end != null) {
+			//					var result = provider.ParseFile ("class A {" + data.Document.GetTextAt (start.Offset, end.EndOffset - start.Offset) + "}");
+			//					result.AcceptVisitor (visitor, null);
+			//				}
+			//			}
 			foreach (var macro in DocConfig.Instance.Macros) {
 				tags.Add (macro.Key, macro.Value);
 			}
-			if (member.DeclaringType != null) {
-				tags["DeclaringType"] = "<see cref=\"" + member.DeclaringType.DecoratedFullName + "\"/>";
-				switch (member.DeclaringType.ClassType) {
-				case ClassType.Class:
-					tags["DeclaringTypeKind"] = "class";
+			if (member.DeclaringTypeDefinition != null) {
+				tags ["DeclaringType"] = "<see cref=\"" + member.DeclaringTypeDefinition.ReflectionName + "\"/>";
+				switch (member.DeclaringTypeDefinition.Kind) {
+				case TypeKind.Class:
+					tags ["DeclaringTypeKind"] = "class";
 					break;
-				case ClassType.Delegate:
-					tags["DeclaringTypeKind"] = "delegate";
+				case TypeKind.Delegate:
+					tags ["DeclaringTypeKind"] = "delegate";
 					break;
-				case ClassType.Enum:
-					tags["DeclaringTypeKind"] = "enum";
+				case TypeKind.Enum:
+					tags ["DeclaringTypeKind"] = "enum";
 					break;
-				case ClassType.Interface:
-					tags["DeclaringTypeKind"] = "interface";
+				case TypeKind.Interface:
+					tags ["DeclaringTypeKind"] = "interface";
 					break;
-				case ClassType.Struct:
-					tags["DeclaringTypeKind"] = "struct";
+				case TypeKind.Struct:
+					tags ["DeclaringTypeKind"] = "struct";
 					break;
 				}
 			}
-			if (member.ReturnType != null)
-				tags["ReturnType"] = member.ReturnType != null ? "<see cref=\"" + member.ReturnType.ToInvariantString () + "\"/>" : "";
-			tags["Member"] = "<see cref=\"" + member.Name+ "\"/>";
+			if (member is IMember)
+				tags ["ReturnType"] = ((IMember)member).ReturnType != null ? "<see cref=\"" + ((IMember)member).ReturnType + "\"/>" : "";
+			tags ["Member"] = "<see cref=\"" + member.Name + "\"/>";
 
 			
-			if (member.CanHaveParameters) {
-				List<string> parameterNames = new List<string> (from p in member.Parameters select p.Name);
-				tags["ParameterSentence"] = string.Join (" ", parameterNames.ToArray ());
+			if (member is IParameterizedMember) {
+				List<string> parameterNames = new List<string> (from p in ((IParameterizedMember)member).Parameters select p.Name);
+				tags ["ParameterSentence"] = string.Join (" ", parameterNames.ToArray ());
 				StringBuilder paramList = new StringBuilder ();
 				for (int i = 0; i < parameterNames.Count; i++) {
 					if (i > 0) {
@@ -434,22 +432,28 @@ namespace MonoDevelop.DocFood
 							paramList.Append (", ");
 						}
 					}
-					paramList.Append (parameterNames[i]);
+					paramList.Append (parameterNames [i]);
 				}
-				tags["ParameterList"] = paramList.ToString ();
-				for (int i = 0; i < member.Parameters.Count; i++) {
-					tags["Parameter" + i +  ".Type"] = member.Parameters[i].ReturnType != null ? "<see cref=\"" + member.Parameters[i].ReturnType.ToInvariantString () + "\"/>" : "";
-					tags["Parameter" + i +  ".Name"] = "<c>" + member.Parameters[i].Name + "</c>";
+				tags ["ParameterList"] = paramList.ToString ();
+				for (int i = 0; i < ((IParameterizedMember)member).Parameters.Count; i++) {
+					tags ["Parameter" + i + ".Type"] = ((IParameterizedMember)member).Parameters [i].Type != null ? "<see cref=\"" + ((IParameterizedMember)member).Parameters [i].Type + "\"/>" : "";
+					tags ["Parameter" + i + ".Name"] = "<c>" + ((IParameterizedMember)member).Parameters [i].Name + "</c>";
 				}
 				
-				IProperty property = member as IProperty;
+				var property = member as IProperty;
 				if (property != null) {
-					if (property.HasGet && property.HasSet) {
-						tags["AccessText"] = "Gets or sets";
-					} else if (property.HasGet) {
-						tags["AccessText"] = "Gets";
+					if (property.CanGet && property.CanSet && property.Getter.Accessibility != Accessibility.Private && property.Setter.Accessibility != Accessibility.Private) {
+						tags ["AccessText"] = "Gets or sets";
+					} else if (property.CanGet && property.Getter.Accessibility != Accessibility.Private) {
+						tags ["AccessText"] = "Gets";
+					} else if (property.Setter.Accessibility != Accessibility.Private) {
+						tags ["AccessText"] = "Sets";
+					} else if (property.CanGet && property.CanSet) {
+						tags ["AccessText"] = "Gets or sets";
+					} else if (property.CanGet) {
+						tags ["AccessText"] = "Gets";
 					} else {
-						tags["AccessText"] = "Sets";
+						tags ["AccessText"] = "Sets";
 					}
 				}
 			}
@@ -535,7 +539,7 @@ namespace MonoDevelop.DocFood
 		public void Set (string name, string parameterName, string doc)
 		{
 			if (name.StartsWith ("param") && name.Length > "param".Length) {
-				parameterName = ((IMember)member).Parameters[int.Parse (name.Substring("param".Length))].Name;
+				parameterName = ((IParameterizedMember)member).Parameters[int.Parse (name.Substring("param".Length))].Name;
 				name = "param";
 			}
 			Section newSection = new Section (name);
