@@ -29,15 +29,9 @@
 using System;
 using Gtk;
 using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Core.Gui.ProgressMonitoring;
-using MonoDevelop.Ide.Gui;
-using MonoDevelop.Ide.Gui.Pads;
-using MonoDevelop.Ide.Gui.Pads.ProjectPad;
-using MonoDevelop.Components;
 using MonoDevelop.Components.Commands;
-using MonoDevelop.Projects;
 using MonoDevelop.Ide.Gui.Components;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.Gettext.NodeBuilders
 {
@@ -127,9 +121,7 @@ namespace MonoDevelop.Gettext.NodeBuilders
 				TranslationProject project = CurrentNode.DataItem as TranslationProject;
 				if (project == null)
 					return;
-				TranslationProjectOptionsDialog options = new TranslationProjectOptionsDialog (project);
-				options.TransientFor = IdeApp.Workbench.RootWindow;
-				options.Run ();
+				MessageService.ShowCustomDialog (new TranslationProjectOptionsDialog (project));
 				IdeApp.Workspace.Save ();
 			}
 			
@@ -149,14 +141,7 @@ namespace MonoDevelop.Gettext.NodeBuilders
 				string monitorTitle = GettextCatalog.GetString ("Translator Output");
 				Translator.LanguageChooserDialog chooser = new Translator.LanguageChooserDialog ();
 				try {
-					int response = 0;
-					chooser.Response += delegate(object o, ResponseArgs args) {
-						response = (int)args.ResponseId;
-					};
-					chooser.TransientFor = IdeApp.Workbench.RootWindow;
-					chooser.Run ();
-					
-					if (response == (int)ResponseType.Ok) {
+					if (MessageService.RunCustomDialog (chooser) == (int)ResponseType.Ok) {
 						string language = chooser.Language + (chooser.HasCountry ? "_" + chooser.Country : "");
 					
 						using (IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetOutputProgressMonitor (monitorTitle, "md-package", true, true)) {
