@@ -34,7 +34,6 @@ using System.CodeDom.Compiler;
 
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom.Parser;
 using MonoDevelop.Projects.Text;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
@@ -42,6 +41,8 @@ using MonoDevelop.Deployment;
 using MonoDevelop.Projects.Policies;
 using MonoDevelop.Ide;
 using MonoDevelop.Ide.Gui.Content;
+using MonoDevelop.Ide.TypeSystem;
+
 
 namespace MonoDevelop.GtkCore.GuiBuilder
 {
@@ -329,18 +330,19 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				};
 				provider.GenerateCodeFromCompileUnit (cu, fileStream, options);
 				text = fileStream.ToString ();
-				text = FormatGeneratedFile (fileName, text, provider);
+				text = FormatGeneratedFile (fileName, text, project, provider);
 			}
 			
 			if (saveToFile)
 				File.WriteAllText (fileName, text);
-			
-			if (ProjectDomService.HasDom (project)) {
-				// Only update the parser database if the project is actually loaded in the IDE.
-				ProjectDomService.Parse (project, fileName, text);
-				if (saveToFile) 
-					FileService.NotifyFileChanged (fileName);
-			}
+			TypeSystemService.ParseFile (project, fileName);
+//			
+//			if (ProjectDomService.HasDom (project)) {
+//				// Only update the parser database if the project is actually loaded in the IDE.
+//				ProjectDomService.Parse (project, fileName, text);
+//				if (saveToFile) 
+//					FileService.NotifyFileChanged (fileName);
+//			}
 
 			return fileName;
 		}
@@ -480,10 +482,9 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 						timer.Trace ("Generating code for " + unit.Name);
 						provider.GenerateCodeFromCompileUnit (unit, sw, codeGeneratorOptions);
 						string content = sw.ToString ();
-						string eol = pol.GetEolMarker ();
-									
+								
 						timer.Trace ("Formatting code");
-						content = FormatGeneratedFile (fname, content, provider);
+						content = FormatGeneratedFile (fname, content, project, provider);
 						timer.Trace ("Writing code");
 						File.WriteAllText (fname, content);
 					} finally {
@@ -525,26 +526,27 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			return pfile.FilePath;
 		}
 		
-		static string FormatGeneratedFile (string file, string content, CodeDomProvider provider)
+		static string FormatGeneratedFile (string file, string content, Project project, CodeDomProvider provider)
 		{
 			content = StripHeaderAndBlankLines (content, provider);
-			
-			var pol = PolicyService.InvariantPolicies.Get<TextStylePolicy> ();
-			string eol = pol.GetEolMarker ();
-			if (Environment.NewLine != eol)
-				content = content.Replace (Environment.NewLine, eol);
-			
+
 			string mt = DesktopService.GetMimeTypeForUri (file);
 			var formatter = MonoDevelop.Ide.CodeFormatting.CodeFormatterService.GetFormatter (mt);
 			if (formatter != null)
 				content = formatter.FormatText (PolicyService.InvariantPolicies, content);
 			
+			// The project policies should be taken for generated files (windows git eol problem)
+			var pol = project.Policies.Get<TextStylePolicy> (DesktopService.GetMimeTypeForUri (file));
+			string eol = pol.GetEolMarker ();
+			if (Environment.NewLine != eol)
+				content = content.Replace (Environment.NewLine, eol);
+			
 			return content;
 		}
 		
 		static string StripHeaderAndBlankLines (string text, CodeDomProvider provider)
 		{
-			Mono.TextEditor.Document doc = new Mono.TextEditor.Document ();
+			Mono.TextEditor.TextDocument doc = new Mono.TextEditor.TextDocument ();
 			doc.Text = text;
 			int realStartLine = 0;
 			for (int i = 1; i <= doc.LineCount; i++) {
@@ -562,11 +564,11 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			if (provider is Microsoft.CSharp.CSharpCodeProvider) {
 				bool previousWasBlank = false;
 				for (int i = 1; i <= doc.LineCount; i++) {
-					Mono.TextEditor.LineSegment line = doc.GetLine (i);
+					Mono.TextEditor.DocumentLine line = doc.GetLine (i);
 					bool isBlank, isBracket;
 					CheckLine (doc, line, out isBlank, out isBracket);
-					if (isBlank && previousWasBlank && line.Length > 0) {
-						((Mono.TextEditor.IBuffer)doc).Remove (line.Offset, line.Length);
+					if (isBlank && previousWasBlank && line.LengthIncludingDelimiter > 0) {
+						doc.Remove (line.Offset, line.LengthIncludingDelimiter);
 						i--;
 					}
 					previousWasBlank = isBlank || isBracket;
@@ -574,14 +576,14 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			}
 			
 			int offset = doc.GetLine (realStartLine).Offset;
-			return doc.GetTextAt (offset, doc.Length - offset);
+			return doc.GetTextAt (offset, doc.TextLength - offset);
 		}
 
-		static void CheckLine (Mono.TextEditor.Document doc, Mono.TextEditor.LineSegment line, out bool isBlank, out bool isBracket)
+		static void CheckLine (Mono.TextEditor.TextDocument doc, Mono.TextEditor.DocumentLine line, out bool isBlank, out bool isBracket)
 		{
 			isBlank = true;
 			isBracket = false;
-			for (int i = 0; i < line.Length; i++) {
+			for (int i = 0; i < line.LengthIncludingDelimiter; i++) {
 				char c = doc.GetCharAt (line.Offset + i);
 				if (c == '{') {
 					isBracket = true;
