@@ -23,6 +23,7 @@
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
+
 using System;
 
 namespace MonoDevelop.MacDev.PlistEditor
@@ -31,5 +32,10 @@ namespace MonoDevelop.MacDev.PlistEditor
 	{
 		void SetPListContainer (PObjectContainer container);
 	}
+	
+	public interface IRawPListDisplayWidget : IPListDisplayWidget
+	{
+		bool ShowDescriptions { get; }
+		PListScheme Scheme { get; }
+	}
 }
-
