@@ -32,8 +32,10 @@ using System.Collections.Generic;
 using System.Text.RegularExpressions;
 
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.Dom.Parser;
+using ICSharpCode.NRefactory.TypeSystem;
+using ICSharpCode.NRefactory.TypeSystem.Implementation;
+using MonoDevelop.Ide.TypeSystem;
+
 using MonoDevelop.Ide;
 using MonoDevelop.Core;
 
@@ -43,43 +45,51 @@ namespace MonoDevelop.MacDev.ObjCIntegration
 	{
 		//static readonly Regex frameworkRegex = new Regex ("#import\\s+<([A-Z][A-Za-z]*)/([A-Z][A-Za-z]*)\\.h>", RegexOptions.Compiled);
 		static readonly Regex typeInfoRegex = new Regex ("@(interface|protocol)\\s+(\\w*)\\s*:\\s*(\\w*)", RegexOptions.Compiled);
-		static readonly Regex ibRegex = new Regex ("(- \\(IBAction\\)|IBOutlet)([^;]*);", RegexOptions.Compiled);
+		static readonly Regex ibRegex = new Regex ("(-\\s*\\(IBAction\\)|IBOutlet)\\s*([^;]*);", RegexOptions.Compiled);
 		static readonly char[] colonChar = { ':' };
 		static readonly char[] whitespaceChars = { ' ', '\t', '\n', '\r' };
 		static readonly char[] splitActionParamsChars = { ' ', '\t', '\n', '\r', '*', '(', ')' };
 		
-		readonly IReturnType nsobjectType, registerAttType, connectAttType, exportAttType, modelAttType,
+		readonly ITypeReference nsobjectType, registerAttType, connectAttType, exportAttType, modelAttType,
 			iboutletAttType, ibactionAttType;
 		
-		static Dictionary<ProjectDom,NSObjectProjectInfo> infos = new Dictionary<ProjectDom, NSObjectProjectInfo> ();
+		static Dictionary<TypeSystemService.ProjectContentWrapper,NSObjectProjectInfo> infos = new Dictionary<TypeSystemService.ProjectContentWrapper, NSObjectProjectInfo> ();
+
+		ITypeDefinition Resolve (TypeSystemService.ProjectContentWrapper dom, ITypeReference reference)
+		{
+			return reference.Resolve (dom.Compilation).GetDefinition ();
+		}
+		
+
+		static NSObjectInfoService ()
+		{
+			TypeSystemService.ProjectUnloaded += HandleDomUnloaded;
+		}
 		
 		public NSObjectInfoService (string wrapperRoot)
 		{
 			this.WrapperRoot = wrapperRoot;
-			string foundation = wrapperRoot + ".Foundation";
-			connectAttType = new DomReturnType (foundation, "ConnectAttribute");
-			exportAttType = new DomReturnType (foundation, "ExportAttribute");
-			iboutletAttType = new DomReturnType (foundation, "OutletAttribute");
-			ibactionAttType = new DomReturnType (foundation, "ActionAttribute");
-			registerAttType = new DomReturnType (foundation, "RegisterAttribute");
-			modelAttType = new DomReturnType (foundation, "ModelAttribute");
-			nsobjectType = new DomReturnType (foundation, "NSObject");
+			var typeNamespace = wrapperRoot + ".Foundation";
+			connectAttType = new GetClassTypeReference (typeNamespace, "ConnectAttribute");
+			exportAttType = new GetClassTypeReference (typeNamespace, "ExportAttribute");
+			iboutletAttType = new GetClassTypeReference (typeNamespace, "OutletAttribute");
+			ibactionAttType = new GetClassTypeReference (typeNamespace, "ActionAttribute");
+			registerAttType = new GetClassTypeReference (typeNamespace, "RegisterAttribute");
+			modelAttType = new GetClassTypeReference (typeNamespace, "ModelAttribute");
+			nsobjectType = new GetClassTypeReference (typeNamespace, "NSObject");
 		}
 		
 		public string WrapperRoot { get; private set; }
 		
-		public NSObjectProjectInfo GetProjectInfo (DotNetProject project)
+		public NSObjectProjectInfo GetProjectInfo (DotNetProject project, IAssembly lookinAssembly = null)
 		{
-			var dom = ProjectDomService.GetProjectDom (project);
-			if (dom == null)
-				return null;
-			
-			dom.ForceUpdate (true);
-			
-			return GetProjectInfo (dom);
+			var dom = TypeSystemService.GetProjectContentWrapper (project);
+			project.ReferenceAddedToProject += HandleDomReferencesUpdated;
+			project.ReferenceRemovedFromProject += HandleDomReferencesUpdated;
+			return GetProjectInfo (dom, lookinAssembly);
 		}
 		
-		public NSObjectProjectInfo GetProjectInfo (ProjectDom dom)
+		public NSObjectProjectInfo GetProjectInfo (TypeSystemService.ProjectContentWrapper dom, IAssembly lookinAssembly = null)
 		{
 			NSObjectProjectInfo info;
 			
@@ -87,24 +97,23 @@ namespace MonoDevelop.MacDev.ObjCIntegration
 				if (infos.TryGetValue (dom, out info))
 					return info;
 				
+				var nso = Resolve (dom, nsobjectType);
 				//only include DOMs that can resolve NSObject
-				var nso = dom.GetType (nsobjectType);
-				if (nso == null) {
-					infos[dom] = null;
+				if (nso == null || nso.Kind == TypeKind.Unknown)
 					return null;
-				}
 				
-				info = new NSObjectProjectInfo (dom, this);
+				info = new NSObjectProjectInfo (dom, this, lookinAssembly);
 				infos[dom] = info;
-				dom.Unloaded += HandleDomUnloaded;
-				dom.ReferencesUpdated += HandleDomReferencesUpdated;
 			}
 			return info;
 		}
 
-		static void HandleDomReferencesUpdated (object sender, EventArgs e)
+		static void HandleDomReferencesUpdated (object sender, ProjectReferenceEventArgs e)
 		{
-			var dom = (ProjectDom)sender;
+			var project = (DotNetProject)sender;
+			var dom = TypeSystemService.GetProjectContentWrapper (project);
+			if (dom == null)
+				return;
 			NSObjectProjectInfo info;
 			lock (infos) {
 				if (!infos.TryGetValue (dom, out info))
@@ -113,137 +122,150 @@ namespace MonoDevelop.MacDev.ObjCIntegration
 			info.SetNeedsUpdating ();
 		}
 
-		static void HandleDomUnloaded (object sender, EventArgs e)
+		static void HandleDomUnloaded (object sender, ProjectUnloadEventArgs e)
 		{
-			var dom = (ProjectDom)sender;
+			var project = e.Project as DotNetProject;
+			if (project == null)
+				return;
+			var dom = e.Wrapper;
+			if (dom == null)
+				return;
 			lock (infos) {
-				dom.Unloaded -= HandleDomUnloaded;
-				dom.ReferencesUpdated -= HandleDomReferencesUpdated;
+				project.ReferenceAddedToProject -= HandleDomReferencesUpdated;
+				project.ReferenceRemovedFromProject -= HandleDomReferencesUpdated;
 				infos.Remove (dom);
 			}
 		}
-		
-		internal IEnumerable<NSObjectTypeInfo> GetRegisteredObjects (ProjectDom dom)
+
+		internal IEnumerable<NSObjectTypeInfo> GetRegisteredObjects (TypeSystemService.ProjectContentWrapper dom, IAssembly assembly)
 		{
-			var nso = dom.GetType (nsobjectType);
-			
-			if (nso == null)
+			var nso = Resolve (dom, nsobjectType);
+			if (nso == null || nso.Kind == TypeKind.Unknown)
 				throw new Exception ("Could not get NSObject from type database");
 			
 			//FIXME: only emit this for the wrapper NS
-			yield return new NSObjectTypeInfo ("NSObject", nsobjectType.FullName, null, null, false, false, false);
-			
-			foreach (var type in dom.GetSubclasses (nso, false)) {
-				var info = ConvertType (dom, type);
-				if (info != null)
-					yield return info;
+//			yield return new NSObjectTypeInfo ("NSObject", nso.GetDefinition ().FullName, null, null, false, false, false);
+			int cnt = 0, infcnt=0, models=0;
+			nso = assembly.Compilation.Import (nso);
+			foreach (var contextType in assembly.GetAllTypeDefinitions ()) {
+				if (contextType.IsDerivedFrom (nso)) {
+					var info = ConvertType (dom, contextType);
+					if (info != null)
+						yield return info;
+				}
 			}
 		}
 		
-		NSObjectTypeInfo ConvertType (ProjectDom dom, IType type)
+		NSObjectTypeInfo ConvertType (TypeSystemService.ProjectContentWrapper dom, ITypeDefinition type)
 		{
 			string objcName = null;
 			bool isModel = false;
 			bool registeredInDesigner = true;
 			
-			foreach (var part in type.Parts) {
-				foreach (var att in part.Attributes) {
-					if (att.AttributeType.FullName == registerAttType.FullName) {
-						if (type.SourceProject != null) {
-							registeredInDesigner &=
-								MonoDevelop.DesignerSupport.CodeBehind.IsDesignerFile (part.CompilationUnit.FileName);
-						}
-						
-						//type registered with an explicit type name are up to the user to provide a valid name
-						// Note that the attribute now takes one *or* two parameters.
-						if (att.PositionalArguments.Count == 1 || att.PositionalArguments.Count == 2)
-							objcName = (string)((System.CodeDom.CodePrimitiveExpression)att.PositionalArguments[0]).Value;
-						//non-nested types in the root namespace have names accessible from obj-c
-						else if (string.IsNullOrEmpty (type.Namespace) && type.Name.IndexOf ('.') < 0)
-							objcName = type.Name;
+			foreach (var att in type.Attributes) {
+				var attType = att.AttributeType;
+				if (attType.Equals (Resolve (dom, registerAttType))) {
+					if (type.GetProjectContent () != null) {
+						registeredInDesigner &=
+							MonoDevelop.DesignerSupport.CodeBehind.IsDesignerFile (att.Region.FileName);
 					}
 					
-					if (att.AttributeType.FullName == modelAttType.FullName) {
-						isModel = true;
-					}
+					//type registered with an explicit type name are up to the user to provide a valid name
+					var posArgs = att.PositionalArguments;
+					if (posArgs.Count == 1 || posArgs.Count == 2)
+						objcName = posArgs [0].ConstantValue as string;
+					//non-nested types in the root namespace have names accessible from obj-c
+					else if (string.IsNullOrEmpty (type.Namespace) && type.Name.IndexOf ('.') < 0)
+						objcName = type.Name;
 				}
+				
+				if (attType.Equals (Resolve (dom, modelAttType)))
+					isModel = true;
 			}
 			
 			if (string.IsNullOrEmpty (objcName))
 				return null;
 			
-			var info = new NSObjectTypeInfo (objcName, type.DecoratedFullName, null, type.BaseType.DecoratedFullName, isModel,
-				type.SourceProject != null, registeredInDesigner);
+			string baseType = type.DirectBaseTypes.First ().ReflectionName;
+			if (baseType == "System.Object")
+				baseType = null;
 			
+			bool isUserType = !type.ParentAssembly.Equals (Resolve (dom, nsobjectType).ParentAssembly);
+			
+			var info = new NSObjectTypeInfo (objcName, type.ReflectionName, null, baseType, isModel, isUserType, registeredInDesigner);
+
 			if (info.IsUserType) {
 				UpdateTypeMembers (dom, info, type);
-				info.DefinedIn = type.Parts.Select (p => (string) p.CompilationUnit.FileName).ToArray ();
+				info.DefinedIn = type.Parts.Select (p => (string) p.Region.FileName).ToArray ();
 			}
 			
 			return info;
 		}
 		
-		void UpdateTypeMembers (ProjectDom dom, NSObjectTypeInfo info, IType type)
+		void UpdateTypeMembers (TypeSystemService.ProjectContentWrapper dom, NSObjectTypeInfo info, ITypeDefinition type)
 		{
 			info.Actions.Clear ();
 			info.Outlets.Clear ();
-			
 			foreach (var prop in type.Properties) {
 				foreach (var att in prop.Attributes) {
-					bool isIBOutlet = att.AttributeType.FullName == iboutletAttType.FullName;
+					var attType = att.AttributeType;
+					bool isIBOutlet = attType.Equals (Resolve (dom, iboutletAttType));
 					if (!isIBOutlet) {
-						if (att.AttributeType.FullName != connectAttType.FullName)
+						if (!attType.Equals (Resolve (dom, connectAttType)))
 							continue;
 					}
 					string name = null;
-					if (att.PositionalArguments.Count == 1)
-						name = (string)((System.CodeDom.CodePrimitiveExpression)att.PositionalArguments[0]).Value;
+					var posArgs = att.PositionalArguments;
+					if (posArgs.Count == 1)
+						name = posArgs [0].ConstantValue as string;
 					if (string.IsNullOrEmpty (name))
 						name = prop.Name;
 					
 					// HACK: Work around bug #1586 in the least obtrusive way possible. Strip out any outlet
 					// with the name 'view' on subclasses of MonoTouch.UIKit.UIViewController to avoid 
 					// conflicts with the view property mapped there
-					if (name == "view")
-						if (dom.GetInheritanceTree (type).Any (p => p.FullName == "MonoTouch.UIKit.UIViewController"))
+					if (name == "view") {
+						if (type.GetAllBaseTypeDefinitions ().Any (p => p.ReflectionName == "MonoTouch.UIKit.UIViewController"))
 							continue;
+					}
 					
-					var ol = new IBOutlet (name, prop.Name, null, prop.ReturnType.FullName);
-					if (MonoDevelop.DesignerSupport.CodeBehind.IsDesignerFile (prop.DeclaringType.CompilationUnit.FileName))
+					var ol = new IBOutlet (name, prop.Name, null, prop.ReturnType.ReflectionName);
+					if (MonoDevelop.DesignerSupport.CodeBehind.IsDesignerFile (prop.Region.FileName))
 						ol.IsDesigner = true;
 					info.Outlets.Add (ol);
 					break;
 				}
 			}
-			
 			foreach (var meth in type.Methods) {
 				foreach (var att in meth.Attributes) {
-					bool isIBAction = att.AttributeType.FullName == ibactionAttType.FullName;
+					var attType = att.AttributeType;
+					bool isIBAction = attType.Equals (Resolve (dom, ibactionAttType));
 					if (!isIBAction) {
-						if (att.AttributeType.FullName != exportAttType.FullName)
+						if (!attType.Equals (Resolve (dom, exportAttType)))
 							continue;
 					}
-					bool isDesigner =  MonoDevelop.DesignerSupport.CodeBehind.IsDesignerFile (
-						meth.DeclaringType.CompilationUnit.FileName);
+
+					bool isDesigner = meth.Parts.Any (part => MonoDevelop.DesignerSupport.CodeBehind.IsDesignerFile (part.Region.FileName));
 					//only support Export from old designer files, user code must be IBAction
 					if (!isDesigner && !isIBAction)
 						continue;
 					
 					string[] name = null;
-					if (att.PositionalArguments.Count == 1) {
-						var n = (string)((System.CodeDom.CodePrimitiveExpression)att.PositionalArguments[0]).Value;
+					var posArgs = att.PositionalArguments;
+					if (posArgs.Count == 1 || posArgs.Count == 2) {
+						var n = posArgs [0].ConstantValue as string;
 						if (!string.IsNullOrEmpty (n))
 							name = n.Split (colonChar);
 					}
-					var action = new IBAction (name != null? name [0] : meth.Name, meth.Name);
+					var action = new IBAction (name != null ? name [0] : meth.Name, meth.Name);
 					int i = 1;
 					foreach (var param in meth.Parameters) {
-						string label = name != null && i < name.Length? name[i] : null;
+						string label = name != null && i < name.Length ? name [i] : null;
 						if (label != null && label.Length == 0)
 							label = null;
-						action.Parameters.Add (new IBActionParameter (label, param.Name, null, param.ReturnType.FullName));
+						action.Parameters.Add (new IBActionParameter (label, param.Name, null, param.Type.ReflectionName));
 					}
-					if (MonoDevelop.DesignerSupport.CodeBehind.IsDesignerFile (meth.DeclaringType.CompilationUnit.FileName))
+					if (meth.Parts.Any (part => MonoDevelop.DesignerSupport.CodeBehind.IsDesignerFile (part.Region.FileName)))
 						action.IsDesigner = true;
 					info.Actions.Add (action);
 					break;
@@ -285,18 +307,36 @@ namespace MonoDevelop.MacDev.ObjCIntegration
 				var def = match.Groups[2].Value;
 				if (kind == "IBOutlet") {
 					var split = def.Split (whitespaceChars, StringSplitOptions.RemoveEmptyEntries);
-					if (split.Length != 2)
-						continue;
-					string objcName = split[1].TrimStart ('*');
 					string objcType = split[0].TrimEnd ('*');
-					if (objcType == "id")
-						objcType = "NSObject";
-					if (string.IsNullOrEmpty (objcType)) {
+					string objcName = null;
+
+					for (int i = 1; i < split.Length; i++) {
+						objcName = split[i].TrimStart ('*');
+						if (string.IsNullOrEmpty (objcName))
+							continue;
+
+						if (i + 1 < split.Length) {
+							// This is a bad sign... what tokens are after the name??
+							objcName = null;
+							break;
+						}
+					}
+
+					if (string.IsNullOrEmpty (objcType) || string.IsNullOrEmpty (objcName)) {
 						MessageService.ShowError (GettextCatalog.GetString ("Error while parsing header file."),
 							string.Format (GettextCatalog.GetString ("The definition '{0}' can't be parsed."), def));
+
+						// We can't recover if objcName is empty...
+						if (string.IsNullOrEmpty (objcName))
+							continue;
+
+						// We can try using NSObject...
 						objcType = "NSObject";
 					}
-					
+
+					if (objcType == "id")
+						objcType = "NSObject";
+
 					IBOutlet outlet = new IBOutlet (objcName, objcName, objcType, null);
 					outlet.IsDesigner = true;
 					
@@ -354,4 +394,5 @@ namespace MonoDevelop.MacDev.ObjCIntegration
 		Removed,
 		Modified
 	}
+	
 }
