@@ -28,6 +28,7 @@ using System;
 using System.IO;
 using System.Xml;
 using System.Linq;
+using System.Diagnostics;
 using System.Collections.Generic;
 
 using MonoDevelop.Core;
@@ -35,6 +36,7 @@ using MonoDevelop.Projects;
 using MonoDevelop.MacDev.ObjCIntegration;
 using System.Threading.Tasks;
 using MonoDevelop.MacDev.XcodeIntegration;
+using MonoDevelop.MacInterop;
 
 namespace MonoDevelop.MacDev.XcodeSyncing
 {
@@ -45,27 +47,38 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 	
 	public abstract class XcodeProjectTracker : IDisposable
 	{
-		NSObjectInfoService infoService;
-		DotNetProject dnp;
+		readonly NSObjectInfoService infoService;
+		readonly DotNetProject dnp;
+
+		object xcode_lock = new object ();
 		List<NSObjectTypeInfo> userTypes;
 		XcodeMonitor xcode;
-		
-		bool syncing;
+
+		bool updatingProjectFiles;
 		bool disposed;
 		
-		bool updatingProjectFiles;
+		FilePath ObjDir {
+			get { return dnp.BaseDirectory.Combine ("obj"); }
+		}
 		
+		FilePath XcodeDir {
+			get { return ObjDir.Combine ("Xcode"); }
+		}
+
 		public XcodeProjectTracker (DotNetProject dnp, NSObjectInfoService infoService)
 		{
+			if (dnp == null)
+				throw new ArgumentNullException ("dnp");
 			this.dnp = dnp;
 			this.infoService = infoService;
 			AppleSdkSettings.Changed += DisableSyncing;
 		}
-		
+
 		public bool ShouldOpenInXcode (FilePath fileName)
 		{
 			if (!HasInterfaceDefinitionExtension (fileName))
 				return false;
+
 			var file = dnp.Files.GetFile (fileName);
 			return file != null && (file.BuildAction == BuildAction.InterfaceDefinition);
 		}
@@ -80,14 +93,18 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 			return new string[] { "Foundation" };
 		}
 		
-		void EnableSyncing ()
+		bool SyncingEnabled {
+			get { return xcode != null; }
+		}
+		
+		void EnableSyncing (IProgressMonitor monitor)
 		{
-			if (syncing)
+			if (SyncingEnabled)
 				return;
-			syncing = true;
-			xcode = new XcodeMonitor (dnp.BaseDirectory.Combine ("obj", "Xcode"), dnp.Name);
 			
-			XC4Debug.Log ("Enabled syncing for project: {0}", dnp.Name);
+			monitor.Log.WriteLine ("Enabled syncing for project: {0}", dnp.Name);
+			
+			xcode = new XcodeMonitor (XcodeDir, dnp.Name);
 			
 			dnp.FileAddedToProject += FileAddedToProject;
 			dnp.FilePropertyChangedInProject += FilePropertyChangedInProject;
@@ -96,19 +113,29 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 			dnp.NameChanged += ProjectNameChanged;
 			MonoDevelop.Ide.IdeApp.CommandService.ApplicationFocusIn += AppRegainedFocus;
 		}
-		
+
 		void DisableSyncing ()
 		{
-			if (!syncing)
+			DisableSyncing (true);
+		}
+		
+		void DisableSyncing (bool closeProject)
+		{
+			if (!SyncingEnabled)
 				return;
-			syncing = false;
-			
-			xcode.CloseProject ();
-			xcode.DeleteProjectDirectory ();
-			xcode = null;
 			
 			XC4Debug.Log ("Disabled syncing for project: {0}", dnp.Name);
 			
+			XC4Debug.Indent ();
+			try {
+				if (closeProject)
+					xcode.CloseProject ();
+				xcode.DeleteProjectDirectory ();
+			} finally {
+				XC4Debug.Unindent ();
+				xcode = null;
+			}
+			
 			dnp.FileAddedToProject -= FileAddedToProject;
 			dnp.FilePropertyChangedInProject -= FilePropertyChangedInProject;;
 			dnp.FileRemovedFromProject -= FileRemovedFromProject;
@@ -117,72 +144,97 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 			MonoDevelop.Ide.IdeApp.CommandService.ApplicationFocusIn -= AppRegainedFocus;
 		}
 		
+		void ShowXcodeScriptError ()
+		{
+			MonoDevelop.Ide.MessageService.ShowError (
+				GettextCatalog.GetString ("MonoDevelop could not communicate with Xcode"),
+				GettextCatalog.GetString ("If Xcode is still running, please ensure that all changes have been saved and " +
+			                          "Xcode has been exited before continuing, otherwise any new changes may be lost."));
+		}
+		
 		void AppRegainedFocus (object sender, EventArgs e)
 		{
-			if (!syncing)
-				return;
-			
-			bool isOpen = false;
-			using (var monitor = GetStatusMonitor (GettextCatalog.GetString ("Synchronizing changes from Xcode"))) {
-				try {
-					isOpen = xcode != null && xcode.IsProjectOpen ();
-					if (isOpen) {
-						monitor.BeginTask (GettextCatalog.GetString ("Saving Xcode project"), 0);
-						xcode.SaveProject ();
+			lock (xcode_lock) {
+				if (!SyncingEnabled)
+					return;
+
+				XC4Debug.Log ("MonoDevelop has regained focus.");
+
+				using (var monitor = GetStatusMonitor (GettextCatalog.GetString ("Synchronizing changes from Xcode..."))) {
+					bool projectOpen = false;
+
+					try {
+						// Note: Both IsProjectOpen() and SaveProject() may throw TimeoutExceptions or AppleScriptExceptions
+						if ((projectOpen = xcode.IsProjectOpen ()))
+							xcode.SaveProject (monitor);
+					} catch (Exception ex) {
+						ShowXcodeScriptError ();
+						monitor.Log.WriteLine ("Xcode failed to save pending changes to project: {0}", ex);
+
+						// Note: This will cause us to disable syncing after we sync whatever we can over from Xcode...
+						projectOpen = false;
+					}
+
+					try {
+						SyncXcodeChanges (monitor);
+					} finally {
+						if (!projectOpen) {
+							XC4Debug.Log ("Xcode project for '{0}' is not open, disabling syncing.", dnp.Name);
+							DisableSyncing (false);
+						}
 					}
-				} catch (Exception ex) {
-					MonoDevelop.Ide.MessageService.ShowError (
-						GettextCatalog.GetString ("MonoDevelop could not communicate with XCode"),
-						GettextCatalog.GetString (
-							"If XCode is still running, please ensure that all changes have been saved and " +
-							"XCode has been exited before continuing, otherwise any new changes may be lost."));
-					monitor.Log.WriteLine ("XCode could not be made save pending changes: {0}", ex);
 				}
-				if (isOpen) {
-					monitor.EndTask ();
-				}
-				
-				SyncXcodeChanges (monitor);
-			}
-			
-			if (!isOpen) {
-				XC4Debug.Log ("Project closed, disabling syncing");
-				DisableSyncing ();
 			}
 		}
 		
-		bool OpenFileInXcodeProject (string path)
+		bool OpenFileInXcodeProject (IProgressMonitor monitor, string path)
 		{
 			bool succeeded = false;
-			using (var monitor = GetStatusMonitor (GettextCatalog.GetString ("Syncing to Xcode..."))) {
-				try {
-					EnableSyncing ();
-					if (!UpdateTypes (monitor, true) || monitor.IsCancelRequested) {
-						return succeeded;
-					}
-					if (!UpdateXcodeProject (monitor) || monitor.IsCancelRequested) {
-						return succeeded;
-					}
-					xcode.OpenFile (path);
-					succeeded = true;
-				} catch (Exception ex) {
-					monitor.ReportError (GettextCatalog.GetString ("Could not open file in Xcode project"), ex);
-				} finally {
-					if (!succeeded)
-						DisableSyncing ();
-				}
+			
+			try {
+				EnableSyncing (monitor);
+				
+				if (!UpdateTypes (monitor) || monitor.IsCancelRequested)
+					return false;
+				
+				if (!UpdateXcodeProject (monitor) || monitor.IsCancelRequested)
+					return false;
+				
+				xcode.OpenFile (monitor, path);
+				succeeded = true;
+			} catch (AppleScriptException asex) {
+				ShowXcodeScriptError ();
+				monitor.ReportError (GettextCatalog.GetString ("Could not open file in Xcode project."), asex);
+			} catch (TimeoutException tex) {
+				ShowXcodeScriptError ();
+				monitor.ReportError (GettextCatalog.GetString ("Could not open file in Xcode project."), tex);
+			} catch (Exception ex) {
+				monitor.ReportError (GettextCatalog.GetString ("Could not open file in Xcode project."), ex);
+			} finally {
+				if (!succeeded)
+					DisableSyncing (true);
 			}
+			
 			return succeeded;
 		}
 		
-		public bool OpenDocument (string file)
+		public bool OpenDocument (string path)
 		{
-			XC4Debug.Log ("Opening file {0}", file);
-			var xibFile = dnp.Files.GetFile (file);
-			System.Diagnostics.Debug.Assert (xibFile != null);
-			System.Diagnostics.Debug.Assert (IsInterfaceDefinition (xibFile));
-			
-			return OpenFileInXcodeProject (xibFile.ProjectVirtualPath);
+			var xibFile = dnp.Files.GetFile (path);
+			bool success = false;
+
+			Debug.Assert (xibFile != null);
+			Debug.Assert (IsInterfaceDefinition (xibFile));
+
+			lock (xcode_lock) {
+				using (var monitor = GetStatusMonitor (GettextCatalog.GetString ("Opening document '{0}' from project '{1}' in Xcode...", xibFile.ProjectVirtualPath, dnp.Name))) {
+					monitor.BeginTask (GettextCatalog.GetString ("Opening document '{0}' from project '{1}' in Xcode...", xibFile.ProjectVirtualPath, dnp.Name), 0);
+					success = OpenFileInXcodeProject (monitor, xibFile.ProjectVirtualPath);
+					monitor.EndTask ();
+				}
+			}
+
+			return success;
 		}
 		
 		static bool IsInterfaceDefinition (ProjectFile pf)
@@ -192,7 +244,7 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 		
 		bool IncludeInSyncedProject (ProjectFile pf)
 		{
-			return pf.BuildAction == BuildAction.Content
+			return (pf.BuildAction == BuildAction.Content && pf.ProjectVirtualPath.ParentDirectory.IsNullOrEmpty)
 				|| (pf.BuildAction == BuildAction.InterfaceDefinition && HasInterfaceDefinitionExtension (pf.FilePath));
 		}
 		
@@ -200,56 +252,105 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 		
 		void ProjectNameChanged (object sender, SolutionItemRenamedEventArgs e)
 		{
-			XC4Debug.Log ("Project name changed, resetting sync");
-			xcode.CloseProject ();
-			xcode.DeleteProjectDirectory ();
-			xcode = new XcodeMonitor (dnp.BaseDirectory.Combine ("obj", "Xcode"), dnp.Name);
+			lock (xcode_lock) {
+				if (!SyncingEnabled)
+					return;
+
+				XC4Debug.Log ("Project '{0}' was renamed to '{1}', resetting Xcode sync.", e.OldName, e.NewName);
+
+				XC4Debug.Indent ();
+				try {
+					xcode.CloseProject ();
+					xcode.DeleteProjectDirectory ();
+				} finally {
+					XC4Debug.Unindent ();
+				}
+
+				xcode = new XcodeMonitor (dnp.BaseDirectory.Combine ("obj", "Xcode"), dnp.Name);
+			}
 		}
 		
 		void FileRemovedFromProject (object sender, ProjectFileEventArgs e)
 		{
-			if (syncing && e.Any (finf => finf.Project == dnp && IsInterfaceDefinition (finf.ProjectFile))) {
-				if (!dnp.Files.Any (IsInterfaceDefinition)) {
-					XC4Debug.Log ("All page files removed, disabling sync");
-					DisableSyncing ();
+			lock (xcode_lock) {
+				if (!SyncingEnabled)
 					return;
+
+				XC4Debug.Log ("Files removed from project '{0}'", dnp.Name);
+				foreach (var file in e)
+					XC4Debug.Log ("   * Removed: {0}", file.ProjectFile.ProjectVirtualPath);
+
+				XC4Debug.Indent ();
+				try {
+					if (e.Any (finf => finf.Project == dnp && IsInterfaceDefinition (finf.ProjectFile))) {
+						if (!dnp.Files.Any (IsInterfaceDefinition)) {
+							XC4Debug.Log ("Last Interface Definition file removed from '{0}', disabling Xcode sync.", dnp.Name);
+							DisableSyncing (true);
+							return;
+						}
+					}
+				} finally {
+					XC4Debug.Unindent ();
 				}
+
+				CheckFileChanges (e);
 			}
-			
-			CheckFileChanges (e);
 		}
 
 		void FileAddedToProject (object sender, ProjectFileEventArgs e)
 		{
-			CheckFileChanges (e);
+			lock (xcode_lock) {
+				if (!SyncingEnabled)
+					return;
+
+				XC4Debug.Log ("Files added to project '{0}'", dnp.Name);
+				foreach (var file in e)
+					XC4Debug.Log ("   * Added: {0}", file.ProjectFile.ProjectVirtualPath);
+
+				CheckFileChanges (e);
+			}
 		}
 
 		void FileChangedInProject (object sender, ProjectFileEventArgs e)
 		{
-			//avoid infinite recursion when we add files
-			if (updatingProjectFiles)
-				return;
-			CheckFileChanges (e);
+			lock (xcode_lock) {
+				// avoid infinite recursion when we add files
+				if (!SyncingEnabled || updatingProjectFiles)
+					return;
+
+				XC4Debug.Log ("Files changed in project '{0}'", dnp.Name);
+				foreach (var file in e)
+					XC4Debug.Log ("   * Changed: {0}", file.ProjectFile.ProjectVirtualPath);
+
+				CheckFileChanges (e);
+			}
 		}
 
 		void FilePropertyChangedInProject (object sender, ProjectFileEventArgs e)
 		{
-			CheckFileChanges (e);
+			lock (xcode_lock) {
+				if (!SyncingEnabled)
+					return;
+
+				XC4Debug.Log ("File properties changed in project '{0}'", dnp.Name);
+				foreach (var file in e)
+					XC4Debug.Log ("   * Property Changed: {0}", file.ProjectFile.ProjectVirtualPath);
+
+				CheckFileChanges (e);
+			}
 		}
-		
+
 		void CheckFileChanges (ProjectFileEventArgs e)
 		{
-			if (!syncing)
-				return;
-			
-			XC4Debug.Log ("Checking for changed files");
 			bool updateTypes = false, updateProject = false;
-			foreach (ProjectFileEventInfo finf in e) {
-				if (finf.Project != dnp)
+			
+			foreach (ProjectFileEventInfo finfo in e) {
+				if (finfo.Project != dnp)
 					continue;
-				if (finf.ProjectFile.BuildAction == BuildAction.Compile) {
+
+				if (finfo.ProjectFile.BuildAction == BuildAction.Compile) {
 					updateTypes = true;
-				} else if (IncludeInSyncedProject (finf.ProjectFile)) {
+				} else if (IncludeInSyncedProject (finfo.ProjectFile)) {
 					updateProject = true;
 				}
 			}
@@ -258,7 +359,7 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 				using (var monitor = GetStatusMonitor (GettextCatalog.GetString ("Syncing types to Xcode..."))) {
 					//FIXME: make this async (and safely async)
 					//FIXME: only update the project if obj-c types change
-					updateProject |= UpdateTypes (monitor, true);
+					updateProject |= UpdateTypes (monitor);
 				}
 			}
 			
@@ -267,8 +368,15 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 					//FIXME: make this async (and safely async)
 					var running = xcode.CheckRunning ();
 					UpdateXcodeProject (monitor);
-					if (running)
-						xcode.OpenProject ();
+					if (running) {
+						try {
+							xcode.OpenProject (monitor);
+						} catch (AppleScriptException) {
+							ShowXcodeScriptError ();
+						} catch (TimeoutException) {
+							ShowXcodeScriptError ();
+						}
+					}
 				}
 			}
 		}
@@ -277,7 +385,7 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 		
 		#region Progress monitors
 		
-		//FIXME: should be use a modal monitor to prevent the user doing unexpected things?
+		//FIXME: should we use a modal monitor to prevent the user doing unexpected things?
 		IProgressMonitor GetStatusMonitor (string title)
 		{
 			IProgressMonitor monitor = MonoDevelop.Ide.IdeApp.Workbench.ProgressMonitors.GetStatusProgressMonitor (
@@ -293,20 +401,22 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 		
 		#region Outbound syncing
 		
-		bool UpdateTypes (IProgressMonitor monitor, bool force)
+		bool UpdateTypes (IProgressMonitor monitor)
 		{
 			monitor.BeginTask (GettextCatalog.GetString ("Updating Objective-C type information"), 0);
 			try {
 				var pinfo = infoService.GetProjectInfo (dnp);
 				if (pinfo == null)
-					throw new Exception ("Did not get project info");
-				//FIXME: report progress
-				pinfo.Update (force);
+					return true;
+				// FIXME: report progress
+				pinfo.Update (true);
 				userTypes = pinfo.GetTypes ().Where (t => t.IsUserType).ToList ();
 				return true;
 			} catch (Exception ex) {
 				monitor.ReportError (GettextCatalog.GetString ("Error updating Objective-C type information"), ex);
 				return false;
+			} finally {
+				monitor.EndTask ();
 			}
 		}
 		
@@ -315,8 +425,22 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 		bool UpdateXcodeProject (IProgressMonitor monitor)
 		{
 			try {
+				// Ensure that the obj directory and all subfiles/subdirectories
+				// are writeable so we can create the temp files for xcode syncing
+				dnp.BaseDirectory.MakeWritable ();
+				ObjDir.MakeWritable ();
+				XcodeDir.MakeWritable (true);
+				
 				xcode.UpdateProject (monitor, CreateSyncList (), CreateProject (dnp.Name));
 				return true;
+			} catch (AppleScriptException asex) {
+				ShowXcodeScriptError ();
+				monitor.ReportError (GettextCatalog.GetString ("Error updating Xcode project"), asex);
+				return false;
+			} catch (TimeoutException tex) {
+				ShowXcodeScriptError ();
+				monitor.ReportError (GettextCatalog.GetString ("Error updating Xcode project"), tex);
+				return false;
 			} catch (Exception ex) {
 				monitor.ReportError (GettextCatalog.GetString ("Error updating Xcode project"), ex);
 				return false;
@@ -341,7 +465,7 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 		bool SyncXcodeChanges (IProgressMonitor monitor)
 		{
 			try {
-				monitor.BeginTask (GettextCatalog.GetString ("Detecting changed files in Xcode"), 0);
+				monitor.BeginTask (GettextCatalog.GetString ("Detecting changed files in Xcode..."), 0);
 				var changeCtx = xcode.GetChanges (monitor, infoService, dnp);
 				monitor.EndTask ();
 				
@@ -400,16 +524,28 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 			if (context.FileSyncJobs.Count == 0)
 				return;
 			
+			monitor.BeginStepTask ("Copying files from Xcode back to MonoDevelop...", context.FileSyncJobs.Count, 1);
+			
 			foreach (var file in context.FileSyncJobs) {
-				monitor.Log.WriteLine ("Copying {0} file from Xcode: {1}", file.IsFreshlyAdded ? "added" : "changed", file.SyncedRelative);
+				monitor.Log.WriteLine ("Copying {0} file from Xcode: {1}", file.IsFreshlyAdded ? "new" : "changed", file.SyncedRelative);
 				
 				if (!Directory.Exists (file.Original.ParentDirectory))
 					Directory.CreateDirectory (file.Original.ParentDirectory);
 				
 				var tempFile = file.Original.ParentDirectory.Combine (".#" + file.Original.ParentDirectory.FileName);
-				File.Copy (context.ProjectDir.Combine (file.SyncedRelative), tempFile);
-				FileService.SystemRename (tempFile, file.Original);
-				context.SetSyncTimeToNow (file.SyncedRelative);
+				FilePath path = context.ProjectDir.Combine (file.SyncedRelative);
+				
+				if (File.Exists (path)) {
+					File.Copy (path, tempFile);
+					FileService.SystemRename (tempFile, file.Original);
+					
+					DateTime mtime = File.GetLastWriteTime (file.Original);
+					context.SetSyncTime (file.SyncedRelative, mtime);
+				} else {
+					monitor.ReportWarning (string.Format ("'{0}' does not exist.", file.SyncedRelative));
+				}
+				
+				monitor.Step (1);
 			}
 			
 			monitor.EndTask ();
@@ -429,7 +565,7 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 		/// </returns>
 		bool AddFilesToMD (IProgressMonitor monitor, XcodeSyncBackContext context)
 		{
-			bool needsEndTask = false;
+			bool filesAdded = false;
 			
 			if (context.FileSyncJobs.Count == 0)
 				return false;
@@ -438,20 +574,15 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 				if (!file.IsFreshlyAdded)
 					continue;
 				
-				monitor.Log.WriteLine ("Adding new file to project: {0}", file.SyncedRelative);
+				monitor.Log.WriteLine ("Adding '{0}' to project '{1}'", file.SyncedRelative, dnp.Name);
 				
 				FilePath path = new FilePath (file.Original);
 				string buildAction = HasInterfaceDefinitionExtension (path) ? BuildAction.InterfaceDefinition : BuildAction.Content;
 				context.Project.AddFile (path, buildAction);
-				needsEndTask = true;
+				filesAdded = true;
 			}
 			
-			if (needsEndTask) {
-				monitor.EndTask ();
-				return true;
-			}
-			
-			return false;
+			return filesAdded;
 		}
 		
 		protected virtual IEnumerable<NSObjectTypeInfo> GetCustomTypesFromUIDefinition (FilePath fileName)
@@ -478,14 +609,18 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 			var writer = MonoDevelop.DesignerSupport.CodeBehindWriter.CreateForProject (
 				new MonoDevelop.Core.ProgressMonitoring.NullProgressMonitor (), dnp);
 			bool addedTypes = false;
-			
-			monitor.BeginTask (GettextCatalog.GetString ("Generating custom classes defined in UI definition files"), 0);
+			bool beganTask = false;
 			
 			// Collect our list of custom classes from UI definition files
 			foreach (var job in context.FileSyncJobs) {
 				if (!HasInterfaceDefinitionExtension (job.Original))
 					continue;
 				
+				if (!beganTask) {
+					monitor.BeginTask (GettextCatalog.GetString ("Generating custom classes defined in UI definition files..."), 0);
+					beganTask = true;
+				}
+				
 				string relative = job.SyncedRelative.ParentDirectory;
 				string dir = dnp.BaseDirectory;
 				
@@ -535,7 +670,8 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 			
 			writer.WriteOpenFiles ();
 			
-			monitor.EndTask ();
+			if (beganTask)
+				monitor.EndTask ();
 			
 			return addedTypes;
 		}
@@ -562,26 +698,26 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 			var writer = MonoDevelop.DesignerSupport.CodeBehindWriter.CreateForProject (
 				new MonoDevelop.Core.ProgressMonitoring.NullProgressMonitor (), dnp);
 			
-			monitor.BeginTask (GettextCatalog.GetString ("Detecting changes made in Xcode"), 0);
+			monitor.BeginTask (GettextCatalog.GetString ("Detecting changes made in Xcode to custom user types..."), 0);
 			Dictionary<string, NSObjectTypeInfo> newTypes;
 			Dictionary<string, ProjectFile> newFiles;
 			var updates = context.GetTypeUpdates (monitor, provider, out newTypes, out newFiles);
 			if ((updates == null || updates.Count == 0) && newTypes == null && newFiles == null) {
-				monitor.Log.WriteLine ("No changes found");
+				monitor.Log.WriteLine ("No changes found.");
 				monitor.EndTask ();
 				typesAdded = false;
 				return false;
 			}
-			
-			monitor.Log.WriteLine ("Found {0} changed types", updates.Count);
 			monitor.EndTask ();
 			
 			int count = updates.Count + (newTypes != null ? newTypes.Count : 0);
-			monitor.BeginTask (GettextCatalog.GetString ("Updating types in MonoDevelop"), count);
+			monitor.BeginTask (GettextCatalog.GetString ("Updating custom user types in MonoDevelop..."), count);
 			
 			// First, add new types...
 			if (newTypes != null && newTypes.Count > 0) {
 				foreach (var nt in newTypes) {
+					monitor.Log.WriteLine ("Adding new type: {0}", nt.Value.CliName);
+					
 					if (provider is Microsoft.CSharp.CSharpCodeProvider) {
 						var cs = new CSharpCodeTypeDefinition () {
 							WrapperNamespace = infoService.WrapperRoot,
@@ -608,7 +744,7 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 			
 			// Next, generate the designer files for any added/changed types
 			foreach (var df in updates) {
-				monitor.Log.WriteLine ("Syncing {0} types from Xcode to file '{1}'", df.Value.Count, df.Key);
+				monitor.Log.WriteLine ("Generating designer file: {0}", df.Key);
 				if (provider is Microsoft.CSharp.CSharpCodeProvider) {
 					var cs = new CSharpCodeCodebehind () {
 						Types = df.Value,
@@ -628,16 +764,14 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 			
 			// Update sync timestamps
 			foreach (var df in updates) {
-				foreach (var type in df.Value) {
-					context.SetSyncTimeToNow (type.ObjCName + ".h");
-					context.SetSyncTimeToNow (type.ObjCName + ".m");
-				}
+				foreach (var type in df.Value)
+					context.SetSyncTime (type.ObjCName + ".h", DateTime.Now);
 			}
 			
 			// Add new files to the DotNetProject
 			if (newFiles != null) {
 				foreach (var f in newFiles) {
-					monitor.Log.WriteLine ("Added new designer file {0}", f.Key);
+					monitor.Log.WriteLine ("Added new designer file: {0}", f.Key);
 					dnp.AddFile (f.Value);
 				}
 			}
@@ -672,45 +806,75 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 		{
 			if (disposed)
 				return;
+			
 			disposed = true;
-			DisableSyncing ();
+			DisableSyncing (true);
 			AppleSdkSettings.Changed -= DisableSyncing;
 		}
 	}
 	
 	static class XC4Debug
 	{
+		static int indentLevel = 0;
 		static TextWriter writer;
 		
 		static XC4Debug ()
 		{
 			FilePath logDir = UserProfile.Current.LogDir;
-			FilePath logFile = logDir.Combine ("Xcode4Sync.log");
+			FilePath logFile = logDir.Combine (UniqueLogFile);
+			
 			FileService.EnsureDirectoryExists (logDir);
+			
 			try {
-				writer = new StreamWriter (logFile) { AutoFlush = true };
+				var stream = File.Open (logFile, FileMode.Create, FileAccess.Write, FileShare.Read);
+				writer = new StreamWriter (stream) { AutoFlush = true };
 			} catch (Exception ex) {
 				LoggingService.LogError ("Could not create Xcode sync logging file", ex);
 			}
 		}
 		
+		static string UniqueLogFile {
+			get {
+				return string.Format ("Xcode4Sync-{0}.log", LoggingService.LogTimestamp.ToString ("yyyy-MM-dd__HH-mm-ss"));
+			}
+		}
+		
+		static string TimeStamp {
+			get { return string.Format ("[{0}] ", DateTime.Now.ToString ("yyyy-MM-dd HH:mm:ss.f")); }
+		}
+		
+		public static void Indent ()
+		{
+			indentLevel++;
+		}
+		
+		public static void Unindent ()
+		{
+			indentLevel--;
+		}
+		
 		public static void Log (string message)
 		{
 			if (writer == null)
 				return;
-			writer.WriteLine (message);
+			
+			writer.WriteLine (TimeStamp + new string (' ', indentLevel * 3) + message);
 		}
 		
-		public static void Log (string messageFormat, params object[] values)
+		public static void Log (string format, params object[] args)
 		{
 			if (writer == null)
 				return;
-			writer.WriteLine (messageFormat, values);
+			
+			writer.WriteLine (TimeStamp + new string (' ', indentLevel * 3) + format, args);
 		}
 		
 		public static IProgressMonitor GetLoggingMonitor ()
 		{
-			return new MonoDevelop.Core.ProgressMonitoring.ConsoleProgressMonitor (writer);
+			if (writer == null)
+				return new MonoDevelop.Core.ProgressMonitoring.NullProgressMonitor ();
+			
+			return new MonoDevelop.Core.ProgressMonitoring.ConsoleProgressMonitor (writer) { EnableTimeStamp = true, WrapText = false };
 		}
 	}
 }
