@@ -28,12 +28,11 @@ using System;
 using System.Collections.Generic;
 using Gtk;
 using Gdk;
- 
-using MonoDevelop.Components.Diff;
+
 using MonoDevelop.Core;
-using MonoDevelop.Projects.Dom.Parser;
 using Mono.TextEditor;
 using MonoDevelop.Ide;
+using ICSharpCode.NRefactory.TypeSystem;
 
 
 namespace MonoDevelop.Refactoring
@@ -49,7 +48,7 @@ namespace MonoDevelop.Refactoring
 
 		List<Change> changes;
 
-		public RefactoringPreviewDialog (ProjectDom ctx, List<Change> changes)
+		public RefactoringPreviewDialog (List<Change> changes)
 		{
 			this.Build ();
 			this.changes = changes;
@@ -87,7 +86,7 @@ namespace MonoDevelop.Refactoring
 			
 			buttonOk.Clicked += delegate {
 				IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetBackgroundProgressMonitor (this.Title, null);
-				RefactoringService.AcceptChanges (monitor, ctx, changes);
+				RefactoringService.AcceptChanges (monitor, changes);
 				
 				Destroy ();
 			};
@@ -108,8 +107,8 @@ namespace MonoDevelop.Refactoring
 				return;
 			}
 			
-			Mono.TextEditor.Document doc = new Mono.TextEditor.Document ();
-			doc.Text = System.IO.File.ReadAllText (replaceChange.FileName);
+			Mono.TextEditor.TextDocument doc = new Mono.TextEditor.TextDocument ();
+			doc.Text = Mono.TextEditor.Utils.TextFileUtility.ReadAllText (replaceChange.FileName);
 			DocumentLocation loc = doc.OffsetToLocation (replaceChange.Offset);
 			
 			string text = string.Format (GettextCatalog.GetString ("(Line:{0}, Column:{1})"), loc.Line, loc.Column);
@@ -134,25 +133,24 @@ namespace MonoDevelop.Refactoring
 				if (replaceChange == null) 
 					return;
 			
-				Mono.TextEditor.Document doc = new Mono.TextEditor.Document ();
-				doc.Text = System.IO.File.ReadAllText (replaceChange.FileName);
-				List<string> before = new List<string> ();
-				foreach (var line in doc.Lines) {
-					before.Add (doc.GetTextAt (line.Offset, line.EditableLength));
+				var openDocument = IdeApp.Workbench.GetDocument (replaceChange.FileName);
+				Mono.TextEditor.TextDocument originalDocument = new Mono.TextEditor.TextDocument ();
+				originalDocument.FileName = replaceChange.FileName;
+				if (openDocument == null) {
+					originalDocument.Text = Mono.TextEditor.Utils.TextFileUtility.ReadAllText (replaceChange.FileName);
+				} else {
+					originalDocument.Text = openDocument.Editor.Document.Text;
 				}
 				
-				((Mono.TextEditor.IBuffer)doc).Replace (replaceChange.Offset, replaceChange.RemovedChars, replaceChange.InsertedText);
+				Mono.TextEditor.TextDocument changedDocument = new Mono.TextEditor.TextDocument ();
+				changedDocument.FileName = replaceChange.FileName;
+				changedDocument.Text = originalDocument.Text;
 				
-				List<string> after = new List<string> ();
-				foreach (var line in doc.Lines) {
-					after.Add (doc.GetTextAt (line.Offset, line.EditableLength));
-				}
+				((Mono.TextEditor.IBuffer)changedDocument).Replace (replaceChange.Offset, replaceChange.RemovedChars, replaceChange.InsertedText);
 				
-				Diff diff = new Diff (before.ToArray (), after.ToArray (), true, true);
+				string diffString = Mono.TextEditor.Utils.Diff.GetDiffString (originalDocument, changedDocument);
 				
-				System.IO.StringWriter w = new System.IO.StringWriter();
-				UnifiedDiff.WriteUnifiedDiff (diff, w, replaceChange.FileName, replaceChange.FileName, 2);
-				cellRendererDiff.InitCell (treeviewPreview, true, w.ToString ().Trim (), replaceChange.FileName);
+				cellRendererDiff.InitCell (treeviewPreview, true, diffString, replaceChange.FileName);
 			} catch (Exception e) {
 				Console.WriteLine (e);
 			}
