@@ -24,7 +24,9 @@
 // THE SOFTWARE.
 
 using System;
-using MonoDevelop.Ide.Gui.Dialogs; 
+using MonoDevelop.Ide.Gui.Dialogs;
+using MonoDevelop.Core;
+using MonoDevelop.Ide.Gui.Content; 
 
 namespace MonoDevelop.SourceEditor.OptionPanels
 {
@@ -34,6 +36,11 @@ namespace MonoDevelop.SourceEditor.OptionPanels
 		{
 			this.Build();
 			this.codeCompletioncheckbutton.Toggled += HandleCodeCompletioncheckbuttonToggled;
+			
+			this.comboboxLineEndings.AppendText (GettextCatalog.GetString ("Always ask for conversion"));
+			this.comboboxLineEndings.AppendText (GettextCatalog.GetString ("Leave line endings as is"));
+			this.comboboxLineEndings.AppendText (GettextCatalog.GetString ("Always convert line endings"));
+			this.comboboxLineEndings.Active = (int)DefaultSourceEditorOptions.Instance.LineEndingConversion;
 		}
 
 		void HandleCodeCompletioncheckbuttonToggled (object sender, EventArgs e)
@@ -41,7 +48,8 @@ namespace MonoDevelop.SourceEditor.OptionPanels
 			this.enableParameterInsightCheckbutton.Sensitive = 
 				this.autoCodeCompletionCheckbutton.Sensitive = 
 					this.spaceOrPunctiuationCheckbutton.Sensitive = 
-						this.codeCompletioncheckbutton.Active;
+						this.hideObsoleteItemsCheckbutton.Sensitive = 
+							this.codeCompletioncheckbutton.Active;
 		}
 		
 	
@@ -56,7 +64,8 @@ namespace MonoDevelop.SourceEditor.OptionPanels
 			this.enableParameterInsightCheckbutton.Active = DefaultSourceEditorOptions.Instance.EnableParameterInsight;
 			this.autoCodeCompletionCheckbutton.Active = DefaultSourceEditorOptions.Instance.EnableAutoCodeCompletion;
 			this.antiAliasingCheckbutton.Active = DefaultSourceEditorOptions.Instance.UseAntiAliasing;
-			
+//			this.hideObsoleteItemsCheckbutton.Active = CompletionTextEditorExtension.HideObsoleteItems;
+			this.hideObsoleteItemsCheckbutton.Hide ();
 			HandleCodeCompletioncheckbuttonToggled (this, EventArgs.Empty);
 			return this;
 		}
@@ -71,6 +80,8 @@ namespace MonoDevelop.SourceEditor.OptionPanels
 			DefaultSourceEditorOptions.Instance.CompleteWithSpaceOrPunctuation = this.spaceOrPunctiuationCheckbutton.Active;
 			DefaultSourceEditorOptions.Instance.EnableParameterInsight = this.enableParameterInsightCheckbutton.Active;
 			DefaultSourceEditorOptions.Instance.UseAntiAliasing = this.antiAliasingCheckbutton.Active;
+			DefaultSourceEditorOptions.Instance.LineEndingConversion = (LineEndingConversion)this.comboboxLineEndings.Active;
+//			CompletionTextEditorExtension.HideObsoleteItems.Set (this.hideObsoleteItemsCheckbutton.Active);
 		}
 
 		public void Initialize (OptionsDialog dialog, object dataObject)
