@@ -60,11 +60,11 @@ namespace MonoDevelop.SourceEditor
 		static int GetNextNonWsCharOffset (TextEditorData data, int offset)
 		{
 			int result = offset;
-			if (result >= data.Document.Length)
+			if (result >= data.Document.TextLength)
 				return -1;
 			while (Char.IsWhiteSpace (data.Document.GetCharAt (result))) {
 				result++;
-				if (result >= data.Document.Length)
+				if (result >= data.Document.TextLength)
 					return -1;
 			}
 			return result;
@@ -76,13 +76,11 @@ namespace MonoDevelop.SourceEditor
 				if (data.Caret.Offset > 0) {
 					var line = data.GetLine (data.Caret.Line);
 					var stack = line.StartSpan.Clone();
-					Mono.TextEditor.Highlighting.SyntaxModeService.ScanSpans (data.Document, data.Document.SyntaxMode, data.Document.SyntaxMode, stack, line.Offset, data.Caret.Offset - 1);
 					if (stack.Any (s => s.Color == "string.other")) {
 						DeleteActions.Backspace (data);
 						return;
 					}
 					stack = line.StartSpan.Clone();
-					Mono.TextEditor.Highlighting.SyntaxModeService.ScanSpans (data.Document, data.Document.SyntaxMode, data.Document.SyntaxMode, stack, line.Offset, data.Caret.Offset);
 					if (stack.Any (s => s.Color == "string.other")) {
 						DeleteActions.Backspace (data);
 						return;
@@ -96,8 +94,6 @@ namespace MonoDevelop.SourceEditor
 						if (nextCharOffset >= 0 && closing[idx] == data.Document.GetCharAt (nextCharOffset)) {
 							bool updateToEnd = data.Document.OffsetToLineNumber (nextCharOffset) != data.Caret.Line;
 							data.Remove (data.Caret.Offset, nextCharOffset - data.Caret.Offset + 1);
-							if (updateToEnd)
-								data.Document.CommitLineToEndUpdate (data.Caret.Line);
 						}
 					}
 				}
