@@ -30,6 +30,7 @@ using MonoDevelop.Projects;
 using MonoDevelop.Projects.Policies;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Ide;
+using Mono.TextEditor;
 
 namespace MonoDevelop.SourceEditor
 {
@@ -124,7 +125,7 @@ namespace MonoDevelop.SourceEditor
 		}
 
 		public int IndentationSize {
-			get { return TabSize; }
+			get { return CurrentPolicy.IndentWidth; }
 			set {
 				throw new NotSupportedException ();
 			}
@@ -136,13 +137,6 @@ namespace MonoDevelop.SourceEditor
 
 		#region ITextEditorOptions implementation
 
-		public bool AutoIndent {
-			get { return DefaultSourceEditorOptions.Instance.AutoIndent; }
-			set {
-				throw new NotSupportedException ();
-			}
-		}
-
 		public bool CanResetZoom {
 			get { return DefaultSourceEditorOptions.Instance.CanResetZoom; }
 		}
@@ -193,7 +187,7 @@ namespace MonoDevelop.SourceEditor
 			set { throw new NotSupportedException (); }
 		}
 
-		public Mono.TextEditor.Highlighting.ColorSheme GetColorStyle (Gtk.Style widgetStyle)
+		public Mono.TextEditor.Highlighting.ColorScheme GetColorStyle (Gtk.Style widgetStyle)
 		{
 			return DefaultSourceEditorOptions.Instance.GetColorStyle (widgetStyle);
 		}
@@ -265,7 +259,7 @@ namespace MonoDevelop.SourceEditor
 
 		public double Zoom {
 			get { return DefaultSourceEditorOptions.Instance.Zoom; }
-			set { throw new NotSupportedException (); }
+			set { DefaultSourceEditorOptions.Instance.Zoom = value; }
 		}
 
 		public void ZoomIn ()
@@ -317,7 +311,14 @@ namespace MonoDevelop.SourceEditor
 		}
 
 		public IndentStyle IndentStyle {
-			get { return DefaultSourceEditorOptions.Instance.IndentStyle; }
+			get {
+				if (DefaultSourceEditorOptions.Instance.IndentStyle == Mono.TextEditor.IndentStyle.Smart && CurrentPolicy.RemoveTrailingWhitespace)
+					return IndentStyle.Virtual;
+				return DefaultSourceEditorOptions.Instance.IndentStyle;
+			}
+			set {
+				throw new NotSupportedException ("Use property 'IndentStyle' instead.");
+			}
 		}
 
 		public bool TabIsReindent {
