@@ -62,8 +62,11 @@ namespace MonoDevelop.WebReferences
 		public static void GenerateDiscoXml (StringBuilder text, DiscoveryDocument doc)
 		{
 			text.Append ("<big><b>" + GettextCatalog.GetString ("Web Service References") + "</b></big>\n\n");
-			foreach (DiscoveryReference dref in doc.References)
+			foreach (object oref in doc.References)
 			{
+				DiscoveryReference dref = oref as DiscoveryReference;
+				if (dref == null)
+					continue;
 				if (dref is ContractReference) {
 					text.AppendFormat ("<b>Service: {0}</b>\n<span size='small'>{1}</span>", System.IO.Path.GetFileNameWithoutExtension (dref.DefaultFilename), dref.Url);
 				}
@@ -162,9 +165,13 @@ namespace MonoDevelop.WebReferences
 		/// <summary>Gets the path where all web references will be stored for the specified project.</summary>
 		/// <param name="project">A Project containing the root project information.</project>
 		/// <returns>A string containing the base path for web references.</returns>
-		public static string GetWebReferencePath (Project project)
+		public static FilePath GetWebReferencePath (Project project)
 		{
-			return Path.Combine(project.BaseDirectory, "WebReferences");
+			FilePath fp = project.BaseDirectory.Combine ("WebReferences");
+			if (Directory.Exists (fp))
+				return fp;
+			else
+				return project.BaseDirectory.Combine ("Web References");
 		}
 		
 		/// <summary>Checks whether or not the current project does contain any web references.</summary>
@@ -174,7 +181,7 @@ namespace MonoDevelop.WebReferences
 			string webRefPath = Library.GetWebReferencePath(project);
 			foreach (ProjectFile file in project.Files)
 			{
-				if (file.FilePath.StartsWith(webRefPath))
+				if (file.FilePath.IsChildPathOf (webRefPath))
 					return true;
 			}
 			return false;
