@@ -28,14 +28,15 @@
 
 using System;
 
-using MonoDevelop.Projects.Gui.Completion;
+using MonoDevelop.Ide.CodeCompletion;
 using MonoDevelop.Ide.Gui.Content;
+using MonoDevelop.Core;
 
 namespace MonoDevelop.XmlEditor.Completion
 {
 	
 	
-	public class XmlTagCompletionData : IActionCompletionData
+	public class XmlTagCompletionData : CompletionData
 	{
 		string element;
 		int cursorOffset;
@@ -53,38 +54,30 @@ namespace MonoDevelop.XmlEditor.Completion
 			this.closing = closing;
 		}
 		
-		public string Icon {
+		public override IconId Icon {
 			get { return closing? Gtk.Stock.GoBack : Gtk.Stock.GoForward; }
 		}
 
-		public string DisplayText {
-			get { return element; }
-		}
-
-		public string Description {
-			get { return null; }
-		}
-
-		public string CompletionText {
+		public override string DisplayText {
 			get { return element; }
 		}
 		
-		public DisplayFlags DisplayFlags {
-			get { return DisplayFlags.None; }
+		public override string CompletionText {
+			get { return element; }
 		}
 		
-		public void InsertCompletionText (ICompletionWidget widget, CodeCompletionContext context)
+		public override void InsertCompletionText (CompletionListWindow window, ref KeyActions ka, Gdk.Key closeChar, char keyChar, Gdk.ModifierType modifier)
 		{
-			IEditableTextBuffer buf = widget as IEditableTextBuffer;
+			IEditableTextBuffer buf = window.CompletionWidget as IEditableTextBuffer;
 			if (buf != null) {
-				buf.BeginAtomicUndo ();
-				buf.InsertText (buf.CursorPosition, element);
-				
-				// Move caret into the middle of the tags
-				buf.CursorPosition = context.TriggerOffset + cursorOffset;
-				buf.Select (buf.CursorPosition, buf.CursorPosition);
-				buf.EndAtomicUndo ();
+				using (var undo = buf.OpenUndoGroup ()) {
+					buf.InsertText (buf.CursorPosition, element);
+					
+					// Move caret into the middle of the tags
+					buf.CursorPosition = window.CodeCompletionContext.TriggerOffset + cursorOffset;
+					buf.Select (buf.CursorPosition, buf.CursorPosition);
+				}
 			}
-		}		
+		}
 	}
 }
