@@ -40,18 +40,20 @@ namespace MonoDevelop.NUnit
 		internal static Gdk.Pixbuf NotRun;
 		internal static Gdk.Pixbuf Success;
 		internal static Gdk.Pixbuf SuccessAndFailure;
+		internal static Gdk.Pixbuf Inconclusive;
 		internal static Gdk.Pixbuf Loading;
 		
 		static CircleImage ()
 		{
 			try {
-				Running = Gdk.Pixbuf.LoadFromResource("NUnit.Running.png");
-				Failure = Gdk.Pixbuf.LoadFromResource("NUnit.Failed.png");
-				None = Gdk.Pixbuf.LoadFromResource("NUnit.None.png");
-				NotRun = Gdk.Pixbuf.LoadFromResource("NUnit.NotRun.png");
-				Success = Gdk.Pixbuf.LoadFromResource("NUnit.Success.png");
-				SuccessAndFailure = Gdk.Pixbuf.LoadFromResource("NUnit.SuccessAndFailed.png");
-				Loading = Gdk.Pixbuf.LoadFromResource("NUnit.Loading.png");
+				Running = Gdk.Pixbuf.LoadFromResource ("NUnit.Running.png");
+				Failure = Gdk.Pixbuf.LoadFromResource ("NUnit.Failed.png");
+				None = Gdk.Pixbuf.LoadFromResource ("NUnit.None.png");
+				NotRun = Gdk.Pixbuf.LoadFromResource ("NUnit.NotRun.png");
+				Success = Gdk.Pixbuf.LoadFromResource ("NUnit.Success.png");
+				SuccessAndFailure = Gdk.Pixbuf.LoadFromResource ("NUnit.SuccessAndFailed.png");
+				Loading = Gdk.Pixbuf.LoadFromResource ("NUnit.Loading.png");
+				Inconclusive = Gdk.Pixbuf.LoadFromResource ("NUnit.Inconclusive.png");
 			} catch (Exception e) {
 				LoggingService.LogError ("Error while loading icons.", e);
 			}
