@@ -597,7 +597,7 @@ namespace MonoDevelop.NUnit
 					foreach (UnitTest t in group.Tests) {
 						UnitTestResult res = t.Results.GetLastResult (chart.CurrentDate);
 						if (res != null)
-							detailsStore.AppendValues (t, t.Name, res.TotalSuccess.ToString(), res.TotalFailures.ToString(), res.TotalIgnored.ToString());
+							detailsStore.AppendValues (t, t.Name, res.Passed.ToString (), res.ErrorsAndFailures.ToString (), res.Ignored.ToString());
 						else
 							detailsStore.AppendValues (t, t.Name, "", "", "");
 					}
