@@ -29,15 +29,17 @@ using System.IO;
 using System.Text.RegularExpressions;
 
 using MonoDevelop.Core;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.Dom.Parser;
+using MonoDevelop.Ide.TypeSystem;
+using ICSharpCode.NRefactory.TypeSystem;
+using ICSharpCode.NRefactory.TypeSystem.Implementation;
+using MonoDevelop.Projects;
 
 namespace MonoDevelop.VersionControl.Views
 {
 	/// <summary>
 	/// Parser for unified diffs
 	/// </summary>
-	public class DiffParser: AbstractParser
+	public class DiffParser: ITypeSystemParser
 	{
 		// Match the original file and time/revstamp line, capturing the filepath and the stamp
 		static Regex fileHeaderExpression = new Regex (@"^---\s+(?<filepath>[^\t]+)\t(?<stamp>.*)$", RegexOptions.Compiled);
@@ -50,56 +52,50 @@ namespace MonoDevelop.VersionControl.Views
 		
 		#region AbstractParser overrides
 		
-		public override ParsedDocument Parse (ProjectDom dom, string fileName, string content)
+		ParsedDocument ITypeSystemParser.Parse (bool storeAst, string fileName, TextReader textReader, Project project = null)
 		{
-			ParsedDocument doc = new ParsedDocument (fileName);
-			if(null == doc.CompilationUnit)
-				doc.CompilationUnit = new CompilationUnit (fileName);
-			CompilationUnit cu = (CompilationUnit)doc.CompilationUnit;
-			DomType currentFile = null;
-			DomProperty currentRegion = null;
+			var doc = new DefaultParsedDocument (fileName);
+			
+			DefaultUnresolvedTypeDefinition currentFile = null;
+			DefaultUnresolvedProperty currentRegion = null;
 			
 			string eol = Environment.NewLine;
+			string content = textReader.ReadToEnd ();
 			Match eolMatch = eolExpression.Match (content);
 			if (eolMatch != null && eolMatch.Success)
-				eol = eolMatch.Groups["eol"].Value;
+				eol = eolMatch.Groups ["eol"].Value;
 			
 			string[] lines = content.Split (new string[]{eol}, StringSplitOptions.None);
 			int linenum = 1;
 			Match lineMatch;
-			foreach (string line in lines)
-			{
-				lineMatch = fileHeaderExpression.Match (line.Trim());
+			foreach (string line in lines) {
+				lineMatch = fileHeaderExpression.Match (line.Trim ());
 				if (lineMatch != null && lineMatch.Success) {
 					if (currentFile != null) // Close out previous file region
-						currentFile.BodyRegion = new DomRegion (currentFile.BodyRegion.Start.Line,
-						                                        currentFile.BodyRegion.Start.Column,
-						                                        linenum-1, int.MaxValue);
+						currentFile.BodyRegion = new DomRegion (currentFile.BodyRegion.BeginLine,
+						                                        currentFile.BodyRegion.BeginColumn,
+						                                        linenum - 1, int.MaxValue);
 					if (currentRegion != null) // Close out previous chunk region
-						currentRegion.BodyRegion = new DomRegion (currentRegion.BodyRegion.Start.Line,
-						                                          currentRegion.BodyRegion.Start.Column,
-						                                          linenum-1, int.MaxValue);
+						currentRegion.BodyRegion = new DomRegion (currentRegion.BodyRegion.BeginLine,
+						                                          currentRegion.BodyRegion.BeginColumn,
+						                                          linenum - 1, int.MaxValue);
 					
 					// Create new file region
-					currentFile = new DomType (cu, ClassType.Unknown, Modifiers.None, 
-					                           lastToken (lineMatch.Groups["filepath"].Value),
-					                           new DomLocation (linenum, 1), 
-					                           string.Empty,
-					                           new DomRegion (linenum, line.Length+1, linenum, int.MaxValue));
-					cu.Add (currentFile);
+					currentFile = new DefaultUnresolvedTypeDefinition (string.Empty, string.Empty);
+					currentFile.Region = currentFile.BodyRegion = new DomRegion (lastToken (lineMatch.Groups ["filepath"].Value), linenum, line.Length + 1, linenum, int.MaxValue);
+					doc.TopLevelTypeDefinitions.Add (currentFile);
 				} else {
 					lineMatch = chunkExpression.Match (line);
 					if (lineMatch != null && lineMatch.Success && currentFile != null) {
 						if (currentRegion != null) // Close out previous chunk region
-							currentRegion.BodyRegion = new DomRegion (currentRegion.BodyRegion.Start.Line,
-							                                          currentRegion.BodyRegion.Start.Column,
-							                                          linenum-1, int.MaxValue);
+							currentRegion.BodyRegion = new DomRegion (currentRegion.BodyRegion.BeginLine,
+							                                          currentRegion.BodyRegion.BeginColumn,
+							                                          linenum - 1, int.MaxValue);
 						
 						// Create new chunk region
-						currentRegion = new DomProperty (lineMatch.Groups["chunk"].Value, Modifiers.None, 
-						                                 new DomLocation (linenum, 1), 
-						                                 new DomRegion (linenum, line.Length+1, linenum, int.MaxValue), null);
-						currentFile.Add (currentRegion);
+						currentRegion = new DefaultUnresolvedProperty (currentFile, lineMatch.Groups ["chunk"].Value);
+						currentRegion.Region = currentRegion.BodyRegion = new DomRegion (currentFile.Region.FileName, linenum, line.Length + 1, linenum, int.MaxValue);
+						currentFile.Members.Add (currentRegion);
 					}
 				}
 				++linenum;
@@ -107,13 +103,13 @@ namespace MonoDevelop.VersionControl.Views
 			
 			// Close out trailing regions
 			if (currentFile != null)
-				currentFile.BodyRegion = new DomRegion (currentFile.BodyRegion.Start.Line,
-				                                        currentFile.BodyRegion.Start.Column, 
-				                                        Math.Max (1, linenum-2), int.MaxValue);
+				currentFile.BodyRegion = new DomRegion (currentFile.BodyRegion.BeginLine,
+				                                        currentFile.BodyRegion.BeginColumn, 
+				                                        Math.Max (1, linenum - 2), int.MaxValue);
 			if (currentRegion != null)
-				currentRegion.BodyRegion = new DomRegion (currentRegion.BodyRegion.Start.Line,
-				                                          currentRegion.BodyRegion.Start.Column, 
-				                                          Math.Max (1, linenum-2), int.MaxValue);
+				currentRegion.BodyRegion = new DomRegion (currentRegion.BodyRegion.BeginLine,
+				                                          currentRegion.BodyRegion.BeginColumn, 
+				                                          Math.Max (1, linenum - 2), int.MaxValue);
 			
 			return doc;
 		}
