@@ -88,7 +88,6 @@ namespace Mono.Debugging.Evaluation
 			if (!CanEvaluate (options))
 				return DC.ObjectValue.CreateImplicitNotSupported (this, new ObjectPath (Name), ctx.Adapter.GetTypeName (GetContext (options), Type), Flags);
 			if (withTimeout) {
-				options = options.Clone ();
 				return ctx.Adapter.CreateObjectValueAsync (Name, Flags, delegate {
 					return CreateObjectValue (options);
 				});
@@ -128,7 +127,18 @@ namespace Mono.Debugging.Evaluation
 				name = "?";
 			
 			EvaluationContext newCtx = GetContext (options);
-			object val = Value;
+			EvaluationContext oldCtx = Context;
+			object val = null;
+			
+			try {
+				// Note: The Value property implementation may make use of the EvaluationOptions,
+				// so we need to override our context temporarily to do the evaluation.
+				ctx = newCtx;
+				val = Value;
+			} finally {
+				ctx = oldCtx;
+			}
+			
 			if (val != null)
 				return newCtx.Adapter.CreateObjectValue (newCtx, this, new ObjectPath (name), val, Flags);
 			else
@@ -180,7 +190,7 @@ namespace Mono.Debugging.Evaluation
 			return GetChildren (path, index, count, options);
 		}
 		
-		public virtual string CallToString ( )
+		public virtual string CallToString ()
 		{
 			return ctx.Adapter.CallToString (ctx, Value);
 		}
@@ -200,7 +210,7 @@ namespace Mono.Debugging.Evaluation
 			try {
 				object val = Value;
 				if (ctx.Adapter.IsClassInstance (Context, val))
-					return ctx.Adapter.GetMembersSorted (GetChildrenContext (options), this, ctx.Adapter.GetValueType (Context, val), val);
+					return ctx.Adapter.GetMembersSorted (GetChildrenContext (options), this, Type, val);
 			} catch {
 				// Ignore
 			}
@@ -237,7 +247,7 @@ namespace Mono.Debugging.Evaluation
 			if (obj == null)
 				return null;
 
-			if (ctx.Adapter.IsArray (Context, obj)) {
+			if (name[0] == '[' && ctx.Adapter.IsArray (Context, obj)) {
 				// Parse the array indices
 				string[] sinds = name.Substring (1, name.Length - 2).Split (',');
 				int[] indices = new int [sinds.Length];
