@@ -35,7 +35,7 @@ namespace Mono.TextEditor
 {
 	public abstract class SemanticRule
 	{
-		public abstract void Analyze (Document doc, LineSegment line, Chunk startChunk, int startOffset, int endOffset);
+		public abstract void Analyze (TextDocument doc, DocumentLine line, Chunk startChunk, int startOffset, int endOffset);
 	}
 	
 	public class HighlightUrlSemanticRule : SemanticRule
@@ -52,9 +52,9 @@ namespace Mono.TextEditor
 		}
 		
 		bool inUpdate = false;
-		public override void Analyze (Document doc, LineSegment line, Chunk startChunk, int startOffset, int endOffset)
+		public override void Analyze (TextDocument doc, DocumentLine line, Chunk startChunk, int startOffset, int endOffset)
 		{
-			if (endOffset <= startOffset || startOffset >= doc.Length || inUpdate)
+			if (endOffset <= startOffset || startOffset >= doc.TextLength || inUpdate)
 				return;
 			inUpdate = true;
 			try {
