@@ -65,6 +65,8 @@ namespace Mono.TextEditor.Theatrics
 
 		public AnimatedWidget (Widget widget, uint duration, Easing easing, Blocking blocking, bool horizontal)
 		{
+			Mono.TextEditor.GtkWorkarounds.FixContainerLeak (this);
+			
 			this.horizontal = horizontal;
 			Widget = widget;
 			Duration = duration;
@@ -77,10 +79,6 @@ namespace Mono.TextEditor.Theatrics
 			ShowAll ();
 		}
 
-		protected AnimatedWidget (IntPtr raw) : base(raw)
-		{
-		}
-
 		public double Percent {
 			get { return percent; }
 			set {
