@@ -38,7 +38,6 @@ using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.Serialization;
 using MonoDevelop.Core.ProgressMonitoring;
-using MonoDevelop.Projects.Dom.Output;
 using MonoDevelop.Projects.Policies;
 using MonoDevelop.Projects.Formats.MD1;
 using MonoDevelop.Projects.Extensions;
@@ -94,8 +93,8 @@ namespace MonoDevelop.Projects
 
 		public DotNetProject (string languageName, ProjectCreateInformation projectCreateInfo, XmlElement projectOptions) : this(languageName)
 		{
-			if ((projectOptions != null) && (projectOptions.Attributes["Target"] != null))
-				CompileTarget = (CompileTarget)Enum.Parse (typeof(CompileTarget), projectOptions.Attributes["Target"].Value);
+			if ((projectOptions != null) && (projectOptions.Attributes ["Target"] != null))
+				CompileTarget = (CompileTarget)Enum.Parse (typeof(CompileTarget), projectOptions.Attributes ["Target"].Value);
 			else if (IsLibraryBasedProjectType)
 				CompileTarget = CompileTarget.Library;
 
@@ -110,7 +109,7 @@ namespace MonoDevelop.Projects
 					if (!projectOptions.HasAttribute ("Platform")) {
 						// Clone the element since we are going to change it
 						platform = GetDefaultTargetPlatform (projectCreateInfo);
-						projectOptions = (XmlElement) projectOptions.CloneNode (true);
+						projectOptions = (XmlElement)projectOptions.CloneNode (true);
 						projectOptions.SetAttribute ("Platform", platform);
 					} else
 						platform = projectOptions.GetAttribute ("Platform");
@@ -126,7 +125,15 @@ namespace MonoDevelop.Projects
 				Configurations.Add (configDebug);
 
 				DotNetProjectConfiguration configRelease = CreateConfiguration ("Release" + platformSuffix) as DotNetProjectConfiguration;
-				configRelease.CompilationParameters = languageBinding.CreateCompilationParameters (projectOptions);
+				
+				if (projectOptions != null) {
+					XmlElement releaseProjectOptions = (XmlElement)projectOptions.CloneNode (true);
+					releaseProjectOptions.SetAttribute ("Release", "True");
+					configRelease.CompilationParameters = languageBinding.CreateCompilationParameters (releaseProjectOptions);
+				} else {
+					configRelease.CompilationParameters = languageBinding.CreateCompilationParameters (null);
+				}
+				
 				configRelease.CompilationParameters.RemoveDefineSymbol ("DEBUG");
 				configRelease.DebugMode = false;
 				configRelease.ExternalConsole = externalConsole;
@@ -142,8 +149,9 @@ namespace MonoDevelop.Projects
 				Name = projectCreateInfo.ProjectName;
 				binPath = projectCreateInfo.BinPath;
 				defaultNamespace = SanitisePotentialNamespace (projectCreateInfo.ProjectName);
-			} else
+			} else {
 				binPath = ".";
+			}
 
 			foreach (DotNetProjectConfiguration dotNetProjectConfig in Configurations) {
 				dotNetProjectConfig.OutputDirectory = Path.Combine (binPath, dotNetProjectConfig.Name);
@@ -160,15 +168,6 @@ namespace MonoDevelop.Projects
 			get { return "DotNet"; }
 		}
 
-		private Ambience ambience;
-		public override Ambience Ambience {
-			get {
-				if (ambience == null)
-					ambience = AmbienceService.GetAmbienceForLanguage (LanguageName);
-				return ambience;
-			}
-		}
-
 		private string languageName;
 		public string LanguageName {
 			get { return languageName; }
@@ -233,7 +232,10 @@ namespace MonoDevelop.Projects
 					languageParameters.ParentProject = this;
 			}
 		}
-
+		
+		/// <summary>
+		/// Default namespace setting. May be empty, use GetDefaultNamespace to get a usable value.
+		/// </summary>
 		public string DefaultNamespace {
 			get { return defaultNamespace; }
 			set {
@@ -485,15 +487,22 @@ namespace MonoDevelop.Projects
 		}
 		
 		[ThreadStatic]
-		static int supportReferDistance = -1;
+		static int supportReferDistance;
+		[ThreadStatic]
+		static HashSet<DotNetProject> processedProjects;
 
 		internal protected override void PopulateSupportFileList (FileCopySet list, ConfigurationSelector configuration)
 		{
 			try {
+				if (supportReferDistance == 0)
+					processedProjects = new HashSet<DotNetProject> ();
 				supportReferDistance++;
+
 				PopulateSupportFileListInternal (list, configuration);
 			} finally {
 				supportReferDistance--;
+				if (supportReferDistance == 0)
+					processedProjects = null;
 			}
 		}
 
@@ -535,8 +544,9 @@ namespace MonoDevelop.Projects
 
 					//VS COMPAT: recursively copy references's "local copy" files
 					//but only copy the "copy to output" files from the immediate references
-					foreach (var f in p.GetSupportFileList (configuration))
-						list.Add (f.Src, f.CopyOnlyIfNewer, f.Target);
+					if (processedProjects.Add (p))
+						foreach (var f in p.GetSupportFileList (configuration))
+							list.Add (f.Src, f.CopyOnlyIfNewer, f.Target);
 
 					DotNetProjectConfiguration refConfig = p.GetConfiguration (configuration) as DotNetProjectConfiguration;
 
@@ -888,7 +898,11 @@ namespace MonoDevelop.Projects
 				return false;
 			return LanguageBinding.IsSourceCodeFile (fileName);
 		}
-
+		
+		/// <summary>
+		/// Gets the default namespace for the file, according to the naming policy.
+		/// </summary>
+		/// <remarks>Always returns a valid namespace, even if the fileName is null.</remarks>
 		public virtual string GetDefaultNamespace (string fileName)
 		{
 			DotNetNamingPolicy pol = Policies.Get<DotNetNamingPolicy> ();
@@ -898,6 +912,10 @@ namespace MonoDevelop.Projects
 			string defaultNmspc = !string.IsNullOrEmpty (DefaultNamespace)
 				? DefaultNamespace
 				: SanitisePotentialNamespace (Name) ?? "Application";
+			
+			if (string.IsNullOrEmpty (fileName)) {
+				return defaultNmspc;
+			}
 
 			string dirname = Path.GetDirectoryName (fileName);
 			string relativeDirname = null;
@@ -914,7 +932,8 @@ namespace MonoDevelop.Projects
 						root = defaultNmspc;
 						goto case DirectoryNamespaceAssociation.Flat;
 					case DirectoryNamespaceAssociation.Flat:
-						dirNamespc = SanitisePotentialNamespace (relativeDirname);
+						//use the last component only
+						dirNamespc = SanitisePotentialNamespace (Path.GetFileName (relativeDirname));
 						break;
 
 					case DirectoryNamespaceAssociation.PrefixedHierarchical:
@@ -1024,9 +1043,6 @@ namespace MonoDevelop.Projects
 			IResourceHandler handler = ItemHandler as IResourceHandler;
 			if (handler != null)
 				MigrateResourceIds (handler, ResourceHandler);
-			
-			if (String.IsNullOrEmpty (defaultNamespace))
-				defaultNamespace = SanitisePotentialNamespace (Name);
 
 			base.OnEndLoad ();
 		}
