@@ -499,7 +499,7 @@ namespace MonoDevelop.Projects
 					return project;
 				}
 			}
-			return null;
+			throw new InvalidOperationException ("Project type '" + type + "' not found");
 		}
 		
 		public Solution GetWrapperSolution (IProgressMonitor monitor, string filename)
@@ -777,16 +777,6 @@ namespace MonoDevelop.Projects
 		{
 			return callback (monitor, item, buildData);
 		}
-		
-		public override void PopulateSupportFileList (Project project, FileCopySet list, ConfigurationSelector configuration)
-		{
-			project.PopulateSupportFileList (list, configuration);
-		}
-		
-		public override void PopulateOutputFileList (Project project, List<FilePath> list, ConfigurationSelector configuration)
-		{
-			project.PopulateOutputFileList (list, configuration);
-		}
 	}	
 	
 	internal static class Counters
