@@ -30,10 +30,11 @@ using System;
 using System.Collections.Generic;
 using System.Linq;
 using MonoDevelop.Core;
+using ICSharpCode.NRefactory.Completion;
 
 namespace MonoDevelop.Ide.CodeCompletion
 {
-	public class CompletionData
+	public class CompletionData : ICompletionData
 	{
 		protected CompletionData () {}
 		
@@ -41,22 +42,28 @@ namespace MonoDevelop.Ide.CodeCompletion
 		public virtual string DisplayText { get; set; }
 		public virtual string Description { get; set; }
 		public virtual string CompletionText { get; set; }
+
 		public virtual string DisplayDescription { get; set; }
 		public virtual CompletionCategory CompletionCategory { get; set; }
 		public virtual DisplayFlags DisplayFlags { get; set; }
 		
-		public virtual bool IsOverloaded { 
+		public virtual bool HasOverloads { 
 			get {
 				return false;
 			}
 		}
 		
-		public virtual IEnumerable<CompletionData> OverloadedData {
+		public virtual IEnumerable<ICompletionData> OverloadedData {
 			get {
 				throw new System.InvalidOperationException ();
 			}
 		}
 		
+		public virtual void AddOverload (ICompletionData data)
+		{
+			throw new System.InvalidOperationException ();
+		}
+		
 		public CompletionData (string text) : this (text, null, null) {}
 		public CompletionData (string text, IconId icon) : this (text, icon, null) {}
 		public CompletionData (string text, IconId icon, string description) : this (text, icon, description, text) {}
@@ -73,14 +80,15 @@ namespace MonoDevelop.Ide.CodeCompletion
 		{
 			int partialWordLength = window.PartialWord != null ? window.PartialWord.Length : 0;
 			int replaceLength = window.CodeCompletionContext.TriggerWordLength + partialWordLength - window.InitialWordLength;
-			return window.CompletionWidget.GetText (window.CodeCompletionContext.TriggerOffset, window.CodeCompletionContext.TriggerOffset + replaceLength);
+			int endOffset = Math.Min (window.CodeCompletionContext.TriggerOffset + replaceLength, window.CompletionWidget.TextLength);
+			var result = window.CompletionWidget.GetText (window.CodeCompletionContext.TriggerOffset, endOffset);
+			return result;
 		}
 
 		public virtual void InsertCompletionText (CompletionListWindow window, ref KeyActions ka, Gdk.Key closeChar, char keyChar, Gdk.ModifierType modifier)
 		{
-			if (CompletionText == GetCurrentWord (window)) 
-				return;
-			window.CompletionWidget.SetCompletionText (window.CodeCompletionContext, GetCurrentWord (window), CompletionText);
+			var currentWord = GetCurrentWord (window);
+			window.CompletionWidget.SetCompletionText (window.CodeCompletionContext, currentWord, CompletionText);
 		}
 		
 		public override string ToString ()
