@@ -33,6 +33,8 @@ using MonoDevelop.Projects;
 using MonoDevelop.Core;
 using MonoDevelop.Components;
 using System.Linq;
+using ICSharpCode.NRefactory.Completion;
+using MonoDevelop.Ide.Gui.Content;
 
 namespace MonoDevelop.Ide.CodeCompletion
 {
@@ -41,9 +43,8 @@ namespace MonoDevelop.Ide.CodeCompletion
 		const int declarationWindowMargin = 3;
 		
 		DeclarationViewWindow declarationviewwindow = new DeclarationViewWindow ();
-		CompletionData currentData;
+		ICompletionData currentData;
 		Widget parsingMessage;
-		System.Action closedDelegate;
 		int initialWordLength;
 		int previousWidth = -1, previousHeight = -1;
 		
@@ -51,12 +52,7 @@ namespace MonoDevelop.Ide.CodeCompletion
 			get;
 			set;
 		}
-		
-		public ICompletionWidget CompletionWidget {
-			get;
-			set;
-		}
-		
+
 		public int X { get; private set; }
 		public int Y { get; private set; }
 		
@@ -75,6 +71,7 @@ namespace MonoDevelop.Ide.CodeCompletion
 		
 		public CompletionListWindow ()
 		{
+			TypeHint = Gdk.WindowTypeHint.Utility;
 			SizeAllocated += new SizeAllocatedHandler (ListSizeChanged);
 			Events = Gdk.EventMask.PropertyChangeMask;
 			WindowTransparencyDecorator.Attach (this);
@@ -111,11 +108,6 @@ namespace MonoDevelop.Ide.CodeCompletion
 				completionDataList = null;
 			}
 
-			if (closedDelegate != null) {
-				closedDelegate ();
-				closedDelegate = null;
-			}
-			
 			HideDeclarationView ();
 			
 			if (declarationviewwindow != null) {
@@ -125,10 +117,23 @@ namespace MonoDevelop.Ide.CodeCompletion
 			base.OnDestroyed ();
 		}
 
-		public void PostProcessKeyEvent (KeyActions ka, Gdk.Key key, char keyChar, Gdk.ModifierType modifier)
+		public void PostProcessKeyEvent (Gdk.Key key, char keyChar, Gdk.ModifierType modifier)
 		{
+			KeyActions ka = KeyActions.None;
+			bool keyHandled = false;
+			foreach (var handler in CompletionDataList.KeyHandler) {
+				if (handler.PostProcessKey (this, key, keyChar, modifier, out ka)) {
+					keyHandled = true;
+					break;
+				}
+			}
+			
+			if (!keyHandled)
+				ka = PostProcessKey (key, keyChar, modifier);
 			if ((ka & KeyActions.Complete) != 0) 
 				CompleteWord (ref ka, key, keyChar, modifier);
+			if ((ka & KeyActions.CloseWindow) != 0)
+				CompletionWindowManager.HideWindow ();
 		}
 		
 		public void ToggleCategoryMode ()
@@ -138,30 +143,21 @@ namespace MonoDevelop.Ide.CodeCompletion
 			this.List.QueueDraw ();
 		}
 		
-		public bool PreProcessKeyEvent (Gdk.Key key, char keyChar, Gdk.ModifierType modifier, out KeyActions ka)
+		public bool PreProcessKeyEvent (Gdk.Key key, char keyChar, Gdk.ModifierType modifier)
 		{
-			ka = KeyActions.None;
+			KeyActions ka = KeyActions.None;
 			bool keyHandled = false;
 			foreach (ICompletionKeyHandler handler in CompletionDataList.KeyHandler) {
-				if (handler.ProcessKey (this, key, keyChar, modifier, out ka)) {
+				if (handler.PreProcessKey (this, key, keyChar, modifier, out ka)) {
 					keyHandled = true;
 					break;
 				}
 			}
 			
-			if (!keyHandled) {
-				ka = ProcessKey (key, keyChar, modifier);
-			}
-			
-			if ((ka & KeyActions.Complete) != 0) {
-				//bool completed =
+			if (!keyHandled)
+				ka = PreProcessKey (key, keyChar, modifier);
+			if ((ka & KeyActions.Complete) != 0)
 				CompleteWord (ref ka, key, keyChar, modifier);
-				//NOTE: this passes the enter keystroke through to the editor if the current item is an exact match
-				//if (!completed) {
-				//	CompletionWindowManager.HideWindow ();
-				//	return false;
-				//}
-			}
 
 			if ((ka & KeyActions.CloseWindow) != 0)
 				CompletionWindowManager.HideWindow ();
@@ -208,19 +204,26 @@ namespace MonoDevelop.Ide.CodeCompletion
 			UpdateDeclarationView ();
 		}
 		
-		internal bool ShowListWindow (char firstChar, ICompletionDataList list, ICompletionWidget completionWidget, CodeCompletionContext completionContext, System.Action closedDelegate)
+		internal bool ShowListWindow (char firstChar, ICompletionDataList list, ICompletionWidget completionWidget, CodeCompletionContext completionContext)
 		{
+			if (list == null)
+				throw new ArgumentNullException ("list");
+			if (completionContext == null)
+				throw new ArgumentNullException ("completionContext");
+			if (completionContext == null)
+				throw new ArgumentNullException ("completionContext");
 			if (mutableList != null) {
 				mutableList.Changing -= OnCompletionDataChanging;
 				mutableList.Changed -= OnCompletionDataChanged;
 				HideFooter ();
 			}
-			//initialWordLength = 0;
-			this.completionDataList = list;
-			this.CompleteWithSpaceOrPunctuation = MonoDevelop.Core.PropertyService.Get ("CompleteWithSpaceOrPunctuation", true);
+			ResetState ();
+			CompletionWidget = completionWidget;
+			CompletionDataList = list;
+
+			CompleteWithSpaceOrPunctuation = PropertyService.Get ("CompleteWithSpaceOrPunctuation", true);
 			
-			this.CodeCompletionContext = completionContext;
-			this.closedDelegate = closedDelegate;
+			CodeCompletionContext = completionContext;
 			mutableList = completionDataList as IMutableCompletionDataList;
 			List.PreviewCompletionString = completionDataList.CompletionSelectionMode == CompletionSelectionMode.OwnTextField;
 
@@ -231,57 +234,54 @@ namespace MonoDevelop.Ide.CodeCompletion
 				if (mutableList.IsChanging)
 					OnCompletionDataChanging (null, null);
 			}
-
-			this.CompletionWidget = completionWidget;
-
 			if (FillList ()) {
-// not neccessarry, because list window is not reused anymore:
-//				Reset (true);
 				this.AutoSelect = list.AutoSelect;
 				this.AutoCompleteEmptyMatch = list.AutoCompleteEmptyMatch;
+				this.CloseOnSquareBrackets = list.CloseOnSquareBrackets;
 				// makes control-space in midle of words to work
 				string text = completionWidget.GetCompletionText (completionContext);
 				DefaultCompletionString = completionDataList.DefaultCompletionString ?? "";
 				if (text.Length == 0) {
 					UpdateWordSelection ();
-					initialWordLength = 0;//completionWidget.SelectedLength;
+					initialWordLength = 0;
+					//completionWidget.SelectedLength;
+					StartOffset = completionWidget.CaretOffset;
 					ResetSizes ();
 					ShowAll ();
 					UpdateWordSelection ();
 					UpdateDeclarationView ();
-					
 					//if there is only one matching result we take it by default
 					if (completionDataList.AutoCompleteUniqueMatch && IsUniqueMatch && !IsChanging) {
 						CompleteWord ();
 						CompletionWindowManager.HideWindow ();
+						return false;
 					}
 					return true;
 				}
 
-				initialWordLength = text.Length /*+ completionWidget.SelectedLength*/;
-				PartialWord = text;
+				initialWordLength = completionWidget.SelectedLength > 0 ? 0 : text.Length;
+				StartOffset = completionWidget.CaretOffset - initialWordLength;
 				HideWhenWordDeleted = initialWordLength != 0;
+				ResetSizes ();
 				UpdateWordSelection ();
-			
 				//if there is only one matching result we take it by default
 				if (completionDataList.AutoCompleteUniqueMatch && IsUniqueMatch && !IsChanging) {
 					CompleteWord ();
 					CompletionWindowManager.HideWindow ();
+					return false;
 				} else {
-					ResetSizes ();
 					ShowAll ();
 					UpdateDeclarationView ();
 				}
 				return true;
 			}
 			CompletionWindowManager.HideWindow ();
-			
 			return false;
 		}
 		
-		class DataItemComparer : IComparer<CompletionData>
+		class DataItemComparer : IComparer<ICompletionData>
 		{
-			public int Compare (CompletionData a, CompletionData b)
+			public int Compare (ICompletionData a, ICompletionData b)
 			{
 				return ((a.DisplayFlags & DisplayFlags.Obsolete) == (b.DisplayFlags & DisplayFlags.Obsolete))
 					? StringComparer.OrdinalIgnoreCase.Compare (a.DisplayText, b.DisplayText)
@@ -296,15 +296,18 @@ namespace MonoDevelop.Ide.CodeCompletion
 			
 			this.Style = CompletionWidget.GtkStyle;
 			
+			if (PropertyService.Get ("HideObsoleteItems", false)) {
+				foreach (var item in completionDataList.Where (x => x.DisplayFlags.HasFlag (DisplayFlags.Obsolete)).ToList ())
+					completionDataList.Remove (item);
+			}
+			
 			//sort, sinking obsolete items to the bottoms
 			//the string comparison is ordinal as that makes it an order of magnitude faster, which 
 			//which makes completion triggering noticeably more responsive
 			if (!completionDataList.IsSorted)
 				completionDataList.Sort (new DataItemComparer ());
-			List.FilterWords ();
 			
 			Reposition (true);
-			
 			return true;
 		}
 		
@@ -349,7 +352,6 @@ namespace MonoDevelop.Ide.CodeCompletion
 			curYPos = Y;
 			Move (X, Y);
 			UpdateDeclarationView ();
-			ParameterInformationWindowManager.UpdateWindow (CompletionWidget);
 		}
 		
 		//smaller lists get size reallocated after FillList, so we have to reposition them
@@ -368,15 +370,15 @@ namespace MonoDevelop.Ide.CodeCompletion
 		
 		public bool CompleteWord (ref KeyActions ka, Gdk.Key closeChar, char keyChar, Gdk.ModifierType modifier)
 		{
-			if (SelectionIndex == -1 || completionDataList == null)
+			if (SelectedItem == -1 || completionDataList == null)
 				return false;
-			CompletionData item = completionDataList [SelectionIndex];
+			var item = completionDataList [SelectedItem];
 			if (item == null)
 				return false;
 			// first close the completion list, then insert the text.
 			// this is required because that's the logical event chain, otherwise things could be messed up
 			CloseCompletionList ();
-			item.InsertCompletionText (this, ref ka, closeChar, keyChar, modifier);
+			((CompletionData)item).InsertCompletionText (this, ref ka, closeChar, keyChar, modifier);
 			AddWordToHistory (PartialWord, item.CompletionText);
 			OnWordCompleted (new CodeCompletionContextEventArgs (CompletionWidget, CodeCompletionContext, item.CompletionText));
 			return true;
@@ -396,6 +398,18 @@ namespace MonoDevelop.Ide.CodeCompletion
 			UpdateDeclarationView ();
 		}
 
+		protected override void OnHidden ()
+		{
+			HideDeclarationView ();
+			base.OnHidden ();
+		}
+
+		public void HideWindow ()
+		{
+			Hide ();
+			HideDeclarationView ();
+		}
+
 		protected override void DoubleClick ()
 		{
 			CompleteWord ();
@@ -415,47 +429,51 @@ namespace MonoDevelop.Ide.CodeCompletion
 		
 		void UpdateDeclarationView ()
 		{
-			if (completionDataList == null || List.Selection >= completionDataList.Count || List.Selection == -1)
+			if (completionDataList == null || List.SelectionFilterIndex >= completionDataList.Count || List.SelectionFilterIndex == -1)
 				return;
 			if (List.GdkWindow == null)
 				return;
 			RemoveDeclarationViewTimer ();
 			// no selection, try to find a selection
-			if (List.SelectionIndex < 0 || List.SelectionIndex >= completionDataList.Count) {
+			if (List.SelectedItem < 0 || List.SelectedItem >= completionDataList.Count) {
 				List.CompletionString = PartialWord;
 				bool hasMismatches;
-				List.Selection = FindMatchedEntry (List.CompletionString, out hasMismatches);
+				List.SelectionFilterIndex = FindMatchedEntry (List.CompletionString, out hasMismatches);
 			}
 			// no success, hide declaration view
-			if (List.SelectionIndex < 0 || List.SelectionIndex >= completionDataList.Count) {
+			if (List.SelectedItem < 0 || List.SelectedItem >= completionDataList.Count) {
 				HideDeclarationView ();
 				return;
 			}
-			CompletionData data = completionDataList[List.SelectionIndex];
+			var data = completionDataList [List.SelectedItem];
 			
-			IList<CompletionData> overloads;
-			if (data.IsOverloaded) {
-				overloads = new List<CompletionData> (data.OverloadedData);
+			IEnumerable<ICompletionData> filteredOverloads;
+			if (data.HasOverloads) {
+				filteredOverloads = data.OverloadedData;
+				if (PropertyService.Get ("HideObsoleteItems", false))
+					filteredOverloads = filteredOverloads.Where (x => !x.DisplayFlags.HasFlag (DisplayFlags.Obsolete));
 			} else {
-				overloads = new CompletionData[] { data };
+				filteredOverloads = new ICompletionData[] { data };
 			}
 			
+			var overloads = new List<ICompletionData> (filteredOverloads);
+			
 			if (data != currentData) {
 				HideDeclarationView ();
 				
 				declarationviewwindow.Clear ();
 				declarationviewwindow.Realize ();
 				
-				foreach (CompletionData overload in overloads) {
+				foreach (var overload in overloads) {
 					bool hasMarkup = (overload.DisplayFlags & DisplayFlags.DescriptionHasMarkup) != 0;
 					declarationviewwindow.AddOverload (hasMarkup ? overload.Description : GLib.Markup.EscapeText (overload.Description));
 				}
 				
-				declarationviewwindow.Multiple = data.IsOverloaded;
+				declarationviewwindow.Multiple = data.HasOverloads;
 				currentData = data;
-				if (data.IsOverloaded) {
+				if (data.HasOverloads) {
 					for (int i = 0; i < overloads.Count; i++) {
-						if ((overloads[i].DisplayFlags & DisplayFlags.Obsolete) != DisplayFlags.Obsolete) {
+						if (!overloads[i].DisplayFlags.HasFlag (DisplayFlags.Obsolete)) {
 							declarationviewwindow.CurrentOverload = i;
 							break;
 						}
@@ -523,12 +541,14 @@ namespace MonoDevelop.Ide.CodeCompletion
 		
 		bool DelayedTooltipShow ()
 		{
-			Gdk.Rectangle rect = List.GetRowArea (List.Selection);
+			Gdk.Rectangle rect = List.GetRowArea (List.SelectionFilterIndex);
 			if (rect.IsEmpty)
 				return false;
-			int listpos_x = 0, listpos_y = 0;
-			while (listpos_x == 0 || listpos_y == 0)
+			int listpos_x = 0, listpos_y = 0, i = 0;
+			while ((listpos_x == 0 || listpos_y == 0) && (i++ < 10))
 				GetPosition (out listpos_x, out listpos_y);
+			if (i >= 10)
+				return false;
 			int vert = listpos_y + rect.Y;
 			int lvWidth = 0, lvHeight = 0;
 			while (lvWidth == 0)
@@ -545,7 +565,7 @@ namespace MonoDevelop.Ide.CodeCompletion
 				vert = listpos_y;
 			}*/
 			
-			if (declarationViewHidden) {
+			if (declarationViewHidden && Visible) {
 				declarationviewwindow.Move (this.Screen.Width + 1, vert);
 				declarationviewwindow.SetFixedWidth (-1);
 				declarationviewwindow.ReshowWithInitialSize ();
@@ -581,6 +601,13 @@ namespace MonoDevelop.Ide.CodeCompletion
 			return false;
 		}
 		
+		protected override void ResetState ()
+		{
+			StartOffset = 0;
+			previousWidth = previousHeight = -1;
+			base.ResetState ();
+		}
+		
 		#region IListDataProvider
 		
 		int IListDataProvider.ItemCount 
@@ -600,20 +627,20 @@ namespace MonoDevelop.Ide.CodeCompletion
 		
 		string IListDataProvider.GetDescription (int n)
 		{
-			return completionDataList[n].DisplayDescription;
+			return ((CompletionData)completionDataList[n]).DisplayDescription;
 		}
 		
 		bool IListDataProvider.HasMarkup (int n)
 		{
-			return (completionDataList[n].DisplayFlags & DisplayFlags.Obsolete) != 0;
+			return completionDataList[n].DisplayFlags.HasFlag (DisplayFlags.Obsolete);
 		}
 		
 		//NOTE: we only ever return markup for items marked as obsolete
 		string IListDataProvider.GetMarkup (int n)
 		{
-			CompletionData completionData = completionDataList[n];
-			if (!completionData.IsOverloaded && (completionData.DisplayFlags & DisplayFlags.Obsolete) == DisplayFlags.Obsolete || 
-			    completionData.OverloadedData.All (data => (data.DisplayFlags & DisplayFlags.Obsolete) == DisplayFlags.Obsolete))
+			var completionData = completionDataList[n];
+			if (!completionData.HasOverloads && completionData.DisplayFlags.HasFlag (DisplayFlags.Obsolete) || 
+			    completionData.OverloadedData.All (data => data.DisplayFlags.HasFlag (DisplayFlags.Obsolete)))
 				return "<s>" + GLib.Markup.EscapeText (completionDataList[n].DisplayText) + "</s>";
 			return GLib.Markup.EscapeText (completionDataList[n].DisplayText);
 		}
@@ -625,7 +652,7 @@ namespace MonoDevelop.Ide.CodeCompletion
 		
 		Gdk.Pixbuf IListDataProvider.GetIcon (int n)
 		{
-			string iconName = completionDataList[n].Icon;
+			string iconName = ((CompletionData)completionDataList[n]).Icon;
 			if (string.IsNullOrEmpty (iconName))
 				return null;
 			return ImageService.GetPixbuf (iconName, Gtk.IconSize.Menu);
@@ -668,7 +695,7 @@ namespace MonoDevelop.Ide.CodeCompletion
 				var tmp = this.List.AutoSelect;
 				// Fill the list before resetting so that we get the correct size
 				FillList ();
-				Reset (false);
+				ResetSizes ();
 				this.List.AutoSelect = tmp;
 				if (last != null )
 					SelectEntry (last);
