@@ -279,7 +279,12 @@ namespace MonoDevelop.Ide.Commands
 			try {
 				string title = GettextCatalog.GetString ("Clear recent files");
 				string question = GettextCatalog.GetString ("Are you sure you want to clear recent files list?");
-				if (MessageService.Confirm (title, question, AlertButton.Clear)) {
+				if (MessageService.GenericAlert (
+					MonoDevelop.Ide.Gui.Stock.Question,
+					title,
+					question,
+					AlertButton.No,
+					AlertButton.Yes) == AlertButton.Yes) {
 					DesktopService.RecentFiles.ClearFiles ();
 				}
 			} catch (Exception ex) {
@@ -338,8 +343,8 @@ namespace MonoDevelop.Ide.Commands
 		protected override void Run (object dataItem)
 		{
 			string filename = (string)dataItem;
-			Gdk.ModifierType mtype;
-			bool inWorkspace = Gtk.Global.GetCurrentEventState (out mtype) && (mtype & Gdk.ModifierType.ControlMask) != 0;
+			Gdk.ModifierType mtype = Mono.TextEditor.GtkWorkarounds.GetCurrentKeyModifiers ();
+			bool inWorkspace = (mtype & Gdk.ModifierType.ControlMask) != 0;
 			IdeApp.Workspace.OpenWorkspaceItem (filename, !inWorkspace);
 		}
 	}
@@ -347,12 +352,17 @@ namespace MonoDevelop.Ide.Commands
 	// MonoDevelop.Ide.Commands.FileCommands.ClearRecentProjects
 	internal class ClearRecentProjectsHandler : CommandHandler
 	{
-		protected override void Run()
+		protected override void Run ()
 		{
 			try {
 				string title = GettextCatalog.GetString ("Clear recent projects");
 				string question = GettextCatalog.GetString ("Are you sure you want to clear recent projects list?");
-				if (MessageService.Confirm (title, question, AlertButton.Clear)) {
+				if (MessageService.GenericAlert (
+					MonoDevelop.Ide.Gui.Stock.Question,
+					title,
+					question,
+					AlertButton.No,
+					AlertButton.Yes) == AlertButton.Yes) {
 					DesktopService.RecentFiles.ClearProjects ();
 				}
 			} catch (Exception ex) {
