@@ -63,9 +63,9 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		NodeBuilder[] builders;
 		Dictionary<Type, NodeBuilder[]> builderChains = new Dictionary<Type, NodeBuilder[]> ();
-		Hashtable nodeHash = new Hashtable ();
+		NodeHashtable nodeHash = new NodeHashtable ();
 		
-		ContextMenuTreeView tree = new ContextMenuTreeView ();
+		ExtensibleTreeViewTree tree;
 		Gtk.TreeStore store;
 		internal Gtk.TreeViewColumn complete_column;
 		internal ZoomableCellRendererPixbuf pix_render;
@@ -77,14 +77,12 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		TreePadOption[] options;
 		TreeOptions globalOptions;
-		Dictionary<Gtk.TreeIter, TreeOptions> nodeOptions;
-		
+
 		TreeNodeNavigator workNode;
 		TreeNodeNavigator compareNode1;
 		TreeNodeNavigator compareNode2;
 		internal bool sorting;
-		
-		object[] dragObjects;
+
 		object[] copyObjects;
 		DragOperation currentTransferOperation;
 
@@ -93,12 +91,6 @@ namespace MonoDevelop.Ide.Gui.Components
 		string contextMenuPath;
 		IDictionary<string,string> contextMenuTypeNameAliases;
 		
-		private static Gtk.TargetEntry [] target_table = new Gtk.TargetEntry [] {
-			new Gtk.TargetEntry ("text/uri-list", 0, 11 ),
-			new Gtk.TargetEntry ("text/plain", 0, 22),
-			new Gtk.TargetEntry ("application/x-rootwindow-drop", 0, 33)
-		};
-		
 		public IDictionary<string,string> ContextMenuTypeNameAliases {
 			get { return contextMenuTypeNameAliases; }
 			set { contextMenuTypeNameAliases = value; }
@@ -118,17 +110,12 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		public string Id { get; set; }
 
-		internal object[] DragObjects {
-			get {
-				return this.dragObjects;
-			}
-		}
-		
 		public ExtensibleTreeView ()
 		{
+			tree = new ExtensibleTreeViewTree (this);
 		}
 		
-		public ExtensibleTreeView (NodeBuilder[] builders, TreePadOption[] options)
+		public ExtensibleTreeView (NodeBuilder[] builders, TreePadOption[] options) : this ()
 		{
 			Initialize (builders, options);
 		}
@@ -180,10 +167,6 @@ namespace MonoDevelop.Ide.Gui.Components
 			store = new Gtk.TreeStore (typeof(string), typeof(Gdk.Pixbuf), typeof(Gdk.Pixbuf), typeof(object), typeof(object), typeof(bool));
 			tree.Model = store;
 			tree.Selection.Mode = Gtk.SelectionMode.Multiple;
-			nodeOptions = new Dictionary<Gtk.TreeIter, TreeOptions> (new IterComparer (store));
-
-			tree.EnableModelDragDest (target_table, Gdk.DragAction.Copy | Gdk.DragAction.Move);
-			Gtk.Drag.SourceSet (tree, Gdk.ModifierType.Button1Mask, target_table, Gdk.DragAction.Copy | Gdk.DragAction.Move);
 			
 			store.DefaultSortFunc = new Gtk.TreeIterCompareFunc (CompareNodes);
 			store.SetSortColumnId (/* GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID */ -1, Gtk.SortType.Ascending);
@@ -227,15 +210,15 @@ namespace MonoDevelop.Ide.Gui.Components
 			compareNode1 = new TreeNodeNavigator (this);
 			compareNode2 = new TreeNodeNavigator (this);
 			
-			tree.DragBegin += OnDragBegin;
-			tree.DragDataReceived += OnDragDataReceived;
-			tree.DragDrop += OnDragDrop;
-			tree.DragEnd += OnDragEnd;
-			tree.DragMotion += OnDragMotion;
-			
 			tree.CursorChanged += OnSelectionChanged;
 			tree.KeyPressEvent += OnKeyPress;
 
+			if (GtkGestures.IsSupported) {
+				tree.AddGestureMagnifyHandler ((sender, args) => {
+					Zoom += Zoom * (args.Magnification / 4d);
+				});
+			}
+
 			for (int n=3; n<16; n++) {
 				Gtk.Rc.ParseString ("style \"MonoDevelop.ExtensibleTreeView_" + n + "\" {\n GtkTreeView::expander-size = " + n + "\n }\n");
 				Gtk.Rc.ParseString ("widget \"*.MonoDevelop.ExtensibleTreeView_" + n + "\" style  \"MonoDevelop.ExtensibleTreeView_" + n + "\"\n");
@@ -320,7 +303,6 @@ namespace MonoDevelop.Ide.Gui.Components
 			globalOptions = new TreeOptions ();
 			foreach (TreePadOption op in options)
 				globalOptions [op.Id] = op.DefaultValue;
-				
 			globalOptions.Pad = this;
 			
 			// Check that there is only one TypeNodeBuilder per type
@@ -329,6 +311,8 @@ namespace MonoDevelop.Ide.Gui.Components
 			foreach (NodeBuilder nb in builders) {
 				TypeNodeBuilder tnb = nb as TypeNodeBuilder;
 				if (tnb != null) {
+					if (tnb.UseReferenceEquality)
+						nodeHash.RegisterByRefType (tnb.NodeDataType);
 					TypeNodeBuilder other = (TypeNodeBuilder) bc [tnb.NodeDataType];
 					if (other != null)
 						throw new ApplicationException (string.Format ("The type node builder {0} can't be used in this context because the type {1} is already handled by {2}", nb.GetType(), tnb.NodeDataType, other.GetType()));
@@ -344,62 +328,36 @@ namespace MonoDevelop.Ide.Gui.Components
 				nb.SetContext (builderContext);
 		}
 
-		void OnDragBegin (object o, Gtk.DragBeginArgs arg)
+		public void EnableDragUriSource (Func<object,string> nodeToUri)
+		{
+			tree.EnableDragUriSource (nodeToUri);
+		}
+
+		object[] GetDragObjects (out Gdk.Pixbuf icon)
 		{
 			ITreeNavigator[] navs = GetSelectedNodes ();
-			if (navs.Length == 0) return;
-			dragObjects = new object [navs.Length];
+			if (navs.Length == 0) {
+				icon = null;
+				return null;
+			}
+			var dragObjects = new object [navs.Length];
 			for (int n=0; n<navs.Length; n++)
 				dragObjects [n] = navs [n].DataItem;
-			Gdk.Pixbuf px = (Gdk.Pixbuf) store.GetValue (navs[0].CurrentPosition._iter, OpenIconColumn);
-			Gtk.Drag.SetIconPixbuf (arg.Context, px, -10, -10);
+			icon = (Gdk.Pixbuf) store.GetValue (navs[0].CurrentPosition._iter, OpenIconColumn);
+			return dragObjects;
 		}
-		
-		void OnDragDataReceived (object o, Gtk.DragDataReceivedArgs args)
-		{
-			if (dragObjects != null) {
-				bool res = CheckAndDrop (args.X, args.Y, true, args.Context, dragObjects);
-				Gtk.Drag.Finish (args.Context, res, true, args.Time);
-			} else {
-				bool res = CheckAndDrop (args.X, args.Y, true, args.Context, new object[] { args.SelectionData });
-//				string fullData = System.Text.Encoding.UTF8.GetString (args.SelectionData.Data);
-				Gtk.Drag.Finish (args.Context, res, true, args.Time);
-			}
-		}
-		
-		void OnDragDrop (object o, Gtk.DragDropArgs args)
-		{
-			if (dragObjects != null) {
-				bool res = CheckAndDrop (args.X, args.Y, true, args.Context, dragObjects);
-				Gtk.Drag.Finish (args.Context, res, true, args.Time);
-			}
-		}
-		
-		void OnDragEnd (object o, Gtk.DragEndArgs args)
-		{
-			dragObjects = null;
-		}
-		
-		[GLib.ConnectBefore]
-		void OnDragMotion (object o, Gtk.DragMotionArgs args)
-		{
-			if (dragObjects != null) {
-				if (!CheckAndDrop (args.X, args.Y, false, args.Context, dragObjects)) {
-					Gdk.Drag.Status (args.Context, (Gdk.DragAction)0, args.Time);
-					args.RetVal = true;
-				}
-			}
-		}
-		
+
 		bool CheckAndDrop (int x, int y, bool drop, Gdk.DragContext ctx, object[] obj)
 		{
 			Gtk.TreePath path;
 			Gtk.TreeViewDropPosition pos;
-			if (!tree.GetDestRowAtPos (x, y, out path, out pos)) return false;
-			
+			if (!tree.GetDestRowAtPos (x, y, out path, out pos))
+				return false;
+
 			Gtk.TreeIter iter;
-			if (!store.GetIter (out iter, path)) return false;
-			
+			if (!store.GetIter (out iter, path))
+				return false;
+
 			TreeNodeNavigator nav = new TreeNodeNavigator (this, iter);
 			NodeBuilder[] chain = nav.BuilderChain;
 			bool foundHandler = false;
@@ -458,7 +416,6 @@ namespace MonoDevelop.Ide.Gui.Components
 				TransactedNodeStore store = transactionStore;
 				transactionStore = null;
 				store.CommitChanges ();
-				CleanupNodeOptions ();
 			}
 		}
 
@@ -486,7 +443,7 @@ namespace MonoDevelop.Ide.Gui.Components
 			}
 		}
 
-		public Hashtable NodeHash {
+		NodeHashtable NodeHash {
 			get {
 				return nodeHash;
 			}
@@ -554,7 +511,7 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		public void Clear ()
 		{
-			copyObjects = dragObjects = null;
+			copyObjects = tree.dragObjects = null;
 			
 			object[] obs = new object [nodeHash.Count];
 			nodeHash.Keys.CopyTo (obs, 0);
@@ -562,8 +519,7 @@ namespace MonoDevelop.Ide.Gui.Components
 			foreach (object dataObject in obs)
 				NotifyNodeRemoved (dataObject, null);
 			
-			nodeHash = new Hashtable ();
-			nodeOptions.Clear ();
+			nodeHash = new NodeHashtable ();
 			store.Clear ();
 		}
 		
@@ -683,8 +639,8 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		public ITreeNavigator GetNodeAtObject (object dataObject, bool createTreeBranch)
 		{
-			object it = nodeHash [dataObject];
-			if (it == null) {
+			object it;
+			if (!nodeHash.TryGetValue (dataObject, out it)) {
 				if (createTreeBranch) {
 					TypeNodeBuilder tnb = GetTypeNodeBuilder (dataObject.GetType());
 					if (tnb == null) return null;
@@ -698,8 +654,7 @@ namespace MonoDevelop.Ide.Gui.Components
 					pnav.MoveToFirstChild ();
 					
 					// The child should be now in the this. Try again.
-					it = nodeHash [dataObject];
-					if (it == null)
+					if (!nodeHash.TryGetValue (dataObject, out it))
 						return null;
 				} else
 					return null;
@@ -896,35 +851,31 @@ namespace MonoDevelop.Ide.Gui.Components
 		}
 		
 		public event EventHandler CurrentItemActivated;
-		
+
 		#region Zoom
-		
-		const int ZOOM_MIN = -4;
-		const int ZOOM_MAX = 8;
+
 		const double ZOOM_FACTOR = 1.1f;
-		
-		int zoomPow = 0;
-		double zoom = 1;
-		
+		const int ZOOM_MIN_POW = -4;
+		const int ZOOM_MAX_POW = 8;
+		static readonly double ZOOM_MIN = System.Math.Pow (ZOOM_FACTOR, ZOOM_MIN_POW);
+		static readonly double ZOOM_MAX = System.Math.Pow (ZOOM_FACTOR, ZOOM_MAX_POW);
+		double zoom;
+
 		public double Zoom {
 			get {
 				 return zoom;
 			}
-			set {
-				ZoomPow = (int) System.Math.Round (System.Math.Log (value) / System.Math.Log (ZOOM_FACTOR));
-			}
-		}
-		
-		int ZoomPow {
-			get {
-				return zoomPow;
-			}
 			set {
 				value = System.Math.Min (ZOOM_MAX, System.Math.Max (ZOOM_MIN, value));
-				if (zoomPow != value) {
-					zoomPow = value;
-					zoom = System.Math.Pow (ZOOM_FACTOR, zoomPow);
-					OnZoomChanged (zoom);
+				if (value > ZOOM_MAX || value < ZOOM_MIN)
+					return;
+				//snap to one, if within 0.001d
+				if ((System.Math.Abs (value - 1d)) < 0.001d) {
+					value = 1d;
+				}
+				if (zoom != value) {
+					zoom = value;
+					OnZoomChanged (value);
 				}
 			}
 		}
@@ -948,47 +899,48 @@ namespace MonoDevelop.Ide.Gui.Components
 			tree.ColumnsAutosize ();
 			if (!string.IsNullOrEmpty (Id)) {
 				PropertyService.Set ("MonoDevelop.Ide.ExtensibleTreeView.Zoom." + Id, Zoom);
-				PropertyService.SaveProperties ();
 			}
 		}
 		
 		[CommandHandler (ViewCommands.ZoomIn)]
 		public void ZoomIn ()
 		{
-			ZoomPow++;
+			int oldPow = (int)System.Math.Round (System.Math.Log (zoom) / System.Math.Log (ZOOM_FACTOR));
+			Zoom = System.Math.Pow (ZOOM_FACTOR, oldPow + 1);
 		}
 
 		[CommandHandler (ViewCommands.ZoomOut)]
 		public void ZoomOut ()
 		{
-			ZoomPow--;
+			int oldPow = (int)System.Math.Round (System.Math.Log (zoom) / System.Math.Log (ZOOM_FACTOR));
+			Zoom = System.Math.Pow (ZOOM_FACTOR, oldPow - 1);
 		}
 		
 		[CommandHandler (ViewCommands.ZoomReset)]
 		public void ZoomReset ()
 		{
-			ZoomPow = 0;
+			Zoom = 1d;
 		}
 
 		[CommandUpdateHandler (ViewCommands.ZoomIn)]
 		protected void UpdateZoomIn (CommandInfo cinfo)
 		{
-			cinfo.Enabled = ZoomPow <= ZOOM_MAX;
+			cinfo.Enabled = zoom < ZOOM_MAX - 0.000001d;
 		}
 
 		[CommandUpdateHandler (ViewCommands.ZoomOut)]
 		protected void UpdateZoomOut (CommandInfo cinfo)
 		{
-			cinfo.Enabled = ZoomPow >= ZOOM_MIN;
+			cinfo.Enabled = zoom > ZOOM_MIN + 0.000001d;
 		}
 		
 		[CommandUpdateHandler (ViewCommands.ZoomReset)]
 		protected void UpdateZoomReset (CommandInfo cinfo)
 		{
-			cinfo.Enabled = ZoomPow != 0;
+			cinfo.Enabled = zoom != 1d;
 		}
 		
-		#endregion
+		#endregion Zoom
 
 		[CommandHandler (EditCommands.Copy)]
 		public void CopyCurrentItem ()
@@ -1284,19 +1236,41 @@ namespace MonoDevelop.Ide.Gui.Components
 			if (root == null) 
 				return null;
 
-			return root.SaveState ();
+			var state = root.SaveState ();
+
+			var s = new Dictionary<string, bool> ();
+			foreach (TreePadOption opt in options) {
+				bool val;
+				if (globalOptions.TryGetValue (opt.Id, out val) && val != opt.DefaultValue)
+					s[opt.Id] = val;
+			}
+			if (s.Count != 0) {
+				state.Options = s;
+			}
+
+			return state;
 		}
 		
 		public void RestoreTreeState (NodeState state)
 		{
-			ITreeNavigator nav = GetRootNode ();
-			
-			if (nav == null)
+			if (state == null)
 				return;
 			
-			if (state != null) {
-				nav.RestoreState (state);
+			ITreeNavigator nav = GetRootNode ();
+			if (nav == null)
+				return;
+
+			nav.RestoreState (state);
+
+			globalOptions = new TreeOptions ();
+			foreach (TreePadOption opt in options) {
+				bool val = false;
+				if (state.Options == null || !state.Options.TryGetValue (opt.Id, out val))
+					val = opt.DefaultValue;
+				globalOptions[opt.Id] = val;
 			}
+			globalOptions.Pad = this;
+			RefreshTree ();
 		}
 		
 		TypeNodeBuilder GetTypeNodeBuilder (Type type)
@@ -1399,8 +1373,8 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		internal bool GetFirstNode (object dataObject, out Gtk.TreeIter iter)
 		{
-			object it = nodeHash [dataObject];
-			if (it == null) {
+			object it;
+			if (!nodeHash.TryGetValue (dataObject, out it)) {
 				iter = Gtk.TreeIter.Zero;
 				return false;
 			}
@@ -1413,10 +1387,9 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		internal bool GetNextNode (object dataObject, ref Gtk.TreeIter iter)
 		{
-			object it = nodeHash [dataObject];
-			if (it == null) {
+			object it;
+			if (!nodeHash.TryGetValue (dataObject, out it))
 				return false;
-			}
 			else if (it is Gtk.TreeIter)
 				return false; // There is only one node, GetFirstNode returned it
 			else {
@@ -1436,8 +1409,8 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		internal void RegisterNode (Gtk.TreeIter it, object dataObject, NodeBuilder[] chain, bool fireAddedEvent)
 		{
-			object currentIt = nodeHash [dataObject];
-			if (currentIt == null) {
+			object currentIt;
+			if (!nodeHash.TryGetValue (dataObject, out currentIt)) {
 				nodeHash [dataObject] = it;
 				if (chain == null) chain = GetBuilderChain (dataObject.GetType());
 				if (fireAddedEvent) {
@@ -1480,21 +1453,20 @@ namespace MonoDevelop.Ide.Gui.Components
 				
 			// Remove object from drag list
 
-			if (dragObjects != null) {
-				int i = Array.IndexOf (dragObjects, dataObject);
+			if (tree.dragObjects != null) {
+				int i = Array.IndexOf (tree.dragObjects, dataObject);
 				if (i != -1) {
-					ArrayList list = new ArrayList (dragObjects);
+					ArrayList list = new ArrayList (tree.dragObjects);
 					list.RemoveAt (i);
 					if (list.Count > 0)
-						dragObjects = list.ToArray ();
+						tree.dragObjects = list.ToArray ();
 					else
-						dragObjects = null;
+						tree.dragObjects = null;
 				}
 			}
-			
-			if (store.IterIsValid (iter))
-				nodeOptions.Remove (iter);
-			object currentIt = nodeHash [dataObject];
+
+			object currentIt;
+			nodeHash.TryGetValue (dataObject, out currentIt);
 			if (currentIt is Gtk.TreeIter[]) {
 				Gtk.TreeIter[] arr = (Gtk.TreeIter[]) currentIt;
 				Gtk.TreePath path = null;
@@ -1532,16 +1504,6 @@ namespace MonoDevelop.Ide.Gui.Components
 				store.Remove (ref child);
 			}
 		}
-		
-		internal void CleanupNodeOptions ()
-		{
-			Dictionary<Gtk.TreeIter, TreeOptions> newOps = new Dictionary<Gtk.TreeIter, TreeOptions> ();
-			foreach (KeyValuePair<Gtk.TreeIter,TreeOptions> val in nodeOptions) {
-				if (store.IterIsValid (val.Key))
-					newOps [val.Key] = val.Value;
-			}
-			nodeOptions = newOps;
-		}
 				
 		void NotifyNodeRemoved (object dataObject, NodeBuilder[] chain)
 		{
@@ -1558,7 +1520,7 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		internal bool IsRegistered (object dataObject)
 		{
-			return nodeHash.Contains (dataObject);
+			return nodeHash.ContainsKey (dataObject);
 		}
 		
 		public void NotifyInserted (Gtk.TreeIter it, object dataObject)
@@ -1577,53 +1539,6 @@ namespace MonoDevelop.Ide.Gui.Components
 			}
 		}
 		
-		internal TreeOptions GetOptions (Gtk.TreeIter iter, bool createSpecificOptions)
-		{
-			if (nodeOptions.Count == 0) {
-				if (createSpecificOptions) {
-					TreeOptions ops = globalOptions.CloneOptions (iter);
-					nodeOptions [iter] = ops;
-					return ops;
-				}
-				else
-					return globalOptions;
-			}
-			
-			TreeOptions result = null;
-			Gtk.TreeIter it = iter;
-			do {
-				nodeOptions.TryGetValue (it, out result);
-			} while (result == null && store.IterParent (out it, it));
-
-			if (result == null)
-				result = globalOptions;
-			
-			if (createSpecificOptions && !it.Equals (iter)) {
-				TreeOptions ops = result.CloneOptions (iter);
-				nodeOptions [iter] = ops;
-				return ops;
-			} else
-				return result;
-		}
-		
-		internal void ClearOptions (Gtk.TreeIter iter)
-		{
-			if (nodeOptions.Count == 0)
-				return;
-
-			List<Gtk.TreeIter> toDelete = new List<Gtk.TreeIter> ();
-			string path = store.GetPath (iter).ToString () + ":";
-			
-			foreach (Gtk.TreeIter nit in nodeOptions.Keys) {
-				string npath = store.GetPath (nit).ToString () + ":";
-				if (npath.StartsWith (path))
-					toDelete.Add (nit);
-			}
-
-			foreach (Gtk.TreeIter ob in toDelete)
-				nodeOptions.Remove (ob);
-		}
-		
 		internal string GetNamePathFromIter (Gtk.TreeIter iter)
 		{
 			workNode.MoveToIter (iter);
@@ -1644,6 +1559,11 @@ namespace MonoDevelop.Ide.Gui.Components
 			ITreeBuilder builder = CreateBuilder (iter);
 			builder.UpdateAll ();
 		}
+
+		public void RefreshNode (ITreeNavigator nav)
+		{
+			RefreshNode (nav.CurrentPosition._iter);
+		}
 		
 		internal void ResetState (ITreeNavigator nav)
 		{
@@ -1656,39 +1576,6 @@ namespace MonoDevelop.Ide.Gui.Components
 				ResetState (builder);
 			}
 		}
-
-		internal void SetNodeOptions (ITreeNavigator nav, TreeOptions ops)
-		{
-			if (nav.CurrentPosition._node != null)
-				return;
-			
-			// TODO transaction
-			Gtk.TreeIter iter = nav.CurrentPosition._iter;
-			ops.Pad = this;
-			ops.Iter = iter;
-			nodeOptions [iter] = ops;
-		}
-
-		internal TreeOptions GetNodeOptions (ITreeNavigator nav)
-		{
-			TreeNode node = nav.CurrentPosition._node as TreeNode;
-			if (node != null && node.HasIter) {
-				TreeOptions ops;
-				if (nodeOptions.TryGetValue (node.NodeIter, out ops))
-					return ops;
-				else
-					return null;
-			}
-			else if (node != null)
-				return null;
-			else {
-				TreeOptions ops;
-				if (nodeOptions.TryGetValue (nav.CurrentPosition._iter, out ops))
-					return ops;
-				else
-					return null;
-			}
-		}
 		
 		internal bool GetIterFromNamePath (string path, out Gtk.TreeIter iter)
 		{
@@ -1815,10 +1702,9 @@ namespace MonoDevelop.Ide.Gui.Components
 		protected void BuildTreeOptionsMenu (CommandArrayInfo info)
 		{
 			ITreeNavigator tnav = GetSelectedNode ();
-			ITreeOptions currentOptions = tnav.Options;
 			foreach (TreePadOption op in options) {
 				CommandInfo ci = new CommandInfo (op.Label);
-				ci.Checked = currentOptions [op.Id];
+				ci.Checked = globalOptions [op.Id];
 				info.Add (ci, op.Id);
 			}
 		}
@@ -1826,9 +1712,8 @@ namespace MonoDevelop.Ide.Gui.Components
 		[CommandHandler (ViewCommands.TreeDisplayOptionList)]
 		protected void OptionToggled (string optionId)
 		{
-			foreach (TreeNodeNavigator node in GetSelectedNodes ()) {
-				node.Options [optionId] = !node.Options [optionId];
-			}
+			globalOptions [optionId] = !globalOptions [optionId];
+			RefreshTree ();
 		}
 		
 		[CommandHandler (ViewCommands.ResetTreeDisplayOptions)]
@@ -1837,7 +1722,6 @@ namespace MonoDevelop.Ide.Gui.Components
 			foreach (TreeNodeNavigator node in GetSelectedNodes ()) {
 				Gtk.TreeIter it = node.CurrentPosition._iter;
 				if (store.IterIsValid (it)) {
-					ClearOptions (it);
 					ITreeBuilder tb = CreateBuilder (it);
 					tb.UpdateAll ();
 				}
@@ -2080,6 +1964,117 @@ namespace MonoDevelop.Ide.Gui.Components
 				get { return pad; }
 			}
 		}
+
+		class ExtensibleTreeViewTree : ContextMenuTreeView
+		{
+			ExtensibleTreeView tv;
+
+			public ExtensibleTreeViewTree (ExtensibleTreeView tv)
+			{
+				this.tv = tv;
+				EnableModelDragDest (targetTable, Gdk.DragAction.Copy | Gdk.DragAction.Move);
+				Gtk.Drag.SourceSet (this, Gdk.ModifierType.Button1Mask, targetTable, Gdk.DragAction.Copy | Gdk.DragAction.Move);
+			}
+
+			static Gtk.TargetEntry [] targetTable = new Gtk.TargetEntry [] {
+				new Gtk.TargetEntry ("text/uri-list", 0, 11 ),
+				new Gtk.TargetEntry ("text/plain", 0, 22),
+				new Gtk.TargetEntry ("application/x-rootwindow-drop", 0, 33)
+			};
+
+			public object[] dragObjects = null;
+			bool dropping = false;
+			Func<object,string> nodeToUri;
+
+			public void EnableDragUriSource (Func<object,string> nodeToUri)
+			{
+				this.nodeToUri = nodeToUri;
+			}
+
+			public delegate object[] GetDragObjects (out Gdk.Pixbuf dragIcon);
+			public delegate bool CheckAndDrop (int x, int y, bool drop, Gdk.DragContext ctx, object[] obj);
+
+			protected override void OnDragBegin (Gdk.DragContext context)
+			{
+				Gdk.Pixbuf dragIcon;
+				dragObjects = tv.GetDragObjects (out dragIcon);
+				Gtk.Drag.SetIconPixbuf (context, dragIcon, -10, -10);
+
+				base.OnDragBegin (context);
+			}
+
+			protected override void OnDragEnd (Gdk.DragContext context)
+			{
+				dragObjects = null;
+				base.OnDragEnd (context);
+			}
+
+			protected override bool OnDragMotion (Gdk.DragContext context, int x, int y, uint time)
+			{
+				//OnDragDataReceived callback loses x/y values, so stash them
+				this.x = x;
+				this.y = y;
+
+				if (dragObjects == null) {
+					//it's a drag from outside, need to retrieve the data. This will cause OnDragDataReceived to be called.
+					Gdk.Atom atom = Gtk.Drag.DestFindTarget (this, context, null);
+					Gtk.Drag.GetData (this, context, atom, time);
+				} else {
+					//it's from inside, can call OnDragDataReceived directly
+					OnDragDataReceived (context, x, y, null, 0, time);
+				}
+				return true;
+			}
+
+			int x, y;
+			protected override void OnDragDataReceived (Gdk.DragContext context, int x, int y, Gtk.SelectionData selection_data, uint info, uint time)
+			{
+				x = this.x;
+				y = this.y;
+
+				object[] data = dragObjects ?? new object[] { selection_data };
+				bool canDrop = tv.CheckAndDrop (x, y, dropping, context, data);
+				if (dropping) {
+					dropping = false;
+					SetDragDestRow (null, 0);
+					Gtk.Drag.Finish (context, canDrop, true, time);
+					return;
+				}
+
+				//let default handler handle hover-to-expand, autoscrolling, etc
+				base.OnDragMotion (context, x, y, time);
+
+				//if we can't handle it, flag as not droppable and remove the drop marker
+				if (!canDrop) {
+					Gdk.Drag.Status (context, (Gdk.DragAction)0, time);
+					SetDragDestRow (null, 0);
+				}
+			}
+
+			protected override bool OnDragDrop (Gdk.DragContext context, int x, int y, uint time_)
+			{
+				dropping = true;
+				return base.OnDragDrop (context, x, y, time_);
+			}
+
+			protected override void OnDragDataGet (Gdk.DragContext context, Gtk.SelectionData selection_data, uint info, uint time_)
+			{
+				if (dragObjects == null || nodeToUri == null)
+					return;
+
+				uint uriListTarget = targetTable[0].Info;
+				if (info == uriListTarget) {
+					var sb = new StringBuilder ();
+					foreach (var dobj in dragObjects) {
+						var val = nodeToUri (dobj);
+						if (val != null) {
+							sb.AppendLine (val);
+						}
+					}
+					selection_data.Set (selection_data.Target, selection_data.Format, Encoding.UTF8.GetBytes (sb.ToString ()));
+				}
+			}
+		}
 	}
 
 	class NodeCommandTargetChain: ICommandDelegatorRouter
@@ -2214,4 +2209,69 @@ namespace MonoDevelop.Ide.Gui.Components
 			return resized;
 		}
 	}
-}
\ No newline at end of file
+
+	class NodeHashtable: Dictionary<object,object>
+	{
+		// This dictionary can be configured to use object reference equality
+		// instead of regular object equality for a specific set of types
+
+		NodeComparer nodeComparer;
+
+		public NodeHashtable (): base (new NodeComparer ())
+		{
+			nodeComparer = (NodeComparer)Comparer;
+		}
+
+		/// <summary>
+		/// Sets that the objects of the specified type have to be compared
+		/// using object reference equality
+		/// </summary>
+		public void RegisterByRefType (Type type)
+		{
+			nodeComparer.byRefTypes.Add (type);
+		}
+
+		class NodeComparer: IEqualityComparer<object>
+		{
+			public HashSet<Type> byRefTypes = new HashSet<Type> ();
+			public Dictionary<Type,bool> typeData = new Dictionary<Type, bool> ();
+
+			bool IEqualityComparer<object>.Equals (object x, object y)
+			{
+				if (CompareByRef (x.GetType ()))
+				    return x == y;
+				else
+					return x.Equals (y);
+			}
+	
+			int IEqualityComparer<object>.GetHashCode (object obj)
+			{
+				if (CompareByRef (obj.GetType ()))
+					return System.Runtime.CompilerServices.RuntimeHelpers.GetHashCode (obj);
+				else
+					return obj.GetHashCode ();
+			}
+
+			bool CompareByRef (Type type)
+			{
+				if (byRefTypes.Count == 0)
+					return false;
+	
+				bool compareRef;
+				if (!typeData.TryGetValue (type, out compareRef)) {
+					compareRef = false;
+					var t = type;
+					while (t != null) {
+						if (byRefTypes.Contains (t)) {
+							compareRef = true;
+							break;
+						}
+						t = t.BaseType;
+					}
+					typeData [type] = compareRef;
+				}
+				return compareRef;
+			}
+		}
+	}
+}
