@@ -29,22 +29,23 @@
 using System;
 using MonoDevelop.Projects;
 using MonoDevelop.Ide.CodeCompletion;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.Dom.Output;
-using MonoDevelop.Projects.Dom.Parser;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.Ide.Commands;
 using MonoDevelop.Core;
 using Mono.TextEditor;
+using ICSharpCode.NRefactory.TypeSystem;
+using MonoDevelop.Ide.TypeSystem;
 
 namespace MonoDevelop.Ide.Gui.Content
 {
 	public class TextEditorExtension : ITextEditorExtension, ICommandRouter
 	{
-		internal Document document;
+		internal protected Document document;
 		
-		internal protected void Initialize (Document document)
+		public void Initialize (Document document)
 		{
+			if (this.document != null)
+				throw new InvalidOperationException ("Extension is already initialized.");
 			this.document = document;
 			Initialize ();
 		}
@@ -57,15 +58,11 @@ namespace MonoDevelop.Ide.Gui.Content
 		protected Document Document {
 			get { return document; }
 		}
-		
-		protected TextEditor Editor {
-			get { return document.TextEditor; }
+
+		protected TextEditorData Editor {
+			get { return document != null ? document.Editor : null; }
 		}
-		
-		protected TextEditorData TextEditorData {
-			get { return document.TextEditorData; }
-		}
-		
+
 		protected FilePath FileName {
 			get {
 				IViewContent view = document.Window.ViewContent;
@@ -73,7 +70,7 @@ namespace MonoDevelop.Ide.Gui.Content
 			}
 		}
 		
-		protected ProjectDom GetParserContext ()
+		protected IProjectContent GetParserContext ()
 		{
 			CheckInitialized ();
 			
@@ -82,9 +79,8 @@ namespace MonoDevelop.Ide.Gui.Content
 			Project project = view.Project;
 			
 			if (project != null)
-				return ProjectDomService.GetProjectDom (project);
-			else
-				return ProjectDomService.GetFileDom (file);
+				return TypeSystemService.GetProjectContext (project);
+			return TypeSystemService.GetContext (file, Document.Editor.Document.MimeType, Document.Editor.Text);
 		}
 		
 		protected Ambience GetAmbience ()
@@ -92,14 +88,8 @@ namespace MonoDevelop.Ide.Gui.Content
 			CheckInitialized ();
 			
 			IViewContent view = document.Window.ViewContent;
-			Project project = view.Project;
-			
-			if (project != null)
-				return project.Ambience;
-			else {
-				string file = view.IsUntitled ? view.UntitledName : view.ContentName;
-				return AmbienceService.GetAmbienceForFile (file);
-			}
+			string file = view.IsUntitled ? view.UntitledName : view.ContentName;
+			return AmbienceService.GetAmbienceForFile (file);
 		}
 		
 		public virtual bool ExtendsEditor (Document doc, IEditableTextBuffer editor)
