@@ -107,6 +107,13 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 				MonoTargetRuntime.UnregisterRuntime (tr);
 			
 		}
+
+		// ProgramFilesX86 is broken on 32-bit WinXP, this is a workaround
+		static string GetProgramFilesX86 ()
+		{
+			return Environment.GetFolderPath (IntPtr.Size == 8?
+				Environment.SpecialFolder.ProgramFilesX86 : Environment.SpecialFolder.ProgramFiles);
+		}
 		
 		protected virtual void OnButtonAddClicked (object sender, System.EventArgs e)
 		{
@@ -117,8 +124,7 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 			//set a platform-dependent default folder for the dialog if possible
 			if (Platform.IsWindows) {
 				// ProgramFilesX86 is broken on 32-bit WinXP
-				string programFilesX86 = Environment.GetFolderPath (
-					IntPtr.Size == 8? Environment.SpecialFolder.ProgramFilesX86 : Environment.SpecialFolder.ProgramFiles);
+				string programFilesX86 = GetProgramFilesX86 ();
 				if (!string.IsNullOrEmpty (programFilesX86) && System.IO.Directory.Exists (programFilesX86))
 					dlg.CurrentFolder = programFilesX86;
 			} else {
