@@ -77,6 +77,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 		const string showWarningsPropertyName = "SharpDevelop.TaskList.ShowWarnings";
 		const string showMessagesPropertyName = "SharpDevelop.TaskList.ShowMessages";
 		const string logSeparatorPositionPropertyName = "SharpDevelop.TaskList.LogSeparatorPosition";
+		const string outputViewVisiblePropertyName = "SharpDevelop.TaskList.OutputViewVisible";
 
 		static class DataColumns
 		{
@@ -184,9 +185,9 @@ namespace MonoDevelop.Ide.Gui.Pads
 			sw = new MonoDevelop.Components.CompactScrolledWindow ();
 			sw.ShadowType = ShadowType.None;
 			sw.Add (view);
-			TaskService.Errors.TasksRemoved      += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (ShowResults));
-			TaskService.Errors.TasksAdded        += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (TaskAdded));
-			TaskService.Errors.TasksChanged      += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (TaskChanged));
+			TaskService.Errors.TasksRemoved      += DispatchService.GuiDispatch<TaskEventHandler> (ShowResults);
+			TaskService.Errors.TasksAdded        += DispatchService.GuiDispatch<TaskEventHandler> (TaskAdded);
+			TaskService.Errors.TasksChanged      += DispatchService.GuiDispatch<TaskEventHandler> (TaskChanged);
 			TaskService.Errors.CurrentLocationTaskChanged += HandleTaskServiceErrorsCurrentLocationTaskChanged;
 			
 			IdeApp.Workspace.FirstWorkspaceItemOpened += OnCombineOpen;
@@ -205,13 +206,18 @@ namespace MonoDevelop.Ide.Gui.Pads
 			
 			Control.ShowAll ();
 			
-			sw.SizeAllocated += delegate {
-				if (outputView.Visible)
-					PropertyService.Set (logSeparatorPositionPropertyName, (double) ((double) control.Position / (double) control.Allocation.Width));
-			};
+			control.SizeAllocated += HandleControlSizeAllocated;
+			
+			bool outputVisible = PropertyService.Get<bool> (outputViewVisiblePropertyName, false);
+			if (outputVisible) {
+				outputView.Visible = true;
+				logBtn.Active = true;
+			} else {
+				outputView.Hide ();
+			}
+			
+			sw.SizeAllocated += HandleSwSizeAllocated;
 			
-			outputView.Hide ();
-
 			// Load existing tasks
 			foreach (Task t in TaskService.Errors) {
 				AddTask (t);
@@ -220,6 +226,23 @@ namespace MonoDevelop.Ide.Gui.Pads
 			control.FocusChain = new Gtk.Widget [] { sw };
 		}
 		
+		void HandleSwSizeAllocated (object o, SizeAllocatedArgs args)
+		{
+			if (!initialLogShow && outputView.Visible) {
+				var val = (double) ((double) control.Position / (double) control.Allocation.Width);
+				PropertyService.Set (logSeparatorPositionPropertyName, val);
+			}
+		}
+		
+		[GLib.ConnectBefore]
+		void HandleControlSizeAllocated (object o, SizeAllocatedArgs args)
+		{
+			if (initialLogShow && outputView.Visible) {
+				SetInitialOutputViewSize (args.Allocation.Width);
+				initialLogShow = false;
+			}
+		}
+		
 		public IProgressMonitor GetBuildProgressMonitor ()
 		{
 			if (control == null)
@@ -548,9 +571,10 @@ namespace MonoDevelop.Ide.Gui.Pads
 			if (task == null)
 				return;
 			
-			string tmpPath = GetPath (task);
-			string fileName;
+			string tmpPath = "";
+			string fileName = "";
 			try {
+				tmpPath = GetPath (task);
 				fileName = Path.GetFileName (tmpPath);
 			} catch (Exception) { 
 				fileName =  tmpPath;
@@ -651,8 +675,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 		{
 			Clear();
 
-			foreach (Task t in TaskService.Errors)
-				AddTask (t);
+			AddTasks (TaskService.Errors);
 		}
 
 		private void Clear()
@@ -742,21 +765,13 @@ namespace MonoDevelop.Ide.Gui.Pads
 			msgBtn.Image.Show ();
 		}
 		
-		public event EventHandler<TaskEventArgs> TaskToggled;
-		protected virtual void OnTaskToggled (TaskEventArgs e)
-		{
-			EventHandler<TaskEventArgs> handler = this.TaskToggled;
-			if (handler != null)
-				handler (this, e);
-		}
-		
 		private void ItemToggled (object o, ToggledArgs args)
 		{
 			Gtk.TreeIter iter;
-			if (store.GetIterFromString(out iter, args.Path)) {
-				Task task = (Task) store.GetValue(iter, DataColumns.Task);
+			if (store.GetIterFromString (out iter, args.Path)) {
+				Task task = (Task)store.GetValue (iter, DataColumns.Task);
 				task.Completed = !task.Completed;
-				OnTaskToggled (new TaskEventArgs (task));
+				TaskService.FireTaskToggleEvent (this, new TaskEventArgs (task));
 			}
 		}
 
@@ -792,15 +807,22 @@ namespace MonoDevelop.Ide.Gui.Pads
 		
 		void HandleLogBtnToggled (object sender, EventArgs e)
 		{
-			if (!outputView.Visible && initialLogShow) {
-				double relPos = PropertyService.Get<double> (logSeparatorPositionPropertyName, 0.5d);
-				int pos = (int) (control.Allocation.Width * relPos);
-				if (pos < 30) pos = 30;
-				control.Position = pos;
-				initialLogShow = false;
-			}
+			var visible = logBtn.Active;
+			PropertyService.Set (outputViewVisiblePropertyName, visible);
+			outputView.Visible = visible;
 			
-			outputView.Visible = logBtn.Active;
+			if (initialLogShow && visible && control.IsRealized) {
+				initialLogShow = false;
+				SetInitialOutputViewSize (control.Allocation.Width);
+			}
+		}
+		
+		void SetInitialOutputViewSize (int controlWidth)
+		{
+			double relPos = PropertyService.Get<double> (logSeparatorPositionPropertyName, 0.5d);
+			int pos = (int) (controlWidth * relPos);
+			pos = Math.Max (30, Math.Min (pos, controlWidth - 30));
+			control.Position = pos;
 		}
 	}
 	
