@@ -51,7 +51,7 @@ namespace MonoDevelop.Ide.Gui
 	/// <summary>
 	/// This is the a Workspace with a multiple document interface.
 	/// </summary>
-	internal class DefaultWorkbench : WorkbenchWindow
+	internal class DefaultWorkbench : WorkbenchWindow, ICommandDelegatorRouter
 	{
 		readonly static string mainMenuPath    = "/MonoDevelop/Ide/MainMenu";
 		readonly static string viewContentPath = "/MonoDevelop/Ide/Pads";
@@ -409,7 +409,6 @@ namespace MonoDevelop.Ide.Gui
 			
 			tabControl.InsertPage (sdiWorkspaceWindow, tabLabel, -1);
 			
-			
 			if (content.Project != null)
 				content.Project.NameChanged += HandleProjectNameChanged;
 			
@@ -741,6 +740,7 @@ namespace MonoDevelop.Ide.Gui
 			
 			if (lastActiveWindows.Count > MAX_LASTACTIVEWINDOWS)
 				lastActiveWindows.RemoveFirst ();
+			
 			lastActiveWindows.AddLast (lastActive);
 			lastActive = ActiveWorkbenchWindow;
 			SetWorkbenchTitle ();
@@ -831,8 +831,8 @@ namespace MonoDevelop.Ide.Gui
 			tabControl = new SdiDragNotebook (dock.ShadedContainer);
 			tabControl.Scrollable = true;
 			tabControl.SwitchPage += OnActiveWindowChanged;
-			tabControl.PageAdded += delegate { OnActiveWindowChanged (null, null); };
-			tabControl.PageRemoved += delegate { OnActiveWindowChanged (null, null); };
+			tabControl.PageAdded += OnActiveWindowChanged;
+			tabControl.PageRemoved += OnActiveWindowChanged;
 		
 			tabControl.ButtonPressEvent += delegate(object sender, ButtonPressEventArgs e) {
 				int tab = tabControl.FindTabAtPosition (e.Event.XRoot, e.Event.YRoot);
@@ -1336,6 +1336,37 @@ namespace MonoDevelop.Ide.Gui
 		}
 		
 		#endregion
+
+		#region ICommandDelegatorRouter implementation
+		
+		// The command route is redirected here to the active document view.
+		// This is done to make the view commands available even when the
+		// view has not the active focus, especially when the focus is
+		// on a pad. Although it may seem a bit inconsistent (because
+		// MD will not have active commands which don't directly apply to
+		// the active widget) in general it makes sense, views are the
+		// main working areas of the IDE and users often expect view commands
+		// to be available even if the focus is in a secondary Pad window.
+		//
+		// This change also fixes some issues with the property and outline
+		// pads, which depend on the current selection, and were being reset
+		// when the main view lost focus.
+		
+		object ICommandDelegatorRouter.GetNextCommandTarget ()
+		{
+			// This is the last object of the chain
+			return null;
+		}
+
+		object ICommandDelegatorRouter.GetDelegatedCommandTarget ()
+		{
+			// The command manager checks if an object has already been visited
+			// while scanning the command route, so if the active command
+			// route already includes the active workbench view, it won't be
+			// visited again
+			return ActiveWorkbenchWindow;
+		}
+		#endregion
 	}
 
 	class PadActivationHandler: CommandHandler
