@@ -41,16 +41,19 @@ using MonoDevelop.Projects;
 using MonoDevelop.Projects.Text;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Ide.Gui.Dialogs;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.Dom.Parser;
 using MonoDevelop.Ide.Tasks;
 using Mono.Addins;
 using MonoDevelop.Ide.Extensions;
 using System.Linq;
+using System.Threading;
+using MonoDevelop.Ide.TypeSystem;
+using ICSharpCode.NRefactory.TypeSystem;
+using ICSharpCode.NRefactory.TypeSystem.Implementation;
+using System.Text;
 
 namespace MonoDevelop.Ide.Gui
 {
-	public class Document
+	public class Document : ICSharpCode.NRefactory.AbstractAnnotatable
 	{
 		internal object MemoryProbe = Counters.DocumentsInMemory.CreateMemoryProbe ();
 		
@@ -120,7 +123,19 @@ namespace MonoDevelop.Ide.Gui
 				nextExtension = nextExtension.Next as TextEditorExtension;
 			}
 		}
-		
+		static Document ()
+		{
+			if (IdeApp.Workbench != null) {
+				IdeApp.Workbench.ActiveDocumentChanged += delegate {
+					// reparse on document switch to update the current file with changes done in other files.
+					var doc = IdeApp.Workbench.ActiveDocument;
+					if (doc == null || doc.Editor == null)
+						return;
+					doc.StartReparseThread ();
+				};
+			}
+		}
+
 		public Document (IWorkbenchWindow window)
 		{
 			Counters.OpenDocuments++;
@@ -130,17 +145,18 @@ namespace MonoDevelop.Ide.Gui
 			window.ActiveViewContentChanged += OnActiveViewContentChanged;
 			if (IdeApp.Workspace != null)
 				IdeApp.Workspace.ItemRemovedFromSolution += OnEntryRemoved;
-			ProjectDomService.DomRegistered += UpdateRegisteredDom;
+			if (window.ViewContent.Project != null)
+				window.ViewContent.Project.Modified += HandleProjectModified;
 		}
 
-		void UpdateRegisteredDom (object sender, ProjectDomEventArgs e)
+/*		void UpdateRegisteredDom (object sender, ProjectDomEventArgs e)
 		{
 			if (dom == null || dom.Project == null)
 				return;
-			var project = e.ProjectDom != null ? e.ProjectDom.Project : null;
+			var project = e.ITypeResolveContext != null ? e.ITypeResolveContext.Project : null;
 			if (project != null && project.FileName == dom.Project.FileName)
-				dom = e.ProjectDom;
-		}
+				dom = e.ITypeResolveContext;
+		}*/
 
 		public FilePath FileName {
 			get {
@@ -165,28 +181,43 @@ namespace MonoDevelop.Ide.Gui
 		
 		public Project Project {
 			get { return Window.ViewContent.Project; }
-			set { 
+/*			set { 
 				Window.ViewContent.Project = value; 
-				dom = null;
+				if (value != null)
+					singleFileContext = null;
+				// File needs to be in sync with the project, otherwise the parsed document at start may be invalid.
+				// better solution: create the document with the project attached.
+				StartReparseThread ();
+			}*/
+		}
+
+		public bool IsCompileableInProject {
+			get {
+				var project = Project;
+				if (project == null)
+					return false;
+				var pf = project.GetProjectFile (FileName);
+				return pf != null && pf.BuildAction == BuildAction.Compile;
+			}
+		}
+
+		IProjectContent singleFileContext;
+		public  virtual IProjectContent ProjectContent {
+			get {
+				return Project != null ? TypeSystemService.GetProjectContext (Project) : GetProjectContext ();
 			}
 		}
 		
-		ProjectDom dom;
-		bool isFileDom;
-		public ProjectDom Dom {
+		public  virtual ICompilation Compilation {
 			get {
-				if (dom == null) {
-					isFileDom = false;
-					dom = ProjectDomService.GetProjectDom (Project);
-					if (dom == null) {
-						dom = ProjectDomService.GetFileDom (FileName);
-						isFileDom = true;
-					}
-				}
-				return dom ?? ProjectDom.Empty;
+				return Project != null ? TypeSystemService.GetCompilation (Project) : GetProjectContext ().CreateCompilation ();
 			}
-			set {
-				dom = value;
+		}
+		
+		ParsedDocument parsedDocument;
+		public virtual ParsedDocument ParsedDocument {
+			get {
+				return parsedDocument;
 			}
 		}
 		
@@ -221,6 +252,8 @@ namespace MonoDevelop.Ide.Gui
 					Mono.TextEditor.ITextEditorDataProvider view = GetContent <Mono.TextEditor.ITextEditorDataProvider> ();
 					if (view != null) {
 						data = view.GetTextEditorData ();
+						if (data.IsDisposed)
+							return null;
 						data.Document.Tag = this;
 					}
 				}
@@ -247,35 +280,48 @@ namespace MonoDevelop.Ide.Gui
 		
 		public virtual void Save ()
 		{
-			if (Window.ViewContent.IsViewOnly || !Window.ViewContent.IsDirty)
-				return;
-
-			if (!Window.ViewContent.IsFile) {
-				Window.ViewContent.Save ();
-				return;
-			}
-			
-			if (Window.ViewContent.ContentName == null) {
-				SaveAs ();
-			} else {
-				if (!FileService.RequestFileEdit (Window.ViewContent.ContentName))
-					MessageService.ShowMessage (GettextCatalog.GetString ("The file could not be saved. Write permission has not been granted."));
+			// suspend type service "check all file loop" since we have already a parsed document.
+			// Or at least one that updates "soon".
+			TypeSystemService.TrackFileChanges = false;
+			try {
+				if (Window.ViewContent.IsViewOnly || !Window.ViewContent.IsDirty)
+					return;
+	
+				if (!Window.ViewContent.IsFile) {
+					Window.ViewContent.Save ();
+					return;
+				}
 				
-				FileAttributes attr = FileAttributes.ReadOnly | FileAttributes.Directory | FileAttributes.Offline | FileAttributes.System;
-
-				if (!File.Exists (Window.ViewContent.ContentName) || (File.GetAttributes(window.ViewContent.ContentName) & attr) != 0) {
+				if (Window.ViewContent.ContentName == null) {
 					SaveAs ();
 				} else {
-					string fileName = Window.ViewContent.ContentName;
-					// save backup first						
-					if((bool) PropertyService.Get ("SharpDevelop.CreateBackupCopy", false)) {
-						Window.ViewContent.Save (fileName + "~");
+					if (!FileService.RequestFileEdit (Window.ViewContent.ContentName))
+						MessageService.ShowMessage (GettextCatalog.GetString ("The file could not be saved. Write permission has not been granted."));
+					
+					FileAttributes attr = FileAttributes.ReadOnly | FileAttributes.Directory | FileAttributes.Offline | FileAttributes.System;
+	
+					if (!File.Exists (Window.ViewContent.ContentName) || (File.GetAttributes (window.ViewContent.ContentName) & attr) != 0) {
+						SaveAs ();
+					} else {
+						string fileName = Window.ViewContent.ContentName;
+						// save backup first						
+						if ((bool)PropertyService.Get ("SharpDevelop.CreateBackupCopy", false)) {
+							Window.ViewContent.Save (fileName + "~");
+							FileService.NotifyFileChanged (fileName);
+						}
+						Window.ViewContent.Save (fileName);
 						FileService.NotifyFileChanged (fileName);
+						OnSaved (EventArgs.Empty);
 					}
-					Window.ViewContent.Save (fileName);
-					FileService.NotifyFileChanged (fileName);
-					OnSaved (EventArgs.Empty);
 				}
+			} finally {
+				// Set the file time of the current document after the file time of the written file, to prevent double file updates.
+				// Note that the parsed document may be overwritten by a background thread to a more recent one.
+				var doc = parsedDocument;
+				if (doc != null && doc.ParsedFile != null) {
+					doc.ParsedFile.LastWriteTime = DateTime.Now;
+				}
+				TypeSystemService.TrackFileChanges = true;
 			}
 		}
 		
@@ -288,14 +334,15 @@ namespace MonoDevelop.Ide.Gui
 		{
 			if (Window.ViewContent.IsViewOnly || !Window.ViewContent.IsFile)
 				return;
-			
-			string encoding = null;
+
+
+			Encoding encoding = null;
 			
 			IEncodedTextContent tbuffer = GetContent <IEncodedTextContent> ();
 			if (tbuffer != null) {
 				encoding = tbuffer.SourceEncoding;
 				if (encoding == null)
-					encoding = TextEncoding.DefaultEncoding;
+					encoding = Encoding.Default;
 			}
 				
 			if (filename == null) {
@@ -324,19 +371,21 @@ namespace MonoDevelop.Ide.Gui
 				return;
 			}
 			// detect preexisting file
-			if(File.Exists(filename)){
+			if (File.Exists (filename)) {
 				if (!MessageService.Confirm (GettextCatalog.GetString ("File {0} already exists. Overwrite?", filename), AlertButton.OverwriteFile))
 					return;
 			}
 			
 			// save backup first
-			if((bool) PropertyService.Get ("SharpDevelop.CreateBackupCopy", false)) {
+			if ((bool)PropertyService.Get ("SharpDevelop.CreateBackupCopy", false)) {
 				if (tbuffer != null && encoding != null)
 					tbuffer.Save (filename + "~", encoding);
 				else
 					Window.ViewContent.Save (filename + "~");
 			}
-			
+			lock (TypeSystemService.FilesSkippedInParseThread) {
+				TypeSystemService.FilesSkippedInParseThread.Remove (FileName);
+			}
 			// do actual save
 			if (tbuffer != null && encoding != null)
 				tbuffer.Save (filename, encoding);
@@ -347,6 +396,7 @@ namespace MonoDevelop.Ide.Gui
 			DesktopService.RecentFiles.AddFile (filename, (Project)null);
 			
 			OnSaved (EventArgs.Empty);
+			UpdateParseDocument ();
 		}
 		
 		public virtual bool IsBuildTarget
@@ -419,13 +469,13 @@ namespace MonoDevelop.Ide.Gui
 		void OnClosed (object s, EventArgs a)
 		{
 			isClosed = true;
-			ProjectDomService.DomRegistered -= UpdateRegisteredDom;
+//			TypeSystemService.DomRegistered -= UpdateRegisteredDom;
 			CancelParseTimeout ();
 			ClearTasks ();
-			
-			string currentParseFile = FileName;
-			Project curentParseProject = Project;
-			
+
+			lock (TypeSystemService.FilesSkippedInParseThread) {
+				TypeSystemService.FilesSkippedInParseThread.Remove (FileName);
+			}
 			if (window is SdiWorkspaceWindow)
 				((SdiWorkspaceWindow)window).DetachFromPathedDocument ();
 			window.Closed -= OnClosed;
@@ -447,52 +497,28 @@ namespace MonoDevelop.Ide.Gui
 				}
 				editorExtension = editorExtension.Next as TextEditorExtension;
 			}
+			editorExtension = null;
 			
 			// Parse the file when the document is closed. In this way if the document
 			// is closed without saving the changes, the saved compilation unit
 			// information will be restored
-			if (currentParseFile != null) {
-				ProjectDomService.QueueParseJob (dom, delegate (string name, IProgressMonitor monitor) {
-					ProjectDomService.Parse (curentParseProject, currentParseFile);
+/*			if (currentParseFile != null) {
+				TypeSystemService.QueueParseJob (dom, delegate (string name, IProgressMonitor monitor) {
+					TypeSystemService.Parse (curentParseProject, currentParseFile);
 				}, FileName);
 			}
 			if (isFileDom) {
-				ProjectDomService.RemoveFileDom (FileName);
+				TypeSystemService.RemoveFileDom (FileName);
 				dom = null;
-			}
+			}*/
 			
+			parsedDocument = null;
+			data = null;
 			Counters.OpenDocuments--;
 		}
 #region document tasks
 		object lockObj = new object ();
 		
-		ParsedDocument lastErrorFreeParsedDocument;
-		
-		ParsedDocument parsedDocument;
-		public ParsedDocument ParsedDocument {
-			get {
-				return parsedDocument;
-			}
-			set {
-				// for unit testing purposes
-				parsedDocument = value;
-			}
-		}
-		
-		public ICompilationUnit CompilationUnit {
-			get {
-				if (parsedDocument == null)
-					return null;
-				var unit = parsedDocument.CompilationUnit;
-				if (unit == null)
-					return null;
-				// HACK: The parser sometimes doesn't send back >any< info on the file on certain errors, this code detects if the last error free parsed document is the better best-fit.
-				if (parsedDocument.HasErrors && !unit.Types.Any () && lastErrorFreeParsedDocument != null && lastErrorFreeParsedDocument.CompilationUnit != null && lastErrorFreeParsedDocument.CompilationUnit.Types.Any ())
-					return lastErrorFreeParsedDocument.CompilationUnit;
-				return unit;
-			}
-		}
-		
 		void ClearTasks ()
 		{
 			lock (lockObj) {
@@ -533,14 +559,15 @@ namespace MonoDevelop.Ide.Gui
 			if (ViewChanged != null)
 				ViewChanged (this, args);
 		}
+		
 		bool wasEdited;
-		internal void OnDocumentAttached ()
+		
+		void InitializeEditor (IExtensibleTextEditor editor)
 		{
-			IExtensibleTextEditor editor = GetContent<IExtensibleTextEditor> ();
-			if (editor == null)
-				return;
-			
 			Editor.Document.TextReplaced += (o, a) => {
+				if (parsedDocument != null)
+					parsedDocument.IsInvalid = true;
+
 				if (Editor.Document.IsInAtomicUndo) {
 					wasEdited = true;
 				} else {
@@ -564,24 +591,37 @@ namespace MonoDevelop.Ide.Gui
 			ExtensionNodeList extensions = window.ExtensionContext.GetExtensionNodes ("/MonoDevelop/Ide/TextEditorExtensions", typeof(TextEditorExtensionNode));
 			editorExtension = null;
 			TextEditorExtension last = null;
-			
 			foreach (TextEditorExtensionNode extNode in extensions) {
 				if (!extNode.Supports (FileName))
 					continue;
-				TextEditorExtension ext = (TextEditorExtension) extNode.CreateInstance ();
+				
+				TextEditorExtension ext = (TextEditorExtension)extNode.CreateInstance ();
 				if (ext.ExtendsEditor (this, editor)) {
 					if (editorExtension == null)
 						editorExtension = ext;
+					
 					if (last != null)
 						last.Next = ext;
+					
 					last = ext;
 					ext.Initialize (this);
 				}
 			}
+			
 			if (editorExtension != null)
 				last.Next = editor.AttachExtension (editorExtension);
+		}
+		
+		internal void OnDocumentAttached ()
+		{
+			IExtensibleTextEditor editor = GetContent<IExtensibleTextEditor> ();
+			if (editor != null) {
+				InitializeEditor (editor);
+				RunWhenLoaded (() => ReparseDocument ());
+			}
+			
 			window.Document = this;
-			RunWhenLoaded (() => UpdateParseDocument ());
+			
 			if (window is SdiWorkspaceWindow)
 				((SdiWorkspaceWindow)window).AttachToPathedDocument (GetContent<MonoDevelop.Ide.Gui.Content.IPathedDocument> ());
 		}
@@ -604,6 +644,8 @@ namespace MonoDevelop.Ide.Gui
 		
 		internal void SetProject (Project project)
 		{
+			if (Window.ViewContent.Project == project)
+				return;
 			while (editorExtension != null) {
 				try {
 					editorExtension.Dispose ();
@@ -618,9 +660,21 @@ namespace MonoDevelop.Ide.Gui
 				Window.ViewContent.Project = project;
 				pr.Update (project);
 			}
+			if (project != null)
+				project.Modified += HandleProjectModified;
 			OnDocumentAttached ();
 		}
-		
+
+		void HandleProjectModified (object sender, SolutionItemModifiedEventArgs e)
+		{
+			if (!e.Any (
+					x => x is SolutionItemModifiedEventInfo &&
+				(((SolutionItemModifiedEventInfo)x).Hint == "TargetFramework" ||
+				((SolutionItemModifiedEventInfo)x).Hint == "References")))
+				return;
+			StartReparseThread ();
+		}
+
 		/// <summary>
 		/// This method can take some time to finish. It's not threaded
 		/// </summary>
@@ -634,46 +688,85 @@ namespace MonoDevelop.Ide.Gui
 				var editor = Editor;
 				if (editor == null)
 					return null;
+				lock (TypeSystemService.FilesSkippedInParseThread) {
+					TypeSystemService.FilesSkippedInParseThread.Add (currentParseFile);
+				}
 				string currentParseText = editor.Text;
-				Project curentParseProject = Project;
-				this.parsedDocument = ProjectDomService.Parse (curentParseProject, currentParseFile, currentParseText);
-				if (this.parsedDocument != null && !this.parsedDocument.HasErrors)
-					this.lastErrorFreeParsedDocument = parsedDocument;
+				this.parsedDocument = TypeSystemService.ParseFile (Project, currentParseFile, editor.Document.MimeType, currentParseText);
+				if (Project == null && this.parsedDocument != null) {
+					singleFileContext = GetProjectContext ().UpdateProjectContent (singleFileContext.GetFile (currentParseFile), parsedDocument.ParsedFile);
+				}
 			} finally {
 				OnDocumentParsed (EventArgs.Empty);
 			}
 			return this.parsedDocument;
 		}
+
+		static readonly Lazy<IUnresolvedAssembly> mscorlib = new Lazy<IUnresolvedAssembly> ( () => new CecilLoader ().LoadAssemblyFile (typeof (object).Assembly.Location));
+		static readonly Lazy<IUnresolvedAssembly> systemCore = new Lazy<IUnresolvedAssembly>( () => new CecilLoader ().LoadAssemblyFile (typeof (System.Linq.Enumerable).Assembly.Location));
+		static readonly Lazy<IUnresolvedAssembly> system = new Lazy<IUnresolvedAssembly>( () => new CecilLoader ().LoadAssemblyFile (typeof (System.Uri).Assembly.Location));
+
+		static IUnresolvedAssembly Mscorlib { get { return mscorlib.Value; } }
+		static IUnresolvedAssembly SystemCore { get { return systemCore.Value; } }
+		static IUnresolvedAssembly System { get { return system.Value; } }
+
+		public virtual IProjectContent GetProjectContext ()
+		{
+			if (Project == null) {
+				if (singleFileContext == null) {
+					singleFileContext = new ICSharpCode.NRefactory.CSharp.CSharpProjectContent ();
+					singleFileContext = singleFileContext.AddAssemblyReferences (new [] { Mscorlib, System, SystemCore });
+				}
+				if (parsedDocument != null)
+					return singleFileContext.UpdateProjectContent (null, parsedDocument.ParsedFile);
+				return singleFileContext;
+			}
+			
+			return TypeSystemService.GetProjectContext (Project);
+		}
 		
 		uint parseTimeout = 0;
-		void StartReparseThread ()
+		internal void StartReparseThread ()
 		{
 			// Don't directly parse the document because doing it at every key press is
 			// very inefficient. Do it after a small delay instead, so several changes can
 			// be parsed at the same time.
 			string currentParseFile = FileName;
 			CancelParseTimeout ();
+			
 			parseTimeout = GLib.Timeout.Add (ParseDelay, delegate {
-				string currentParseText = Editor.Text;
-				Project curentParseProject = Project;
-				// parser revice queue takes care of the locking
-				ProjectDomService.QueueParseJob (dom, delegate (string name, IProgressMonitor monitor) {
-					var currentParsedDocument = ProjectDomService.Parse (curentParseProject, currentParseFile, currentParseText);
+				var editor = Editor;
+				if (editor == null)
+					return false;
+				string currentParseText = editor.Text;
+				string mimeType = editor.Document.MimeType;
+				ThreadPool.QueueUserWorkItem (delegate {
+					lock (TypeSystemService.FilesSkippedInParseThread) {
+						TypeSystemService.FilesSkippedInParseThread.Add (currentParseFile);
+					}
+					var currentParsedDocument = TypeSystemService.ParseFile (Project, currentParseFile, mimeType, currentParseText);
 					Application.Invoke (delegate {
 						// this may be called after the document has closed, in that case the OnDocumentParsed event shouldn't be invoked.
 						if (isClosed)
 							return;
 						this.parsedDocument = currentParsedDocument;
-						if (this.parsedDocument != null && !this.parsedDocument.HasErrors)
-							this.lastErrorFreeParsedDocument = parsedDocument;
 						OnDocumentParsed (EventArgs.Empty);
 					});
-				}, FileName);
+				});
 				parseTimeout = 0;
 				return false;
 			});
 		}
 		
+		/// <summary>
+		/// This method kicks off an async document parser and should be used instead of 
+		/// <see cref="UpdateParseDocument"/> unless you need the parsed document immediately.
+		/// </summary>
+		public void ReparseDocument ()
+		{
+			StartReparseThread ();
+		}
+		
 		internal object ExtendedCommandTargetChain {
 			get {
 				// Only go through the text editor chain, if the text editor is selected as subview
@@ -683,12 +776,6 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 
-		public ParsedDocument LastErrorFreeParsedDocument {
-			get {
-				return lastErrorFreeParsedDocument;
-			}
-		}
-		
 		void OnEntryRemoved (object sender, SolutionItemEventArgs args)
 		{
 			if (args.SolutionItem == window.ViewContent.Project)
@@ -721,7 +808,7 @@ namespace MonoDevelop.Ide.Gui
 			
 			Mono.TextEditor.Highlighting.SyntaxMode mode = null;
 			foreach (string mt in DesktopService.GetMimeTypeInheritanceChain (loadedMimeType)) {
-				mode = Mono.TextEditor.Highlighting.SyntaxModeService.GetSyntaxMode (mt);
+				mode = Mono.TextEditor.Highlighting.SyntaxModeService.GetSyntaxMode (null, mt);
 				if (mode != null)
 					break;
 			}
@@ -744,11 +831,11 @@ namespace MonoDevelop.Ide.Gui
 				return null;
 		}
 	
-		public MonoDevelop.Projects.CodeGeneration.CodeGenerator CreateCodeGenerator ()
-		{
-			return MonoDevelop.Projects.CodeGeneration.CodeGenerator.CreateGenerator (Editor.Document.MimeType, 
-				Editor.Options.TabsToSpaces, Editor.Options.TabSize, Editor.EolMarker);
-		}
+//		public MonoDevelop.Projects.CodeGeneration.CodeGenerator CreateCodeGenerator ()
+//		{
+//			return MonoDevelop.Projects.CodeGeneration.CodeGenerator.CreateGenerator (Editor.Document.MimeType, 
+//				Editor.Options.TabsToSpaces, Editor.Options.TabSize, Editor.EolMarker);
+//		}
 
 	}
 	
