@@ -64,13 +64,32 @@ namespace MonoDevelop.Ide
 			get { return mainContext; }
 		}
 		
-		internal MonoDevelopStatusBar()
+		/// <summary>
+		/// For small size changes the caret label is grow only. That ensures that normal caret movement doesn't
+		/// update the whole status bar all the time. But for big jumps in size a resize is done.
+		/// </summary>
+		class CaretStatusLabel : Gtk.Label
+		{
+			public CaretStatusLabel (string label): base (label)
+			{
+			}
+
+			protected override void OnSizeRequested (ref Requisition requisition)
+			{
+				base.OnSizeRequested (ref requisition);
+				const int upperBound = 20;
+				if (Allocation.Width > 0 && Math.Abs (Allocation.Width - requisition.Width) < upperBound)
+					requisition.Width = Math.Max (Allocation.Width, requisition.Width);
+			}
+		}
+
+		internal MonoDevelopStatusBar ()
 		{
 			mainContext = new MainStatusBarContextImpl (this);
 			activeContext = mainContext;
 			contexts.Add (mainContext);
 			
-			Frame originalFrame = (Frame)Children[0];
+			Frame originalFrame = (Frame)Children [0];
 //			originalFrame.WidthRequest = 8;
 //			originalFrame.Shadow = ShadowType.In;
 //			originalFrame.BorderWidth = 0;
@@ -80,33 +99,36 @@ namespace MonoDevelop.Ide
 			
 			// Feedback button
 			
-			CustomFrame fr = new CustomFrame (0, 0, 1, 1);
-			Gdk.Pixbuf px = Gdk.Pixbuf.LoadFromResource ("balloon.png");
-			HBox b = new HBox (false, 3);
-			b.PackStart (new Gtk.Image (px));
-			b.PackStart (new Gtk.Label ("Feedback"));
-			Gtk.Alignment al = new Gtk.Alignment (0f, 0f, 1f, 1f);
-			al.RightPadding = 5;
-			al.LeftPadding = 3;
-			al.Add (b);
-			feedbackButton = new MiniButton (al);
-			//feedbackButton.BackroundColor = new Gdk.Color (200, 200, 255);
-			fr.Add (feedbackButton);
-			PackStart (fr, false, false, 0);
-			feedbackButton.Clicked += HandleFeedbackButtonClicked;
-			feedbackButton.ButtonPressEvent += HandleFeedbackButtonButtonPressEvent;;
-			feedbackButton.ClickOnRelease = true;
-			FeedbackService.FeedbackPositionGetter = delegate {
-				int x, y;
-				feedbackButton.GdkWindow.GetOrigin (out x, out y);
-				x += feedbackButton.Allocation.Width;
-				y -= 6;
-				return new Gdk.Point (x, y);
-			};
+			if (FeedbackService.Enabled) {
+				CustomFrame fr = new CustomFrame (0, 0, 1, 1);
+				Gdk.Pixbuf px = Gdk.Pixbuf.LoadFromResource ("balloon.png");
+				HBox b = new HBox (false, 3);
+				b.PackStart (new Gtk.Image (px));
+				b.PackStart (new Gtk.Label ("Feedback"));
+				Gtk.Alignment al = new Gtk.Alignment (0f, 0f, 1f, 1f);
+				al.RightPadding = 5;
+				al.LeftPadding = 3;
+				al.Add (b);
+				feedbackButton = new MiniButton (al);
+				//feedbackButton.BackroundColor = new Gdk.Color (200, 200, 255);
+				fr.Add (feedbackButton);
+				PackStart (fr, false, false, 0);
+				feedbackButton.Clicked += HandleFeedbackButtonClicked;
+				feedbackButton.ButtonPressEvent += HandleFeedbackButtonButtonPressEvent;
+				;
+				feedbackButton.ClickOnRelease = true;
+				FeedbackService.FeedbackPositionGetter = delegate {
+					int x, y;
+					feedbackButton.GdkWindow.GetOrigin (out x, out y);
+					x += feedbackButton.Allocation.Width;
+					y -= 6;
+					return new Gdk.Point (x, y);
+				};
+			}
 			
 			// Dock area
 			
-			DefaultWorkbench wb = (DefaultWorkbench) IdeApp.Workbench.RootWindow;
+			DefaultWorkbench wb = (DefaultWorkbench)IdeApp.Workbench.RootWindow;
 			wb.DockFrame.ShadedContainer.Add (this);
 			Gtk.Widget dockBar = wb.DockFrame.ExtractDockBar (PositionType.Bottom);
 			dockBar.NoShowAll = true;
@@ -141,15 +163,20 @@ namespace MonoDevelop.Ide
 			textStatusBarPanel.BorderWidth = 0;
 			textStatusBarPanel.ShadowType = ShadowType.None;
 			textStatusBarPanel.Add (statusBox);
-			Label fillerLabel = new Label ();
-			fillerLabel.WidthRequest = 8;
-			statusBox.PackEnd (fillerLabel, false, false, 0);
 			
+			var eventCaretBox = new EventBox ();
+			var caretStatusBox = new HBox ();
 			modeLabel = new Label (" ");
-			statusBox.PackEnd (modeLabel, false, false, 8);
+			caretStatusBox.PackEnd (modeLabel, false, false, 8);
 			
-			cursorLabel = new Label (" ");
-			statusBox.PackEnd (cursorLabel, false, false, 0);
+			cursorLabel = new CaretStatusLabel (" ");
+			caretStatusBox.PackEnd (cursorLabel, false, false, 0);
+			
+			caretStatusBox.GetSizeRequest (out w, out h);
+			caretStatusBox.WidthRequest = w;
+			caretStatusBox.HeightRequest = h;
+			eventCaretBox.Add (caretStatusBox);
+			statusBox.PackEnd (eventCaretBox, false, false, 0);
 			
 			statusIconBox = new HBox ();
 			statusIconBox.BorderWidth = 0;
@@ -194,15 +221,32 @@ namespace MonoDevelop.Ide
 			};
 		}
 		
+		[System.Runtime.InteropServices.DllImport ("libc")]
+		static extern void abort ();
 		
-		bool ignoreFeedbackButtonClick;
-		
+		static readonly bool FeedbackButtonThrowsException = Environment.GetEnvironmentVariable ("MONODEVELOP_TEST_CRASH_REPORTING") != null;
 		void HandleFeedbackButtonButtonPressEvent (object o, ButtonPressEventArgs args)
 		{
 			if (FeedbackService.IsFeedbackWindowVisible)
 				ignoreFeedbackButtonClick = true;
+
+			if (FeedbackButtonThrowsException) {
+				// Control == hard crash
+				if ((args.Event.State & Gdk.ModifierType.ControlMask) != 0) {
+					abort ();
+				}
+				//Alt = terminating exception
+				var ex = new Exception ("Feedback Button is throwing an exception", new Exception (Environment.StackTrace));
+				if ((args.Event.State & Gdk.ModifierType.Mod1Mask) != 0) {
+					throw ex;
+				}
+				// None: Nonterminating exception
+				GLib.ExceptionManager.RaiseUnhandledException (new Exception ("Feedback Button is throwing an exception", new Exception (Environment.StackTrace)), false);
+				ignoreFeedbackButtonClick = true;
+			}
 		}
 
+		bool ignoreFeedbackButtonClick;
 		void HandleFeedbackButtonClicked (object sender, EventArgs e)
 		{
 			if (!ignoreFeedbackButtonClick)
@@ -260,7 +304,6 @@ namespace MonoDevelop.Ide
 		public void ShowCaretState (int line, int column, int selectedChars, bool isInInsertMode)
 		{
 			DispatchService.AssertGuiThread ();
-			
 			string cursorText = selectedChars > 0 ? String.Format ("{0,3} : {1,-3} - {2}", line, column, selectedChars) : String.Format ("{0,3} : {1,-3}", line, column);
 			if (cursorLabel.Text != cursorText)
 				cursorLabel.Text = cursorText;
