@@ -232,44 +232,73 @@ namespace MonoDevelop.Ide.Gui
 		public void OnUppercaseSelection ()
 		{
 			IEditableTextBuffer buffer = GetContent <IEditableTextBuffer> ();
-			if (buffer != null)
-			{
-				if (buffer.SelectedText == String.Empty)
-				{
-					int pos = buffer.CursorPosition;
-					string ch = buffer.GetText (pos, pos + 1);
-					buffer.DeleteText (pos, 1);
-					buffer.InsertText (pos, ch.ToUpper ());
+			if (buffer == null)
+				return;
+			
+			string selectedText = buffer.SelectedText;
+			if (string.IsNullOrEmpty (selectedText)) {
+				int pos = buffer.CursorPosition;
+				string ch = buffer.GetText (pos, pos + 1);
+				string upper = ch.ToUpper ();
+				if (upper == ch) {
 					buffer.CursorPosition = pos + 1;
-				} else
-				{
-					string newText = buffer.SelectedText.ToUpper ();
-					int startPos = buffer.SelectionStartPosition;
-					buffer.DeleteText (startPos, buffer.SelectedText.Length);
+					return;
+				}
+				using (var undo = buffer.OpenUndoGroup ()) {
+					buffer.DeleteText (pos, 1);
+					buffer.InsertText (pos, upper);
+					buffer.CursorPosition = pos + 1;
+				}
+			} else {
+				string newText = selectedText.ToUpper ();
+				if (newText == selectedText)
+					return;
+				int startPos = buffer.SelectionStartPosition;
+				using (var undo = buffer.OpenUndoGroup ()) {
+					buffer.DeleteText (startPos, selectedText.Length);
 					buffer.InsertText (startPos, newText);
+					buffer.Select (startPos, startPos + newText.Length);
 				}
 			}
 		}
 		
+		[CommandUpdateHandler (EditCommands.UppercaseSelection)]
+		protected void OnUppercaseSelection (CommandInfo info)
+		{
+			IEditableTextBuffer buffer = GetContent <IEditableTextBuffer> ();
+			info.Enabled = buffer != null;
+		}
+		
 		[CommandHandler (EditCommands.LowercaseSelection)]
 		public void OnLowercaseSelection ()
 		{
 			IEditableTextBuffer buffer = GetContent <IEditableTextBuffer> ();
-			if (buffer != null)
-			{
-				if (buffer.SelectedText == String.Empty)
-				{
-					int pos = buffer.CursorPosition;
-					string ch = buffer.GetText (pos, pos + 1);
-					buffer.DeleteText (pos, 1);
-					buffer.InsertText (pos, ch.ToLower ());
+			if (buffer == null)
+				return;
+			
+			string selectedText = buffer.SelectedText;
+			if (string.IsNullOrEmpty (selectedText)) {
+				int pos = buffer.CursorPosition;
+				string ch = buffer.GetText (pos, pos + 1);
+				string lower = ch.ToLower ();
+				if (lower == ch) {
 					buffer.CursorPosition = pos + 1;
-				} else
-				{
-					string newText = buffer.SelectedText.ToLower ();
-					int startPos = buffer.SelectionStartPosition;
-					buffer.DeleteText (startPos, buffer.SelectedText.Length);
+					return;
+				};
+				using (var undo = buffer.OpenUndoGroup ()) {
+					buffer.DeleteText (pos, 1);
+					buffer.InsertText (pos, lower);
+					buffer.CursorPosition = pos + 1;
+				}
+			} else {
+				string newText = selectedText.ToLower ();
+				if (newText == selectedText)
+					return;
+				int startPos = buffer.SelectionStartPosition;
+				using (var undo = buffer.OpenUndoGroup ()) {
+					buffer.DeleteText (startPos, selectedText.Length);
 					buffer.InsertText (startPos, newText);
+					buffer.Select (startPos, startPos + newText.Length);
 				}
 			}
 		}
@@ -277,7 +306,8 @@ namespace MonoDevelop.Ide.Gui
 		[CommandUpdateHandler (EditCommands.LowercaseSelection)]
 		protected void OnLowercaseSelection (CommandInfo info)
 		{
-			info.Enabled = GetContent <IEditableTextBuffer> () != null;
+			IEditableTextBuffer buffer = GetContent <IEditableTextBuffer> ();
+			info.Enabled = buffer != null;
 		}
 		
 
@@ -369,7 +399,7 @@ namespace MonoDevelop.Ide.Gui
 		protected void OnDeleteLine ()
 		{
 			var line = doc.Editor.Document.GetLine (doc.Editor.Caret.Line);
-			doc.Editor.Remove (line.Offset, line.Length);
+			doc.Editor.Remove (line.Offset, line.LengthIncludingDelimiter);
 		}
 		
 		struct RemoveInfo
@@ -396,7 +426,7 @@ namespace MonoDevelop.Ide.Gui
 				return ch == ' ' || ch == '\t' || ch == '\v';
 			}
 			
-			public static RemoveInfo GetRemoveInfo (Mono.TextEditor.Document document, ref int pos)
+			public static RemoveInfo GetRemoveInfo (Mono.TextEditor.TextDocument document, ref int pos)
 			{
 				int len = 0;
 				while (pos > 0 && IsWhiteSpace (document.GetCharAt (pos))) {
@@ -424,7 +454,7 @@ namespace MonoDevelop.Ide.Gui
 				return;
 			
 			System.Collections.Generic.List<RemoveInfo> removeList = new System.Collections.Generic.List<RemoveInfo> ();
-			int pos = data.Document.Length - 1;
+			int pos = data.Document.TextLength - 1;
 			RemoveInfo removeInfo = RemoveInfo.GetRemoveInfo (data.Document, ref pos);
 			if (!removeInfo.IsEmpty)
 				removeList.Add (removeInfo);
@@ -441,14 +471,12 @@ namespace MonoDevelop.Ide.Gui
 				}
 				--pos;
 			}
-			
-			data.Document.BeginAtomicUndo ();
-			foreach (var info in removeList) {
-				((Mono.TextEditor.IBuffer)data.Document).Remove (info.Position, info.Length);
-				data.Document.CommitLineUpdate (data.Document.OffsetToLineNumber (info.Position));
+			using (var undo = data.OpenUndoGroup ()) {
+				foreach (var info in removeList) {
+					data.Document.Remove (info.Position, info.Length);
+					data.Document.CommitLineUpdate (data.Document.OffsetToLineNumber (info.Position));
+				}
 			}
-			data.Caret.Offset = Math.Min (data.Caret.Offset, data.Document.Length - 1);
-			data.Document.EndAtomicUndo ();
 		}
 		
 		[CommandUpdateHandler (EditCommands.RemoveTrailingWhiteSpaces)]
