@@ -48,6 +48,8 @@ using Mono.Addins;
 using MonoDevelop.Ide.Projects;
 using MonoDevelop.Core.StringParsing;
 using MonoDevelop.Ide.Navigation;
+using MonoDevelop.Components.Docking;
+using System.Text;
 
 namespace MonoDevelop.Ide.Gui
 {
@@ -83,7 +85,7 @@ namespace MonoDevelop.Ide.Gui
 				monitor.Step (1);
 				
 				((Gtk.Window)workbench).Visible = false;
-				workbench.ActiveWorkbenchWindowChanged += new EventHandler (OnDocumentChanged);
+				workbench.ActiveWorkbenchWindowChanged += OnDocumentChanged;
 				IdeApp.Workspace.StoringUserPreferences += OnStoringWorkspaceUserPreferences;
 				IdeApp.Workspace.LoadingUserPreferences += OnLoadingWorkspaceUserPreferences;
 				
@@ -309,10 +311,20 @@ namespace MonoDevelop.Ide.Gui
 			return AddPad (new PadCodon (padContent, id, label, defaultPlacement, icon));
 		}
 		
+		public Pad AddPad (IPadContent padContent, string id, string label, string defaultPlacement, DockItemStatus defaultStatus, IconId icon)
+		{
+			return AddPad (new PadCodon (padContent, id, label, defaultPlacement, defaultStatus, icon));
+		}
+		
 		public Pad ShowPad (IPadContent padContent, string id, string label, string defaultPlacement, IconId icon)
 		{
 			return ShowPad (new PadCodon (padContent, id, label, defaultPlacement, icon));
 		}
+		
+		public Pad ShowPad (IPadContent padContent, string id, string label, string defaultPlacement, DockItemStatus defaultStatus, IconId icon)
+		{
+			return ShowPad (new PadCodon (padContent, id, label, defaultPlacement, defaultStatus, icon));
+		}
 
 		public Document OpenDocument (FilePath fileName, bool bringToFront)
 		{
@@ -324,7 +336,7 @@ namespace MonoDevelop.Ide.Gui
 			return OpenDocument (fileName, -1, -1, options, null, null);
 		}
 
-		public Document OpenDocument (FilePath fileName, string encoding, OpenDocumentOptions options = OpenDocumentOptions.Default)
+		public Document OpenDocument (FilePath fileName, Encoding encoding, OpenDocumentOptions options = OpenDocumentOptions.Default)
 		{
 			return OpenDocument (fileName, -1, -1, options, encoding, null);
 		}
@@ -334,78 +346,105 @@ namespace MonoDevelop.Ide.Gui
 			return OpenDocument (fileName, line, column, options, null, null);
 		}
 
-		public Document OpenDocument (FilePath fileName, int line, int column, string encoding, OpenDocumentOptions options = OpenDocumentOptions.Default)
+		public Document OpenDocument (FilePath fileName, int line, int column, Encoding encoding, OpenDocumentOptions options = OpenDocumentOptions.Default)
 		{
 			return OpenDocument (fileName, line, column, options, encoding, null);
 		}
 
-		internal Document OpenDocument (FilePath fileName, int line, int column, OpenDocumentOptions options, string encoding, IViewDisplayBinding binding)
+		internal Document OpenDocument (FilePath fileName, int line, int column, OpenDocumentOptions options, Encoding encoding, IViewDisplayBinding binding)
 		{
 			if (string.IsNullOrEmpty (fileName))
 				return null;
-			using (Counters.OpenDocumentTimer.BeginTiming ("Opening file " + fileName)) {
+			// Ensure that paths like /a/./a.cs are equalized 
+			var uniqueName = Path.GetFullPath (fileName);
+			using (Counters.OpenDocumentTimer.BeginTiming ("Opening file " + uniqueName)) {
 				NavigationHistoryService.LogActiveDocument ();
-				
-				Counters.OpenDocumentTimer.Trace ("Look for open document");
-				
-				foreach (Document doc in Documents) {
-					IBaseViewContent vcFound = null;
-					int vcIndex = 0;
-					
-					//search all ViewContents to see if they can "re-use" this filename
-					if (doc.Window.ViewContent.CanReuseView (fileName))
-						vcFound = doc.Window.ViewContent;
-					
-					
-					//old method as fallback
-					if ((vcFound == null) && (doc.FileName == fileName))
-						vcFound = doc.Window.ViewContent;
-					
-					//if found, select window and jump to line
-					if (vcFound != null) {
-						IEditableTextBuffer ipos = vcFound.GetContent<IEditableTextBuffer> ();
-						if (line >= 1 && ipos != null) {
-							ipos.SetCaretTo (line, column >= 1 ? column : 1, options.HasFlag (OpenDocumentOptions.HighlightCaretLine), options.HasFlag (OpenDocumentOptions.CenterCaretLine));
-						}
+				if (options.HasFlag (OpenDocumentOptions.TryToReuseViewer)) {
+					Counters.OpenDocumentTimer.Trace ("Look for open document");
+					foreach (Document doc in Documents) {
+						IBaseViewContent vcFound = null;
+						int vcIndex = 0;
+
+						//search all ViewContents to see if they can "re-use" this filename
+						if (doc.Window.ViewContent.CanReuseView (uniqueName))
+							vcFound = doc.Window.ViewContent;
 						
-						if (options.HasFlag (OpenDocumentOptions.BringToFront)) {
-							doc.Select ();
-							doc.Window.SwitchView (vcIndex);
-							doc.Window.SelectWindow ();
-							NavigationHistoryService.LogActiveDocument ();
-							Present ();
+						//old method as fallback
+						if ((vcFound == null) && (doc.FileName == uniqueName || doc.FileName == fileName))
+							vcFound = doc.Window.ViewContent;
+						//if found, select window and jump to line
+						if (vcFound != null) {
+							IEditableTextBuffer ipos = vcFound.GetContent<IEditableTextBuffer> ();
+							if (line >= 1 && ipos != null) {
+								ipos.SetCaretTo (
+									line,
+									column >= 1 ? column : 1,
+									options.HasFlag (OpenDocumentOptions.HighlightCaretLine),
+									options.HasFlag (OpenDocumentOptions.CenterCaretLine)
+								);
+							}
+							
+							if (options.HasFlag (OpenDocumentOptions.BringToFront)) {
+								doc.Select ();
+								doc.Window.SwitchView (vcIndex);
+								doc.Window.SelectWindow ();
+								NavigationHistoryService.LogActiveDocument ();
+								Present ();
+							}
+							return doc;
 						}
-						return doc;
 					}
 				}
-				
 				Counters.OpenDocumentTimer.Trace ("Initializing monitor");
-				IProgressMonitor pm = ProgressMonitors.GetStatusProgressMonitor (GettextCatalog.GetString ("Opening {0}", fileName), Stock.OpenFileIcon, true);
+				IProgressMonitor pm = ProgressMonitors.GetStatusProgressMonitor (
+					GettextCatalog.GetString ("Opening {0}", uniqueName),
+					Stock.OpenFileIcon,
+					true
+				);
 				var openFileInfo = new FileOpenInformation () {
-					ProgressMonitor = pm,
-					FileName = fileName,
+					FileName = uniqueName,
 					Options = options,
 					Line = line,
 					Column = column,
 					DisplayBinding = binding,
 					Encoding = encoding
 				};
-				RealOpenFile (openFileInfo);
 				
-				if (!pm.AsyncOperation.Success)
-					return null;
+				RealOpenFile (pm, openFileInfo);
+				pm.Dispose ();
 				
 				if (openFileInfo.NewContent != null) {
 					Counters.OpenDocumentTimer.Trace ("Wrapping document");
 					Document doc = WrapDocument (openFileInfo.NewContent.WorkbenchWindow);
-					if (options.HasFlag (OpenDocumentOptions.BringToFront)) {
+					if (doc != null && options.HasFlag (OpenDocumentOptions.BringToFront)) {
 						Present ();
-						doc.RunWhenLoaded (() => doc.Window.SelectWindow ());
+						doc.RunWhenLoaded (() => {
+							if (doc.Window != null)
+								doc.Window.SelectWindow ();
+						});
 					}
 					return doc;
-				} else {
-					return null;
 				}
+				return null;
+			}
+		}
+		
+		IViewContent BatchOpenDocument (IProgressMonitor monitor, FilePath fileName, int line, int column)
+		{
+			if (string.IsNullOrEmpty (fileName))
+				return null;
+			
+			using (Counters.OpenDocumentTimer.BeginTiming ("Batch opening file " + fileName)) {
+				var openFileInfo = new FileOpenInformation () {
+					FileName = fileName,
+					Options = OpenDocumentOptions.OnlyInternalViewer,
+					Line = line,
+					Column = column,
+				};
+				
+				RealOpenFile (monitor, openFileInfo);
+				
+				return openFileInfo.NewContent;
 			}
 		}
 		
@@ -451,7 +490,9 @@ namespace MonoDevelop.Ide.Gui
 			workbench.ShowView (newContent, true);
 			DisplayBindingService.AttachSubWindows (newContent.WorkbenchWindow);
 			
-			return WrapDocument (newContent.WorkbenchWindow);
+			var document = WrapDocument (newContent.WorkbenchWindow);
+			document.StartReparseThread ();
+			return document;
 		}
 		
 		public void ShowGlobalPreferencesDialog (Gtk.Window parentWindow)
@@ -459,6 +500,9 @@ namespace MonoDevelop.Ide.Gui
 			ShowGlobalPreferencesDialog (parentWindow, null);
 		}
 		
+		static Properties properties = ((Properties) PropertyService.Get (
+			"MonoDevelop.TextEditor.Document.Document.DefaultDocumentAggregatorProperties",
+			new Properties()));
 		public void ShowGlobalPreferencesDialog (Gtk.Window parentWindow, string panelId)
 		{
 			if (parentWindow == null)
@@ -466,7 +510,7 @@ namespace MonoDevelop.Ide.Gui
 
 			OptionsDialog ops = new OptionsDialog (
 				parentWindow,
-				TextEditorProperties.Properties,
+				properties,
 				"/MonoDevelop/Ide/GlobalOptionsDialog");
 
 			try {
@@ -578,6 +622,7 @@ namespace MonoDevelop.Ide.Gui
 			
 			doc.OnDocumentAttached ();
 			OnDocumentOpened (new DocumentEventArgs (doc));
+			
 			return doc;
 		}
 		
@@ -652,6 +697,8 @@ namespace MonoDevelop.Ide.Gui
 			if (IdeApp.ProjectOperations.CurrentSelectedProject != null) {
 				if (IdeApp.ProjectOperations.CurrentSelectedProject.Files.GetFile (fileName) != null)
 					project = IdeApp.ProjectOperations.CurrentSelectedProject;
+				else if (IdeApp.ProjectOperations.CurrentSelectedProject.FileName == fileName)
+					project = IdeApp.ProjectOperations.CurrentSelectedProject;
 			}
 			if (project == null && IdeApp.ProjectOperations.CurrentSelectedWorkspaceItem != null) {
 				project = IdeApp.ProjectOperations.CurrentSelectedWorkspaceItem.GetProjectContainingFile (fileName);
@@ -669,118 +716,100 @@ namespace MonoDevelop.Ide.Gui
 			return project;
 		}
 		
-		
-		void RealOpenFile (FileOpenInformation openFileInfo)
+		void RealOpenFile (IProgressMonitor monitor, FileOpenInformation openFileInfo)
 		{
 			FilePath fileName;
-			IProgressMonitor monitor = openFileInfo.ProgressMonitor;
-
-			using (monitor)
-			{
-				Counters.OpenDocumentTimer.Trace ("Checking file");
-				
-				string origName = openFileInfo.FileName;
-
-				if (origName == null) {
-					monitor.ReportError (GettextCatalog.GetString ("Invalid file name"), null);
-					return;
-				}
-
-				if (origName.StartsWith ("file://"))
-					fileName = new Uri (origName).LocalPath;
-				else
-					fileName = origName;
-
-				if (!origName.StartsWith ("http://"))
-					fileName = fileName.FullPath;
-				
-				//Debug.Assert(FileService.IsValidPath(fileName));
-				if (FileService.IsDirectory (fileName)) {
-					monitor.ReportError (GettextCatalog.GetString ("{0} is a directory", fileName), null);
-					return;
-				}
-				// test, if file fileName exists
-				if (!origName.StartsWith("http://")) {
-					// test, if an untitled file should be opened
-					if (!System.IO.Path.IsPathRooted(origName)) { 
-						foreach (Document doc in Documents) {
-							if (doc.Window.ViewContent.IsUntitled && doc.Window.ViewContent.UntitledName == origName) {
-								doc.Select ();
-								openFileInfo.NewContent = doc.Window.ViewContent;
-								return;
-							}
-						}
-					}
-					if (!File.Exists (fileName)) {
-						monitor.ReportError (GettextCatalog.GetString ("File not found: {0}", fileName), null);
-						return;
-					}
-				}
-				
-				foreach (Document doc in Documents) {
-					if (doc.FileName == fileName) {
-						if (openFileInfo.Options.HasFlag (OpenDocumentOptions.BringToFront)) {
+			
+			Counters.OpenDocumentTimer.Trace ("Checking file");
+			
+			string origName = openFileInfo.FileName;
+			
+			if (origName == null) {
+				monitor.ReportError (GettextCatalog.GetString ("Invalid file name"), null);
+				return;
+			}
+			
+			if (origName.StartsWith ("file://"))
+				fileName = new Uri (origName).LocalPath;
+			else
+				fileName = origName;
+			
+			if (!origName.StartsWith ("http://"))
+				fileName = fileName.FullPath;
+			
+			//Debug.Assert(FileService.IsValidPath(fileName));
+			if (FileService.IsDirectory (fileName)) {
+				monitor.ReportError (GettextCatalog.GetString ("{0} is a directory", fileName), null);
+				return;
+			}
+			
+			// test, if file fileName exists
+			if (!origName.StartsWith("http://")) {
+				// test, if an untitled file should be opened
+				if (!System.IO.Path.IsPathRooted(origName)) {
+					foreach (Document doc in Documents) {
+						if (doc.Window.ViewContent.IsUntitled && doc.Window.ViewContent.UntitledName == origName) {
 							doc.Select ();
-							doc.RunWhenLoaded (delegate {
-								IEditableTextBuffer ipos = doc.GetContent <IEditableTextBuffer> ();
-								if (openFileInfo.Line > 0 && ipos != null) {
-									ipos.SetCaretTo (openFileInfo.Line, Math.Max (1, openFileInfo.Column), openFileInfo.Options.HasFlag (OpenDocumentOptions.HighlightCaretLine));
-								}
-							});
+							openFileInfo.NewContent = doc.Window.ViewContent;
+							return;
 						}
-						openFileInfo.NewContent = doc.Window.ViewContent;
-						return;
 					}
 				}
 				
-				Counters.OpenDocumentTimer.Trace ("Looking for binding");
-				
-				IDisplayBinding binding = null;
-				IViewDisplayBinding viewBinding = null;
-				Project project = GetProjectContainingFile (fileName);
-				
-				if (openFileInfo.DisplayBinding != null) {
-					binding = viewBinding = openFileInfo.DisplayBinding;
-				} else {
-					var bindings = DisplayBindingService.GetDisplayBindings (fileName, null, project).Where (d => d.CanUseAsDefault);
-					if (openFileInfo.Options.HasFlag (OpenDocumentOptions.OnlyInternalViewer)) {
-						binding = bindings.OfType<IViewDisplayBinding>().FirstOrDefault ();
-						viewBinding = (IViewDisplayBinding) binding;
+				if (!File.Exists (fileName)) {
+					monitor.ReportError (GettextCatalog.GetString ("File not found: {0}", fileName), null);
+					return;
+				}
+			}
+			
+			Counters.OpenDocumentTimer.Trace ("Looking for binding");
+			
+			IDisplayBinding binding = null;
+			IViewDisplayBinding viewBinding = null;
+			Project project = GetProjectContainingFile (fileName);
+			
+			if (openFileInfo.DisplayBinding != null) {
+				binding = viewBinding = openFileInfo.DisplayBinding;
+			} else {
+				var bindings = DisplayBindingService.GetDisplayBindings (fileName, null, project).Where (d => d.CanUseAsDefault);
+				if (openFileInfo.Options.HasFlag (OpenDocumentOptions.OnlyInternalViewer)) {
+					binding = bindings.OfType<IViewDisplayBinding>().FirstOrDefault ();
+					viewBinding = (IViewDisplayBinding) binding;
+				}
+				else if (openFileInfo.Options.HasFlag (OpenDocumentOptions.OnlyExternalViewer)) {
+					binding = bindings.OfType<IExternalDisplayBinding>().FirstOrDefault ();
+					viewBinding = null;
+				}
+				else {
+					binding = bindings.FirstOrDefault ();
+					viewBinding = binding as IViewDisplayBinding;
+				}
+			}
+			
+			try {
+				if (binding != null) {
+					if (viewBinding != null)  {
+						var fw = new LoadFileWrapper (monitor, workbench, viewBinding, project, openFileInfo);
+						fw.Invoke (fileName);
+					} else {
+						var extBinding = (IExternalDisplayBinding)binding;
+						var app = extBinding.GetApplication (fileName, null, project);
+						app.Launch (fileName);
 					}
-					else if (openFileInfo.Options.HasFlag (OpenDocumentOptions.OnlyExternalViewer)) {
-						binding = bindings.OfType<IExternalDisplayBinding>().FirstOrDefault ();
-						viewBinding = null;
-					}
-					else {
-						binding = bindings.FirstOrDefault ();
-						viewBinding = binding as IViewDisplayBinding;
+					
+					Counters.OpenDocumentTimer.Trace ("Adding to recent files");
+					DesktopService.RecentFiles.AddFile (fileName, project);
+				} else if (!openFileInfo.Options.HasFlag (OpenDocumentOptions.OnlyInternalViewer)) {
+					try {
+						Counters.OpenDocumentTimer.Trace ("Showing in browser");
+						DesktopService.OpenFile (fileName);
+					} catch (Exception ex) {
+						LoggingService.LogError ("Error opening file: " + fileName, ex);
+						MessageService.ShowError (GettextCatalog.GetString ("File '{0}' could not be opened", fileName));
 					}
 				}
-				try {
-					if (binding != null) {
-						if (viewBinding != null)  {
-							var fw = new LoadFileWrapper (workbench, viewBinding, project, openFileInfo);
-							fw.Invoke (fileName);
-						} else {
-							var extBinding = (IExternalDisplayBinding)binding;
-							var app = extBinding.GetApplication (fileName, null, project);
-							app.Launch (fileName);
-						}
-						
-						Counters.OpenDocumentTimer.Trace ("Adding to recent files");
-						DesktopService.RecentFiles.AddFile (fileName, project);
-					} else if (!openFileInfo.Options.HasFlag (OpenDocumentOptions.OnlyInternalViewer)) {
-						try {
-							Counters.OpenDocumentTimer.Trace ("Showing in browser");
-							DesktopService.OpenFile (fileName);
-						} catch (Exception ex) {
-							LoggingService.LogError ("Error opening file: " + fileName, ex);
-							MessageService.ShowError (GettextCatalog.GetString ("File '{0}' could not be opened", fileName));
-						}
-					}
-				} catch (Exception ex) {
-					monitor.ReportError ("", ex);
-				}
+			} catch (Exception ex) {
+				monitor.ReportError ("", ex);
 			}
 		}
 		
@@ -830,15 +859,41 @@ namespace MonoDevelop.Ide.Gui
 			if (prefs == null)
 				return;
 			
-			string currentFileName = prefs.ActiveDocument != null ? Path.GetFullPath (Path.Combine (args.Item.BaseDirectory, prefs.ActiveDocument)) : null;
+			NavigationHistoryService.LogActiveDocument ();
 			
-			foreach (DocumentUserPrefs doc in prefs.Files) {
-				FilePath fileName = args.Item.BaseDirectory.Combine (doc.FileName).FullPath;
-				if (File.Exists (fileName)) {
-					OpenDocumentOptions ops = OpenDocumentOptions.OnlyInternalViewer;
-					if (fileName == currentFileName)
-						ops |= OpenDocumentOptions.BringToFront;
-					IdeApp.Workbench.OpenDocument (fileName, doc.Line, doc.Column, ops, null, null);
+			List<IViewContent> docViews = new List<IViewContent> ();
+			FilePath baseDir = args.Item.BaseDirectory;
+			IViewContent currentView = null;
+			
+			using (IProgressMonitor pm = ProgressMonitors.GetStatusProgressMonitor (GettextCatalog.GetString ("Loading workspace documents"), Stock.OpenFileIcon, true)) {
+				string currentFileName = prefs.ActiveDocument != null ? baseDir.Combine (prefs.ActiveDocument).FullPath : null;
+				
+				foreach (DocumentUserPrefs doc in prefs.Files.Distinct (new DocumentUserPrefsFilenameComparer ())) {
+					string fileName = baseDir.Combine (doc.FileName).FullPath;
+					if (File.Exists (fileName)) {
+						var view = IdeApp.Workbench.BatchOpenDocument (pm, fileName, doc.Line, doc.Column);
+						if (fileName == currentFileName)
+							currentView = view;
+						
+						if (view != null)
+							docViews.Add (view);
+					}
+				}
+				
+				// Note: At this point, the progress monitor will be disposed which causes the gtk main-loop to be pumped.
+				// This is EXTREMELY important, because without this main-loop pumping action, the next foreach() loop will
+				// not cause the Solution tree-view to properly expand, nor will the ActiveDocument be set properly.
+			}
+			
+			foreach (var view in docViews) {
+				Document doc = WrapDocument (view.WorkbenchWindow);
+				if (view == currentView) {
+					Present ();
+					doc.RunWhenLoaded (() => {
+						var window = doc.Window;
+						if (window != null)
+							window.SelectWindow ();
+					});
 				}
 			}
 			
@@ -911,7 +966,7 @@ namespace MonoDevelop.Ide.Gui
 					foreach (FilePath file in files) {
 						try {
 							FileInfo fi = new FileInfo (file);
-							FileData fd = new FileData (file, fi.Exists ? fi.LastWriteTime : DateTime.MinValue);
+							FileData fd = new FileData (file, fi.Exists ? fi.LastWriteTimeUtc : DateTime.MinValue);
 							fileStatus.Add (fd);
 						} catch {
 							// Ignore
@@ -937,9 +992,9 @@ namespace MonoDevelop.Ide.Gui
 						try {
 							FileInfo fi = new FileInfo (fd.File);
 							if (fi.Exists) {
-								if (fi.LastWriteTime != fd.Time)
+								if (fi.LastWriteTimeUtc != fd.TimeUtc)
 									modified.Add (fd.File);
-							} else if (fd.Time != DateTime.MinValue) {
+							} else if (fd.TimeUtc != DateTime.MinValue) {
 								FileService.NotifyFileRemoved (fd.File);
 							}
 						} catch {
@@ -961,18 +1016,22 @@ namespace MonoDevelop.Ide.Gui
 				foreach (FilePath file in item.GetItemFiles (true))
 					yield return file;
 			}
+			foreach (Document doc in documents) {
+				if (!doc.HasProject && doc.IsFile)
+					yield return doc.FileName;
+			}
 		}
 		
 		struct FileData
 		{
-			public FileData (FilePath file, DateTime time)
+			public FileData (FilePath file, DateTime timeUtc)
 			{
 				this.File = file;
-				this.Time = time;
+				this.TimeUtc = timeUtc;
 			}
 			
 			public FilePath File;
-			public DateTime Time;
+			public DateTime TimeUtc;
 		}
 		
 		protected virtual void OnDocumentOpened (DocumentEventArgs e)
@@ -988,21 +1047,20 @@ namespace MonoDevelop.Ide.Gui
 		{
 			foreach (var doc in Documents) {
 				if (doc.ParsedDocument != null)
-					doc.UpdateParseDocument ();
+					doc.ReparseDocument ();
 			}
 		}
 	}
 
 	public class FileOpenInformation
 	{
-		public IProgressMonitor ProgressMonitor { get; set; }
 		public string FileName { get; set; }
 		public OpenDocumentOptions Options { get; set; }
 		public int Line { get; set; }
 		public int Column { get; set; }
 		public IViewDisplayBinding DisplayBinding { get; set; }
 		public IViewContent NewContent { get; set; }
-		public string Encoding { get; set; }
+		public Encoding Encoding { get; set; }
 		
 		public FileOpenInformation ()
 		{
@@ -1023,17 +1081,19 @@ namespace MonoDevelop.Ide.Gui
 		Project project;
 		FileOpenInformation fileInfo;
 		DefaultWorkbench workbench;
+		IProgressMonitor monitor;
 		IViewContent newContent;
 		
-		public LoadFileWrapper (DefaultWorkbench workbench, IViewDisplayBinding binding, FileOpenInformation fileInfo)
+		public LoadFileWrapper (IProgressMonitor monitor, DefaultWorkbench workbench, IViewDisplayBinding binding, FileOpenInformation fileInfo)
 		{
+			this.monitor = monitor;
 			this.workbench = workbench;
 			this.fileInfo = fileInfo;
 			this.binding = binding;
 		}
 		
-		public LoadFileWrapper (DefaultWorkbench workbench, IViewDisplayBinding binding, Project project, FileOpenInformation fileInfo)
-			: this (workbench, binding, fileInfo)
+		public LoadFileWrapper (IProgressMonitor monitor, DefaultWorkbench workbench, IViewDisplayBinding binding, Project project, FileOpenInformation fileInfo)
+			: this (monitor, workbench, binding, fileInfo)
 		{
 			this.project = project;
 		}
@@ -1046,10 +1106,10 @@ namespace MonoDevelop.Ide.Gui
 				if (binding.CanHandle (fileName, mimeType, project)) {
 					newContent = binding.CreateContent (fileName, mimeType, project);
 				} else {
-					fileInfo.ProgressMonitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), null);
+					monitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), null);
 				}
 				if (newContent == null) {
-					fileInfo.ProgressMonitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), null);
+					monitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), null);
 					return;
 				}
 				
@@ -1065,16 +1125,17 @@ namespace MonoDevelop.Ide.Gui
 					else
 						newContent.Load (fileName);
 				} catch (InvalidEncodingException iex) {
-					fileInfo.ProgressMonitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not opened. {1}", fileName, iex.Message), null);
+					monitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not opened. {1}", fileName, iex.Message), null);
 					return;
 				} catch (OverflowException) {
-					fileInfo.ProgressMonitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not opened. File too large.", fileName), null);
+					monitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not opened. File too large.", fileName), null);
 					return;
 				}
 			} catch (Exception ex) {
-				fileInfo.ProgressMonitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), ex);
+				monitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), ex);
 				return;
 			}
+			
 			// content got re-used
 			if (newContent.WorkbenchWindow != null) {
 				newContent.WorkbenchWindow.SelectWindow ();
@@ -1086,12 +1147,12 @@ namespace MonoDevelop.Ide.Gui
 			
 			workbench.ShowView (newContent, fileInfo.Options.HasFlag (OpenDocumentOptions.BringToFront));
 			DisplayBindingService.AttachSubWindows (newContent.WorkbenchWindow);
-			
 			newContent.WorkbenchWindow.DocumentType = binding.Name;
 			
 			IEditableTextBuffer ipos = newContent.GetContent<IEditableTextBuffer> ();
 			if (fileInfo.Line > 0 && ipos != null)
 				JumpToLine ();
+			
 			fileInfo.NewContent = newContent;
 		}
 		
@@ -1113,7 +1174,9 @@ namespace MonoDevelop.Ide.Gui
 		HighlightCaretLine = 1 << 2,
 		OnlyInternalViewer = 1 << 3,
 		OnlyExternalViewer = 1 << 4,
+		TryToReuseViewer = 1 << 5,
 		
-		Default = BringToFront | CenterCaretLine | HighlightCaretLine
+		Default = BringToFront | CenterCaretLine | HighlightCaretLine | TryToReuseViewer,
+		Debugger = BringToFront | CenterCaretLine | TryToReuseViewer
 	}
 }
