@@ -42,8 +42,8 @@ using MonoDevelop.Core.Instrumentation;
 using MonoDevelop.Core.Text;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.Dom.Parser;
+using ICSharpCode.NRefactory.TypeSystem;
+using MonoDevelop.Ide.TypeSystem;
 
 namespace MonoDevelop.Ide.NavigateToDialog
 {
@@ -172,44 +172,17 @@ namespace MonoDevelop.Ide.NavigateToDialog
 				PropertyService.Set ("NavigateToDialog.DialogWidth", args.Allocation.Width);
 				PropertyService.Set ("NavigateToDialog.DialogHeight", args.Allocation.Height);
 			};
+			
 		}
 		
-		Thread collectFiles, collectTypes;
+		Thread collectFiles;
 		void StartCollectThreads ()
 		{
-			members = new List<IMember> ();
-			types = new List<IType> ();
-			
 			StartCollectFiles ();
-			StartCollectTypes ();
 		}
 		
 		static TimerCounter getMembersTimer = InstrumentationService.CreateTimerCounter ("Time to get all members", "NavigateToDialog");
 		
-		void StartCollectTypes ()
-		{
-			ThreadPool.QueueUserWorkItem (delegate {
-				CollectTypes ();
-				
-				if (isAbleToSearchMembers) {
-					getMembersTimer.BeginTiming ();
-					try {
-						lock (members) {
-							foreach (IType type in types) {
-								foreach (IMember m in type.Members) {
-									if (m is IType)
-										continue;
-									members.Add (m);
-								}
-							}
-						}
-					} finally {
-						getMembersTimer.EndTiming ();
-					}
-				}
-			});
-		}
-		
 		void StartCollectFiles ()
 		{
 			ThreadPool.QueueUserWorkItem (delegate {
@@ -274,7 +247,6 @@ namespace MonoDevelop.Ide.NavigateToDialog
 			StopActiveSearch ();
 			
 			WaitForCollectFiles ();
-			WaitForCollectTypes ();
 			
 			string toMatch = matchEntry.Query;
 			
@@ -312,7 +284,7 @@ namespace MonoDevelop.Ide.NavigateToDialog
 		class WorkerResult 
 		{
 			public List<ProjectFile> filteredFiles = null;
-			public List<IType> filteredTypes = null;
+			public List<ITypeDefinition> filteredTypes = null;
 			public List<IMember> filteredMembers  = null;
 			
 			public string pattern = null;
@@ -323,6 +295,8 @@ namespace MonoDevelop.Ide.NavigateToDialog
 			
 			public bool IncludeFiles, IncludeTypes, IncludeMembers;
 			
+			public Ambience ambience;
+			
 			public StringMatcher matcher = null;
 			
 			public WorkerResult (Widget widget)
@@ -340,36 +314,36 @@ namespace MonoDevelop.Ide.NavigateToDialog
 				if (!FullSearch)
 					return null;
 				matchString = FileSearchResult.GetRelProjectPath (file);
-				if (MatchName (FileSearchResult.GetRelProjectPath (file), out rank)) 
+				if (MatchName (matchString, out rank)) 
 					return new FileSearchResult (pattern, matchString, rank, file, false);
 				
 				return null;
 			}
 			
-			internal SearchResult CheckType (IType type)
+			internal SearchResult CheckType (ITypeDefinition type)
 			{
 				int rank;
 				if (MatchName (type.Name, out rank))
-					return new TypeSearchResult (pattern, type.Name, rank, type, false);
+					return new TypeSearchResult (pattern, type.Name, rank, type, false) { Ambience = ambience };
 				if (!FullSearch)
 					return null;
 				if (MatchName (type.FullName, out rank))
-					return new TypeSearchResult (pattern, type.FullName, rank, type, true);
+					return new TypeSearchResult (pattern, type.FullName, rank, type, true) { Ambience = ambience };
 				return null;
 			}
 			
 			internal SearchResult CheckMember (IMember member)
 			{
 				int rank;
-				bool useDeclaringTypeName = member is IMethod && (((IMethod)member).IsConstructor || ((IMethod)member).IsFinalizer);
+				bool useDeclaringTypeName = member is IMethod && (((IMethod)member).IsConstructor || ((IMethod)member).IsDestructor);
 				string memberName = useDeclaringTypeName ? member.DeclaringType.Name : member.Name;
 				if (MatchName (memberName, out rank))
-					return new MemberSearchResult (pattern, memberName, rank, member, false);
+					return new MemberSearchResult (pattern, memberName, rank, member, false) { Ambience = ambience };
 				if (!FullSearch)
 					return null;
 				memberName = useDeclaringTypeName ? member.DeclaringType.FullName : member.FullName;
 				if (MatchName (memberName, out rank))
-					return new MemberSearchResult (pattern, memberName, rank, member, true);
+					return new MemberSearchResult (pattern, memberName, rank, member, true) { Ambience = ambience };
 				return null;
 			}
 			
@@ -388,8 +362,24 @@ namespace MonoDevelop.Ide.NavigateToDialog
 		}
 		
 		IEnumerable<ProjectFile> files;
-		List<IType> types;
-		List<IMember> members;
+		
+		IEnumerable<IMember> members {
+			get {
+				getMembersTimer.BeginTiming ();
+				try {
+					lock (members) {
+						foreach (var type in types) {
+							foreach (var m in type.Members) {
+								yield return m;
+							}
+						}
+					}
+				} finally {
+					getMembersTimer.EndTiming ();
+				}
+				
+			}
+		}
 		
 		WorkerResult lastResult;
 		
@@ -405,6 +395,8 @@ namespace MonoDevelop.Ide.NavigateToDialog
 			newResult.IncludeFiles = (NavigateToType & NavigateToType.Files) == NavigateToType.Files;
 			newResult.IncludeTypes = (NavigateToType & NavigateToType.Types) == NavigateToType.Types;
 			newResult.IncludeMembers = (NavigateToType & NavigateToType.Members) == NavigateToType.Members;
+			var firstType = types.FirstOrDefault ();
+			newResult.ambience = firstType != null ? AmbienceService.GetAmbienceForFile (firstType.Region.FileName) : AmbienceService.DefaultAmbience;
 			
 			string toMatch = arg.Key;
 			int i = toMatch.IndexOf (':');
@@ -452,11 +444,11 @@ namespace MonoDevelop.Ide.NavigateToDialog
 			
 			// Search Types
 			if (newResult.IncludeTypes) {
-				newResult.filteredTypes = new List<IType> ();
+				newResult.filteredTypes = new List<ITypeDefinition> ();
 				lock (types) {
 					bool startsWithLastFilter = lastResult.pattern != null && newResult.pattern.StartsWith (lastResult.pattern) && lastResult.filteredTypes != null;
-					List<IType> allTypes = startsWithLastFilter ? lastResult.filteredTypes : types;
-					foreach (IType type in allTypes) {
+					var allTypes = startsWithLastFilter ? lastResult.filteredTypes : types;
+					foreach (var type in allTypes) {
 						if (worker.CancellationPending)
 							yield break;
 						SearchResult curResult = newResult.CheckType (type);
@@ -473,8 +465,8 @@ namespace MonoDevelop.Ide.NavigateToDialog
 				newResult.filteredMembers = new List<IMember> ();
 				lock (members) {
 					bool startsWithLastFilter = lastResult.pattern != null && newResult.pattern.StartsWith (lastResult.pattern) && lastResult.filteredMembers != null;
-					List<IMember> allMembers = startsWithLastFilter ? lastResult.filteredMembers : members;
-					foreach (IMember member in allMembers) {
+					var allMembers = startsWithLastFilter ? lastResult.filteredMembers : members;
+					foreach (var member in allMembers) {
 						if (worker.CancellationPending)
 							yield break;
 						SearchResult curResult = newResult.CheckMember (member);
@@ -488,14 +480,6 @@ namespace MonoDevelop.Ide.NavigateToDialog
 		}
 		
 		
-		void WaitForCollectTypes ()
-		{
-			if (collectTypes != null) {
-				collectTypes.Join ();
-				collectTypes= null;
-			}
-		}
-		
 		void WaitForCollectFiles ()
 		{
 			if (collectFiles != null) {
@@ -509,6 +493,8 @@ namespace MonoDevelop.Ide.NavigateToDialog
 			public int Compare (SearchResult o1, SearchResult o2)
 			{
 				var r = o2.Rank.CompareTo (o1.Rank);
+				if (r == 0)
+					r = o1.SearchResultType.CompareTo (o2.SearchResultType);
 				if (r == 0)
 					return String.CompareOrdinal (o1.MatchedString, o2.MatchedString);
 				return r;
@@ -539,33 +525,32 @@ namespace MonoDevelop.Ide.NavigateToDialog
 		
 		static TimerCounter getTypesTimer = InstrumentationService.CreateTimerCounter ("Time to get all types", "NavigateToDialog");
 		
-		void CollectTypes ()
-		{
-			lock (types) {
+		
+		IEnumerable<ITypeDefinition> types {
+			get {
 				getTypesTimer.BeginTiming ();
 				try {
 					foreach (Document doc in IdeApp.Workbench.Documents) {
 						// We only want to check it here if it's not part
-						// of the open combine.  Otherwise, it will get
+						// of the open combine. Otherwise, it will get
 						// checked down below.
 						if (doc.Project == null && doc.IsFile) {
-							ICompilationUnit info = doc.CompilationUnit;
+							var info = doc.ParsedDocument;
 							if (info != null) {
-								foreach (IType c in info.Types) {
-									types.Add (c);
+								var ctx = doc.Compilation;
+								foreach (var type in ctx.MainAssembly.GetAllTypeDefinitions ()) {
+									yield return type;
 								}
 							}
 						}
 					}
 					
 					ReadOnlyCollection<Project> projects = IdeApp.Workspace.GetAllProjects ();
-		
+					
 					foreach (Project p in projects) {
-						ProjectDom dom = ProjectDomService.GetProjectDom (p);
-						if (dom == null)
-							continue;
-						foreach (IType c in dom.Types)
-							AddType (c, types);
+						var pctx = TypeSystemService.GetCompilation (p);
+						foreach (var type in pctx.MainAssembly.GetAllTypeDefinitions ())
+							yield return type;
 					}
 				} finally {
 					getTypesTimer.EndTiming ();
@@ -573,13 +558,6 @@ namespace MonoDevelop.Ide.NavigateToDialog
 			}
 		}
 
-		void AddType (IType c, List<IType> list)
-		{
-			list.Add (c);
-			foreach (IType ct in c.InnerTypes)
-				AddType (ct, list);
-		}
-		
 		struct MatchResult 
 		{
 			public bool Match;
