@@ -32,6 +32,8 @@ using System.Xml;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.Ide.Extensions;
 using System.Reflection;
+using System.Linq;
+using Mono.Addins;
 
 namespace MonoDevelop.Ide
 {
@@ -40,6 +42,20 @@ namespace MonoDevelop.Ide
 		static bool sending;
 		static object sendingLock = new object ();
 		static FeedbackDialog currentFeedbackDialog;
+		static Lazy<string> feedbackUrl;
+
+		internal static bool Enabled {
+			get { return !string.IsNullOrEmpty (feedbackUrl.Value); }
+		}
+
+		static FeedbackService ()
+		{
+			feedbackUrl = new Lazy<string> (() => {
+				var node = AddinManager.GetExtensionNodes<ServiceUrlExtensionNode> ("/MonoDevelop/Ide/FeedbackService")
+					.FirstOrDefault ();
+				return node != null? node.Url : null;
+			});
+		}
 		
 		public static bool IsFeedbackWindowVisible {
 			get { return currentFeedbackDialog != null && currentFeedbackDialog.Visible; }
@@ -47,6 +63,9 @@ namespace MonoDevelop.Ide
 		
 		public static void ShowFeedbackWindow ()
 		{
+			if (string.IsNullOrEmpty (feedbackUrl.Value))
+				return;
+
 			if (currentFeedbackDialog == null) {
 				var p = FeedbackPositionGetter ();
 				currentFeedbackDialog = new FeedbackDialog (p.X, p.Y);
@@ -121,6 +140,9 @@ namespace MonoDevelop.Ide
 		
 		public static void SendPendingFeedback ()
 		{
+			if (string.IsNullOrEmpty (feedbackUrl.Value))
+				return;
+
 			lock (sendingLock) {
 				if (sending)
 					return;
@@ -152,7 +174,7 @@ namespace MonoDevelop.Ide
 		{
 			string email = feedbackElem.GetAttribute ("email");
 			string body = feedbackElem.InnerText;
-			var request = (HttpWebRequest) HttpWebRequest.Create ("http://software.xamarin.com/Service/Feedback?m=" + email);
+			var request = (HttpWebRequest) HttpWebRequest.Create (feedbackUrl.Value + "?m=" + email);
 			request.Method = "POST";
 			request.BeginGetRequestStream (delegate (IAsyncResult res) {
 				HandleGetRequestStream (res, request, body);
