@@ -35,8 +35,6 @@ using System.IO;
 
 using MonoDevelop.Projects;
 using MonoDevelop.Projects.Text;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.Dom.Parser;
 using MonoDevelop.Components;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
@@ -50,6 +48,8 @@ using MonoDevelop.Core.Assemblies;
 using MonoDevelop.Core.Instrumentation;
 using Mono.TextEditor;
 using System.Diagnostics;
+using ICSharpCode.NRefactory.TypeSystem;
+using ICSharpCode.NRefactory.Documentation;
 
 namespace MonoDevelop.Ide
 {
@@ -213,115 +213,89 @@ namespace MonoDevelop.Ide
 			return (GetDeclaredFile(item) != null);
 		}*/
 		
-		public bool CanJumpToDeclaration (MonoDevelop.Projects.Dom.INode visitable)
+		public bool CanJumpToDeclaration (INamedElement element)
 		{
-			if (visitable is MonoDevelop.Projects.Dom.IType) 
-				return ((MonoDevelop.Projects.Dom.IType)visitable).CompilationUnit != null;
-			if (visitable is LocalVariable)
-				return true;
-			if (visitable is IParameter)
-				return true;
-			IMember member = visitable as MonoDevelop.Projects.Dom.IMember;
-			if (member == null || member.DeclaringType == null) 
-				return false ;
-			return member.DeclaringType.CompilationUnit != null;
-		}
-		
-		public void JumpToDeclaration (MonoDevelop.Projects.Dom.INode visitable, bool askIfMultipleLocations)
-		{
-			if (askIfMultipleLocations) {
-				var type = visitable as IType;
-				if (type != null && type.HasParts) {
-					using (var monitor = IdeApp.Workbench.ProgressMonitors.GetSearchProgressMonitor (true, true)) {
-						foreach (IType part in DomType.GetSortedParts (type))
-							monitor.ReportResult (GetJumpTypePartSearchResult (part));
-					}
-					return;
-				}
-			}
+			var entity = element as IEntity;
+			if (entity == null && element is IType)
+				entity = ((IType)element).GetDefinition ();
+			if (entity == null)
+				return false;
 			
-			JumpToDeclaration (visitable);
+			if (entity.Region.IsEmpty) {
+				return !string.IsNullOrEmpty (entity.ParentAssembly.UnresolvedAssembly.Location);
+			}
+			return true;
 		}
 		
-		static MonoDevelop.Ide.FindInFiles.SearchResult GetJumpTypePartSearchResult (IType part)
+		
+		static MonoDevelop.Ide.FindInFiles.SearchResult GetJumpTypePartSearchResult (IUnresolvedTypeDefinition part)
 		{
-			var provider = new MonoDevelop.Ide.FindInFiles.FileProvider (part.CompilationUnit.FileName);
-			var doc = new Mono.TextEditor.Document ();
+			var provider = new MonoDevelop.Ide.FindInFiles.FileProvider (part.Region.FileName);
+			var doc = new Mono.TextEditor.TextDocument ();
 			doc.Text = provider.ReadString ();
-			int position = doc.LocationToOffset (part.Location.Line, part.Location.Column);
-			while (position + part.Name.Length < doc.Length) {
+			int position = doc.LocationToOffset (part.Region.BeginLine, part.Region.BeginColumn);
+			while (position + part.Name.Length < doc.TextLength) {
 				if (doc.GetTextAt (position, part.Name.Length) == part.Name)
 					break;
 				position++;
 			}
 			return new MonoDevelop.Ide.FindInFiles.SearchResult (provider, position, part.Name.Length);
 		}
-		
-		public void JumpToDeclaration (MonoDevelop.Projects.Dom.INode visitable)
+
+		public void JumpToDeclaration (INamedElement visitable, bool askIfMultipleLocations = true)
 		{
-			if (visitable is LocalVariable) {
-				LocalVariable localVar = (LocalVariable)visitable;
-				IdeApp.Workbench.OpenDocument (localVar.FileName,
-				                               localVar.Region.Start.Line,
-				                               localVar.Region.Start.Column);
-				return;
-			}
-			
-			if (visitable is IParameter) {
-				IParameter para = (IParameter)visitable;
-				IdeApp.Workbench.OpenDocument (para.DeclaringMember.DeclaringType.CompilationUnit.FileName,
-				                               para.Location.Line,
-				                               para.Location.Column);
-				return;
-			}
-			
-			IMember member = visitable as MonoDevelop.Projects.Dom.IMember;
-			if (member == null) 
-				return;
-			string fileName;
-			if (member is MonoDevelop.Projects.Dom.IType) {
-				try {
-					fileName = ((MonoDevelop.Projects.Dom.IType)member).CompilationUnit.FileName;
-				} catch (Exception e) {
-					LoggingService.LogError ("Can't get file name for type:" + member + ". Try to restart monodevelop.", e);
-					fileName = null;
-				}
-			} else {
-				if (member.DeclaringType == null) 
+			if (askIfMultipleLocations) {
+				var type = visitable as IType;
+				if (type != null && type.GetDefinition () != null && type.GetDefinition ().Parts.Count > 1) {
+					using (var monitor = IdeApp.Workbench.ProgressMonitors.GetSearchProgressMonitor (true, true)) {
+						foreach (var part in type.GetDefinition ().Parts)
+							monitor.ReportResult (GetJumpTypePartSearchResult (part));
+					}
 					return;
-				IType declaringType = SearchContainingPart (member);
-				fileName = declaringType.CompilationUnit.FileName;
+				}
 			}
-			var doc = IdeApp.Workbench.OpenDocument (fileName, member.Location.Line, member.Location.Column);
-			if (doc != null) {
+
+			JumpToDeclaration (visitable);
+		}
+
+		void JumpToDeclaration (INamedElement element)
+		{
+			IEntity entity = element as IEntity;
+
+			if (entity == null && element is IType)
+				entity = ((IType)element).GetDefinition ();
+			if (entity == null) {
+				LoggingService.LogError ("Unknown element:" + element);
+				return;
+			}
+			string fileName;
+			bool isCecilProjectContent = entity.Region.IsEmpty;
+			if (isCecilProjectContent) {
+				fileName = entity.ParentAssembly.UnresolvedAssembly.Location;
+			} else {
+				fileName = entity.Region.FileName;
+			}
+			var doc = IdeApp.Workbench.OpenDocument (fileName,
+			                               entity.Region.BeginLine,
+			                               entity.Region.BeginColumn);
+
+			if (isCecilProjectContent && doc != null) {
 				doc.RunWhenLoaded (delegate {
 					MonoDevelop.Ide.Gui.Content.IUrlHandler handler = doc.ActiveView as MonoDevelop.Ide.Gui.Content.IUrlHandler;
 					if (handler != null)
-						handler.Open (member.HelpUrl);
+						handler.Open (entity.GetIdString ());
 				});
 			}
 		}
 
-		static IType SearchContainingPart (IMember member)
+		public void JumpToDeclaration (IVariable entity)
 		{
-			IType declaringType = member.DeclaringType;
-			if (member is ExtensionMethod)
-				declaringType = ((ExtensionMethod)member).OriginalMethod.DeclaringType;
-			
-			if (declaringType is InstantiatedType)
-				declaringType = ((InstantiatedType)declaringType).UninstantiatedType;
-			if (declaringType.HasParts) {
-				foreach (IType part in declaringType.Parts) {
-					IMember searchedMember = part.SearchMember (member.Name, true).FirstOrDefault (m => m.Location == member.Location);
-					if (searchedMember != null) 
-						return part;
-				}
-			}
-			
-			return declaringType;
+			if (entity == null)
+				throw new ArgumentNullException ("entity");
+			string fileName = entity.Region.FileName;
+			var doc = IdeApp.Workbench.OpenDocument (fileName, entity.Region.BeginLine, entity.Region.BeginColumn);
 		}
 
-		
 		public void RenameItem (IWorkspaceFileObject item, string newName)
 		{
 			ProjectOptionsDialog.RenameItem (item, newName);
@@ -333,22 +307,19 @@ namespace MonoDevelop.Ide
 			}
 		}
 		
-		public void Export (IWorkspaceObject item)
+		public void Export (WorkspaceItem item)
 		{
 			Export (item, null);
 		}
 		
-		public void Export (IWorkspaceObject entry, FileFormat format)
+		public void Export (WorkspaceItem item, FileFormat format)
 		{
-			ExportProjectDialog dlg = new ExportProjectDialog (entry, format);
+			ExportSolutionDialog dlg = new ExportSolutionDialog (item, format);
+			
 			try {
 				if (MessageService.RunCustomDialog (dlg) == (int) Gtk.ResponseType.Ok) {
-					
-					using (IProgressMonitor mon = IdeApp.Workbench.ProgressMonitors.GetToolOutputProgressMonitor (true)) {
-						string folder = dlg.TargetFolder;
-						
-						string file = entry is WorkspaceItem ? ((WorkspaceItem)entry).FileName : ((SolutionEntityItem)entry).FileName;
-						Services.ProjectService.Export (mon, file, folder, dlg.Format);
+					using (IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetToolOutputProgressMonitor (true)) {
+						Services.ProjectService.Export (monitor, item.FileName, dlg.TargetFolder, dlg.Format);
 					}
 				}
 			} finally {
@@ -691,8 +662,12 @@ namespace MonoDevelop.Ide
 		{
 			string basePath = parentFolder != null ? parentFolder.BaseDirectory : null;
 			NewProjectDialog npdlg = new NewProjectDialog (parentFolder, false, basePath);
-			if (MessageService.ShowCustomDialog (npdlg) == (int)Gtk.ResponseType.Ok)
-				return npdlg.NewItem as SolutionItem;
+			if (MessageService.ShowCustomDialog (npdlg) == (int)Gtk.ResponseType.Ok) {
+				var item = npdlg.NewItem as SolutionItem;
+				if ((item is Project) && ProjectCreated != null)
+					ProjectCreated (this, new ProjectCreatedEventArgs (item as Project));
+				return item;
+			}
 			return null;
 		}
 
@@ -831,6 +806,8 @@ namespace MonoDevelop.Ide
 		
 		void RemoveItemFromSolution (SolutionItem prj)
 		{
+			foreach (var doc in IdeApp.Workbench.Documents.Where (d => d.Project == prj).ToArray ())
+				doc.Close ();
 			Solution sol = prj.ParentSolution;
 			prj.ParentFolder.Items.Remove (prj);
 			prj.Dispose ();
@@ -930,14 +907,8 @@ namespace MonoDevelop.Ide
 			ITimeTracker tt = Counters.BuildItemTimer.BeginTiming ("Cleaning " + entry.Name);
 			try {
 				IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetCleanProgressMonitor ();
-				
-				tt.Trace ("Start clean event");
-				TaskService.Errors.ClearByOwner (this);
-				OnStartClean (monitor);
-				
-				DispatchService.ThreadDispatch (delegate {
-					CleanAsync (entry, monitor, tt);
-				}, null);
+				OnStartClean (monitor, tt);
+				DispatchService.ThreadDispatch (() => CleanAsync (entry, monitor, tt, false));
 				
 				currentBuildOperation = monitor.AsyncOperation;
 				currentBuildOperationOwner = entry;
@@ -953,7 +924,7 @@ namespace MonoDevelop.Ide
 			return currentBuildOperation;
 		}
 		
-		void CleanAsync (IBuildTarget entry, IProgressMonitor monitor, ITimeTracker tt)
+		void CleanAsync (IBuildTarget entry, IProgressMonitor monitor, ITimeTracker tt, bool isRebuilding)
 		{
 			try {
 				tt.Trace ("Cleaning item");
@@ -969,10 +940,14 @@ namespace MonoDevelop.Ide
 			} finally {
 				tt.Trace ("Done cleaning");
 			}
-			DispatchService.GuiDispatch (
-				delegate {
-					CleanDone (monitor, entry, tt);
-			});
+			
+			if (isRebuilding) {
+				if (EndClean != null) {
+					DispatchService.GuiSyncDispatch (() => OnEndClean (monitor, tt));
+				}
+			} else {
+				DispatchService.GuiDispatch (() => CleanDone (monitor, entry, tt));
+			}
 		}
 		
 		void CleanDone (IProgressMonitor monitor, IBuildTarget entry, ITimeTracker tt)
@@ -983,8 +958,7 @@ namespace MonoDevelop.Ide
 				monitor.Log.WriteLine (GettextCatalog.GetString ("---------------------- Done ----------------------"));
 				tt.Trace ("Reporting result");			
 				monitor.ReportSuccess (GettextCatalog.GetString ("Clean successful."));
-				tt.Trace ("End clean event");
-				OnEndClean (monitor);
+				OnEndClean (monitor, tt);
 			} finally {
 				monitor.Dispose ();
 				tt.End ();
@@ -1063,18 +1037,33 @@ namespace MonoDevelop.Ide
 		{
 			if (currentBuildOperation != null && !currentBuildOperation.IsCompleted) return currentBuildOperation;
 			
-			var asyncOperation = new AsyncOperation ();
-			IAsyncOperation cleanOperation = Clean (entry);
+			ITimeTracker tt = Counters.BuildItemTimer.BeginTiming ("Rebuilding " + entry.Name);
+			try {
+				IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetRebuildProgressMonitor ();
+				OnStartClean (monitor, tt);
 
-			asyncOperation.TrackOperation (cleanOperation, false);
-			
-			cleanOperation.Completed += (aop) => {
-				if (aop.Success)
-					asyncOperation.TrackOperation (Build (entry), true);
-			};
-			
-			return asyncOperation;
+				DispatchService.ThreadDispatch (delegate {
+					CleanAsync (entry, monitor, tt, true);
+					if (monitor.AsyncOperation.IsCompleted && !monitor.AsyncOperation.Success) {
+						tt.End ();
+						monitor.Dispose ();
+						return;
+					}
+					if (StartBuild != null) {
+						DispatchService.GuiSyncDispatch (() => BeginBuild (monitor, tt, true));
+					}
+					BuildSolutionItemAsync (entry, monitor, tt);
+				}, null);
+				currentBuildOperation = monitor.AsyncOperation;
+				currentBuildOperationOwner = entry;
+				currentBuildOperation.Completed += delegate { currentBuildOperationOwner = null; };
+			} catch {
+				tt.End ();
+				throw;
+			}
+			return currentBuildOperation;
 		}
+		
 //		bool errorPadInitialized = false;
 		public IAsyncOperation Build (IBuildTarget entry)
 		{
@@ -1100,13 +1089,8 @@ namespace MonoDevelop.Ide
 			ITimeTracker tt = Counters.BuildItemTimer.BeginTiming ("Building " + entry.Name);
 			try {
 				IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetBuildProgressMonitor ();
-			
-				tt.Trace ("Start build event");
-				BeginBuild (monitor);
-
-				DispatchService.ThreadDispatch (delegate {
-					BuildSolutionItemAsync (entry, monitor, tt);
-				}, null);
+				BeginBuild (monitor, tt, false);
+				DispatchService.ThreadDispatch (() => BuildSolutionItemAsync (entry, monitor, tt));
 				currentBuildOperation = monitor.AsyncOperation;
 				currentBuildOperationOwner = entry;
 				currentBuildOperation.Completed += delegate { currentBuildOperationOwner = null; };
@@ -1184,19 +1168,23 @@ namespace MonoDevelop.Ide
 			
 			switch (action) {
 			case BeforeCompileAction.Nothing: break;
-			case BeforeCompileAction.PromptForSave: DispatchService.GuiDispatch (delegate { PromptForSave (result); }); break;
-			case BeforeCompileAction.SaveAllFiles: DispatchService.GuiDispatch (delegate { SaveAllFiles (result); }); break;
+			case BeforeCompileAction.PromptForSave: DispatchService.GuiSyncDispatch (delegate { PromptForSave (result); }); break;
+			case BeforeCompileAction.SaveAllFiles: DispatchService.GuiSyncDispatch (delegate { SaveAllFiles (result); }); break;
 			default: System.Diagnostics.Debug.Assert (false); break;
 			}
 			
 			return result;
 		}
 
-		void BeginBuild (IProgressMonitor monitor)
+		void BeginBuild (IProgressMonitor monitor, ITimeTracker tt, bool isRebuilding)
 		{
-			TaskService.Errors.ClearByOwner (this);
-			if (StartBuild != null)
+			tt.Trace ("Start build event");
+			if (!isRebuilding) {
+				TaskService.Errors.ClearByOwner (this);
+			}
+			if (StartBuild != null) {
 				StartBuild (this, new BuildEventArgs (monitor, true));
+			}
 		}
 		
 		void BuildDone (IProgressMonitor monitor, BuildResult result, IBuildTarget entry, ITimeTracker tt)
@@ -1235,7 +1223,7 @@ namespace MonoDevelop.Ide
 						monitor.ReportError(GettextCatalog.GetString("Build failed."), null);
 					}
 					tt.Trace ("End build event");
-					OnEndBuild (monitor, lastResult.FailedBuildCount == 0);
+					OnEndBuild (monitor, lastResult.FailedBuildCount == 0, lastResult, entry as SolutionItem);
 				} else {
 					tt.Trace ("End build event");
 					OnEndBuild (monitor, false);
@@ -1404,11 +1392,13 @@ namespace MonoDevelop.Ide
 			//and add the ProjectFiles directly. With large project and many files, this should really help perf.
 			//Also, this is a better check because we handle vpaths and links.
 			//FIXME: it would be really nice if project.Files maintained these hashmaps
-			var vpathsInProject = new HashSet<FilePath> (project.Files.Select (pf => pf.ProjectVirtualPath));
+			var vpathsInProject = new Dictionary<FilePath, ProjectFile> ();
 			var filesInProject = new Dictionary<FilePath,ProjectFile> ();
-			foreach (var pf in project.Files)
+			foreach (var pf in project.Files) {
 				filesInProject [pf.FilePath] = pf;
-			
+				vpathsInProject [pf.ProjectVirtualPath] = pf;
+			}
+
 			using (monitor)
 			{
 				for (int i = 0; i < files.Length; i++) {
@@ -1427,11 +1417,13 @@ namespace MonoDevelop.Ide
 					
 					FilePath targetPath = targetPaths[i].CanonicalPath;
 					Debug.Assert (targetPath.IsChildPathOf (project.BaseDirectory));
-					
+
+					ProjectFile vfile;
 					var vpath = targetPath.ToRelative (project.BaseDirectory);
-					if (vpathsInProject.Contains (vpath)) {
-						MessageService.ShowWarning (GettextCatalog.GetString (
-							"There is a already a file or link in the project with the name '{0}'", vpath));
+					if (vpathsInProject.TryGetValue (vpath, out vfile)) {
+						if (vfile.FilePath != file)
+							MessageService.ShowWarning (GettextCatalog.GetString (
+								"There is a already a file or link in the project with the name '{0}'", vpath));
 						continue;
 					}
 					
@@ -1484,7 +1476,7 @@ namespace MonoDevelop.Ide
 							ProjectFile pf = new ProjectFile (file, fileBuildAction) {
 								Link = vpath
 							};
-							vpathsInProject.Add (pf.ProjectVirtualPath);
+							vpathsInProject [pf.ProjectVirtualPath] = pf;
 							filesInProject [pf.FilePath] = pf;
 							newFileList.Add (pf);
 							continue;
@@ -1496,7 +1488,7 @@ namespace MonoDevelop.Ide
 							
 							if (MoveCopyFile (file, targetPath, action == AddAction.Move)) {
 								var pf = new ProjectFile (targetPath, fileBuildAction);
-								vpathsInProject.Add (pf.ProjectVirtualPath);
+								vpathsInProject [pf.ProjectVirtualPath] = pf;
 								filesInProject [pf.FilePath] = pf;
 								newFileList.Add (pf);
 							}
@@ -1519,7 +1511,7 @@ namespace MonoDevelop.Ide
 			return newFileList;
 		}
 		
-		void AddFileToFolder (List<ProjectFile> newFileList, HashSet<FilePath> vpathsInProject, Dictionary<FilePath, ProjectFile> filesInProject, FilePath file, string fileBuildAction)
+		void AddFileToFolder (List<ProjectFile> newFileList, Dictionary<FilePath, ProjectFile> vpathsInProject, Dictionary<FilePath, ProjectFile> filesInProject, FilePath file, string fileBuildAction)
 		{
 			//FIXME: MD project system doesn't cope with duplicate includes - project save/load will remove the file
 			ProjectFile pf;
@@ -1530,7 +1522,7 @@ namespace MonoDevelop.Ide
 				return;
 			}
 			pf = new ProjectFile (file, fileBuildAction);
-			vpathsInProject.Add (pf.ProjectVirtualPath);
+			vpathsInProject [pf.ProjectVirtualPath] = pf;
 			filesInProject [pf.FilePath] = pf;
 			newFileList.Add (pf);
 		}
@@ -1590,10 +1582,12 @@ namespace MonoDevelop.Ide
 						// Grab all the child nodes of the folder we just dragged/dropped
 						filesToRemove = sourceProject.Files.GetFilesInVirtualPath (virtualPath).ToList ();
 						// Add the folder itself so we can remove it from the soruce project if its a Move operation
-						filesToRemove.Add (sourceProject.Files.Where (f => f.ProjectVirtualPath == virtualPath).FirstOrDefault ());
+						var folder = sourceProject.Files.Where (f => f.ProjectVirtualPath == virtualPath).FirstOrDefault ();
+						if (folder != null)
+							filesToRemove.Add (folder);
 					} else {
 						filesToRemove = new List<ProjectFile> ();
-						var pf = sourceProject.GetProjectFile (sourcePath);
+						var pf = sourceProject.Files.GetFileWithVirtualPath (sourceProject.GetRelativeChildPath (sourcePath));
 						if (pf != null)
 							filesToRemove.Add (pf);
 					}
@@ -1690,7 +1684,7 @@ namespace MonoDevelop.Ide
 				
 				if (sourceProject != null) {
 					if (fileIsLink) {
-						var linkFile = (sourceProject == targetProject)? file : (ProjectFile) file.Clone ();
+						var linkFile = (ProjectFile) file.Clone ();
 						if (movingFolder) {
 							var abs = linkFile.Link.ToAbsolute (sourceProject.BaseDirectory);
 							var relSrc = abs.ToRelative (sourcePath);
@@ -1813,22 +1807,35 @@ namespace MonoDevelop.Ide
 			}
 		}
 
-		void OnEndBuild (IProgressMonitor monitor, bool success)
+		void OnEndBuild (IProgressMonitor monitor, bool success, BuildResult result = null, SolutionItem item = null)
 		{
-			if (EndBuild != null) {
-				EndBuild (this, new BuildEventArgs (monitor, success));
+			if (EndBuild == null)
+				return;
+					
+			var args = new BuildEventArgs (monitor, success) {
+				SolutionItem = item
+			};
+			if (result != null) {
+				args.WarningCount = result.WarningCount;
+				args.ErrorCount = result.ErrorCount;
+				args.BuildCount = result.BuildCount;
+				args.FailedBuildCount = result.FailedBuildCount;
 			}
+			EndBuild (this, args);
 		}
 		
-		void OnStartClean (IProgressMonitor monitor)
+		void OnStartClean (IProgressMonitor monitor, ITimeTracker tt)
 		{
+			tt.Trace ("Start clean event");
+			TaskService.Errors.ClearByOwner (this);
 			if (StartClean != null) {
 				StartClean (this, new CleanEventArgs (monitor));
 			}
 		}
 		
-		void OnEndClean (IProgressMonitor monitor)
+		void OnEndClean (IProgressMonitor monitor, ITimeTracker tt)
 		{
+			tt.Trace ("End clean event");
 			if (EndClean != null) {
 				EndClean (this, new CleanEventArgs (monitor));
 			}
@@ -1884,6 +1891,7 @@ namespace MonoDevelop.Ide
 		
 		public event EventHandler<SolutionEventArgs> CurrentSelectedSolutionChanged;
 		public event ProjectEventHandler CurrentProjectChanged;
+		public event EventHandler<ProjectCreatedEventArgs> ProjectCreated;
 		
 		// Fired just before an entry is added to a combine
 		public event AddEntryEventHandler AddingEntryToCombine;
