@@ -30,28 +30,52 @@ using System;
 using System.Collections.Generic;
 using System.Linq;
 using NUnit.Framework;
+using Mono.TextEditor.Tests.Actions;
 
 namespace Mono.TextEditor.Tests
 {
 	[TestFixture()]
 	public class FoldingTests
 	{
-		[TestFixtureSetUp] 
-		public void SetUp()
+		public static TextEditorData Create (string content)
 		{
-			Gtk.Application.Init ();
+			int caretIndex = content.IndexOf ("$");
+			if (caretIndex >= 0)
+				content = content.Substring (0, caretIndex) + content.Substring (caretIndex + 1);
+
+			int selection1 = content.IndexOf ("<-");
+			int selection2 = content.IndexOf ("->");
+			
+			int selectionStart = 0;
+			int selectionEnd = 0;
+			if (0 <= selection1 && selection1 < selection2) {
+				content = content.Substring (0, selection2) + content.Substring (selection2 + 2);
+				content = content.Substring (0, selection1) + content.Substring (selection1 + 2);
+				selectionStart = selection1;
+				selectionEnd = selection2 - 2;
+				caretIndex = selectionEnd;
+			}
+
+			var data = new TextEditorData ();
+			data.Text = content;
+			if (caretIndex >= 0)
+				data.Caret.Offset = caretIndex;
+			if (selection1 >= 0) {
+				if (caretIndex == selectionStart) {
+					data.SetSelection (selectionEnd, selectionStart);
+				} else {
+					data.SetSelection (selectionStart, selectionEnd);
+				}
+			}
+			return data;
 		}
-		
-		[TestFixtureTearDown] 
-		public void Dispose()
-		{
-		}		
-		static List<FoldSegment> GetFoldSegments (Document doc)
+
+		static List<FoldSegment> GetFoldSegments (TextDocument doc)
 		{
 			List<FoldSegment> result = new List<FoldSegment> ();
 			Stack<FoldSegment> foldSegments = new Stack<FoldSegment> ();
 			
-			for (int i = 0; i < doc.Length - 1; ++i) {
+			for (int i = 0; i < doc.TextLength - 1; ++i) {
 				char ch = doc.GetCharAt (i);
 				
 				if ((ch == '+' || ch == '-') && doc.GetCharAt(i + 1) == '[') {
@@ -70,7 +94,7 @@ namespace Mono.TextEditor.Tests
 		[Test()]
 		public void TestLogicalToVisualLine ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"-[1
 +[2
 3
@@ -89,7 +113,7 @@ namespace Mono.TextEditor.Tests
 16
 ]17
 18");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			Assert.AreEqual (5, data.LogicalToVisualLine (12));
 			Assert.AreEqual (8, data.LogicalToVisualLine (16));
 			Assert.AreEqual (8, data.LogicalToVisualLine (17));
@@ -98,7 +122,7 @@ namespace Mono.TextEditor.Tests
 		[Test()]
 		public void TestLogicalToVisualLineStartLine ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"-[1
 -[2
 3
@@ -117,14 +141,14 @@ namespace Mono.TextEditor.Tests
 16
 ]17
 18");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			Assert.AreEqual (7, data.LogicalToVisualLine (7));
 		}
 		
 		[Test()]
 		public void TestVisualToLogicalLineStartLine ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"-[1
 -[2
 3
@@ -143,14 +167,14 @@ namespace Mono.TextEditor.Tests
 16
 ]17
 18");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			Assert.AreEqual (7, data.VisualToLogicalLine (7));
 		}
 		
 		[Test()]
 		public void TestVisualToLogicalLineCase2 ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"-[1
 +[2
 3
@@ -169,7 +193,7 @@ namespace Mono.TextEditor.Tests
 16
 ]17
 18");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			Assert.AreEqual (6, data.VisualToLogicalLine (4));
 			Assert.AreEqual (14, data.VisualToLogicalLine (6));
 			Assert.AreEqual (15, data.VisualToLogicalLine (7));
@@ -178,7 +202,7 @@ namespace Mono.TextEditor.Tests
 		[Test()]
 		public void TestVisualToLogicalLineCase3 ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"-[1
 +[2
 3
@@ -197,7 +221,7 @@ namespace Mono.TextEditor.Tests
 16
 ]17
 18");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			Assert.AreEqual (2, data.VisualToLogicalLine (2));
 			Assert.AreEqual (2, data.LogicalToVisualLine (2));
 		}
@@ -205,7 +229,7 @@ namespace Mono.TextEditor.Tests
 		[Test()]
 		public void TestUpdateFoldSegmentBug ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"-[0
 1
 +[2
@@ -234,18 +258,18 @@ namespace Mono.TextEditor.Tests
 25
 26");
 			var segments = GetFoldSegments (data.Document);
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
 			Assert.AreEqual (25, data.VisualToLogicalLine (9));
 			Assert.AreEqual (3, data.Document.FoldSegments.Count ());
 			segments.RemoveAt (1);
 			
 			
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
 			
 			Assert.AreEqual (2, data.Document.FoldSegments.Count ());
 			Assert.AreEqual (17, data.LogicalToVisualLine (25));
 			segments.RemoveAt (1);
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
 			Assert.AreEqual (1, data.Document.FoldSegments.Count ());
 			Assert.AreEqual (25, data.LogicalToVisualLine (25));
 		}
@@ -256,7 +280,7 @@ namespace Mono.TextEditor.Tests
 		[Test()]
 		public void TestBug682466 ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"0
 1
 2
@@ -269,18 +293,18 @@ namespace Mono.TextEditor.Tests
 9
 10");
 			var segments = GetFoldSegments (data.Document);
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
 			Assert.AreEqual (true, data.Document.FoldSegments.FirstOrDefault ().IsFolded);
 			segments = GetFoldSegments (data.Document);
 			segments[0].IsFolded = false;
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
 			Assert.AreEqual (5, data.LogicalToVisualLine (8));
 		}
 		
 		[Test()]
 		public void TestVisualToLogicalLine ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"-[0
 +[1
 2
@@ -299,7 +323,7 @@ namespace Mono.TextEditor.Tests
 15
 ]16
 17");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			Assert.AreEqual (13, data.VisualToLogicalLine (5));
 			Assert.AreEqual (18, data.VisualToLogicalLine (8));
 		}
@@ -309,13 +333,13 @@ namespace Mono.TextEditor.Tests
 		[Test()]
 		public void TestCaretRight ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"1234567890
 1234567890
 123$4+[567890
 1234]567890
 1234567890");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			CaretMoveActions.Right (data);
 			Assert.AreEqual (new DocumentLocation (3, 5), data.Caret.Location);
 			CaretMoveActions.Right (data);
@@ -325,13 +349,13 @@ namespace Mono.TextEditor.Tests
 		[Test()]
 		public void TestCaretLeft ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"1234567890
 1234567890
 1234+[567890
 1234]5$67890
 1234567890");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			CaretMoveActions.Left (data);
 			Assert.AreEqual (new DocumentLocation (4, 6), data.Caret.Location);
 			CaretMoveActions.Left (data);
@@ -341,13 +365,13 @@ namespace Mono.TextEditor.Tests
 		[Test()]
 		public void TestCaretLeftCase2 ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"1234567890
 1234567890
 1234+[567890
 1234567890]
 $1234567890");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			CaretMoveActions.Left (data);
 			Assert.AreEqual (new DocumentLocation (4, 12), data.Caret.Location);
 			CaretMoveActions.Left (data);
@@ -358,7 +382,7 @@ $1234567890");
 		[Test()]
 		public void TestUpdateFoldSegmentBug2 ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"-[1
 2
 +[3
@@ -376,7 +400,7 @@ $1234567890");
 15
 16");
 			var segments = GetFoldSegments (data.Document);
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
 			Assert.AreEqual (10, data.VisualToLogicalLine (8));
 			Assert.AreEqual (3, data.Document.FoldSegments.Count ());
 			int start = data.GetLine (2).Offset;
@@ -389,7 +413,7 @@ $1234567890");
 		[Test()]
 		public void TestGetStartFoldingsGetStartFoldings ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"+[1
 2
 3
@@ -407,9 +431,9 @@ $1234567890");
 15
 16");
 			var segments = GetFoldSegments (data.Document);
-			data.Document.UpdateFoldSegments (segments, false);
-			data.Document.UpdateFoldSegments (segments, false);
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
+			data.Document.UpdateFoldSegments (segments);
+			data.Document.UpdateFoldSegments (segments);
 			
 			Assert.AreEqual (1, data.Document.GetStartFoldings (1).Count ());
 			Assert.AreEqual (1, data.Document.GetStartFoldings (4).Count ());
@@ -421,7 +445,7 @@ $1234567890");
 		[Test()]
 		public void TestIsFoldedSetFolded ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"-[1
 2
 3
@@ -439,7 +463,7 @@ $1234567890");
 15
 16");
 			var segments = GetFoldSegments (data.Document);
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
 			Assert.AreEqual (15, data.LogicalToVisualLine (15));
 			data.Document.GetStartFoldings (6).First ().IsFolded = true;
 			data.Document.GetStartFoldings (4).First ().IsFolded = true;
@@ -449,7 +473,7 @@ $1234567890");
 		[Test()]
 		public void TestIsFoldedUnsetFolded ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"-[1
 2
 3
@@ -467,7 +491,7 @@ $1234567890");
 15
 16");
 			var segments = GetFoldSegments (data.Document);
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
 			Assert.AreEqual (11, data.LogicalToVisualLine (15));
 			data.Document.GetStartFoldings (6).First ().IsFolded = false;
 			data.Document.GetStartFoldings (4).First ().IsFolded = false;
@@ -477,7 +501,7 @@ $1234567890");
 		[Test()]
 		public void TestCaretDown ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"AAAAAAAA
 AAAAAAAA$
 AAAAAAAA+[BBBBBBB
@@ -487,7 +511,7 @@ AAAAAAAABBBBBBBBBB
 AAAAAAAABBBBBBBBBB]
 AAAAAAAA
 ");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			
 			Assert.AreEqual (new DocumentLocation (2, 9), data.Caret.Location);
 			CaretMoveActions.Down (data);
@@ -499,7 +523,7 @@ AAAAAAAA
 		[Test()]
 		public void TestCaretUp ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"AAAAAAAA
 AAAAAAAA
 AAAAAAAA+[BBBBBBB
@@ -509,7 +533,7 @@ AAAAAAAABBBBBBBBBB
 AAAAAAAABBBBBBBBBB]
 AAAAAAAA$
 ");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			
 			Assert.AreEqual (new DocumentLocation (8, 9), data.Caret.Location);
 			CaretMoveActions.Up (data);
@@ -520,7 +544,7 @@ AAAAAAAA$
 		[Test()]
 		public void TestCaretUpCase2 ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"AAAAAAAA
 AAAAAAAA
 AA+[AAAAAABBBBBBB
@@ -530,7 +554,7 @@ AAAAAAAABBBBBBBBBB
 AAAAAAAABBBBBBBBBB]
 AAAAAAAA$
 ");
-			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document), false);
+			data.Document.UpdateFoldSegments (GetFoldSegments (data.Document));
 			
 			Assert.AreEqual (new DocumentLocation (8, 9), data.Caret.Location);
 			CaretMoveActions.Up (data);
@@ -544,7 +568,7 @@ AAAAAAAA$
 		[Test()]
 		public void TestBug1134 ()
 		{
-			var data = CaretMoveActionTests.Create (
+			var data = Create (
 @"0
 1
 -[2
@@ -560,11 +584,11 @@ AAAAAAAA$
 			var segments = GetFoldSegments (data.Document);
 			var seg = segments[0];
 			segments.RemoveAt (0);
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
 			Assert.AreEqual (2, data.Document.FoldSegments.Count ());
 			
 			segments.Insert (0, seg);
-			data.Document.UpdateFoldSegments (segments, false);
+			data.Document.UpdateFoldSegments (segments);
 			Assert.AreEqual (3, data.Document.FoldSegments.Count ());
 			
 		}	
